package org.vcs.bazaar.eclipse;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFile;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFolder;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.utils.BzrStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.ProjectBranch;
import org.vcs.bazaar.eclipse.internal.core.model.local.BzrLocalFile;
import org.vcs.bazaar.eclipse.internal.core.model.local.BzrLocalFolder;

/**
 * This idea for a repository specific WorkspaceRoot has been taken form
 * Subclipse's SVNWorkspaceRoot. This class provides static methods for common
 * operations between a project and it's repository (i.e: converting IResources
 * into Bazaar Resources)
 * <p>
 * Instances of this class represent a local workspace root (i.e. a project).
 * <br>
 * Here is how you can get a BzrWorkspaceRoot from an IProject : <br>
 * <code>
 * BazaarTeamProvider prov = RepositoryProvider.getProvider(project); <br>
 * BzrWorkspaceRoot bzrProject = prov.getBzrWorkspaceRoot();
 * </code>
 * </p>
 */
public class BzrWorkspaceRoot {

	private IBzrLocalBranch branch;
	private IBazaarClient client;

	public BzrWorkspaceRoot(IProject project) {
		try {
			ensureMetaFolderIsTeamPrivate(project);
		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
		}
		this.branch = new ProjectBranch(project);
	}

	/**
	 * Returns a resource path to the given local location. Returns null if it
	 * is not under a project's location.
	 * 
	 * @see FileSystemResourceManager.pathForLocation(org.eclipse.core.runtime.IPath)
	 */
	public static IPath pathForLocation(IPath location) {
		if (Platform.getLocation().equals(location))
			return Path.ROOT;
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++) {
			IProject project = projects[i];
			IPath projectLocation = project.getLocation();
			if (projectLocation != null && projectLocation.isPrefixOf(location)) {
				int segmentsToRemove = projectLocation.segmentCount();
				return project.getFullPath().append(location.removeFirstSegments(segmentsToRemove));
			}
		}
		return null;
	}

	/**
	 * get the IBzrLocalFile for the given resource
	 * @param resource
	 * @return IBzrLocalFile
	 */
	public static IBzrLocalFile getBzrFileFor(IResource resource) {
		BzrWorkspaceRoot wkroot = getBzrWorkspaceRootFor(resource.getProject());
		if(wkroot == null)
			return null;
		IBzrBranch branch = wkroot.getBranch();
		return new BzrLocalFile(resource, branch);
	}

	/**
	 * get the IBzrLocalFolder for the given resource
	 * @param resource
	 * @return IBzrLocalFolder
	 */
	public static IBzrLocalFolder getBzrFolderFor(IContainer resource) {
		BzrWorkspaceRoot wkroot = getBzrWorkspaceRootFor(resource.getProject());
		if(wkroot == null)
			return null;
		IBzrBranch branch = wkroot.getBranch();
		return new BzrLocalFolder(resource, branch);
	}

	/**
	 * get the IBzrLocalResource for the given resource
	 * @param resource
	 * @return IBzrLocalResource
	 */
	public static IBzrLocalResource getBzrResourceFor(IResource resource) {
		if (resource.getType() == IResource.FILE) {
			return getBzrFileFor(resource);
		} else {
			return getBzrFolderFor((IContainer) resource);
		}
	}

	/**
	 * get the IBzrLocalResource's for the given resources
	 * 
	 * @param resources
	 * @return IBzrLocalResource[]
	 */
	public static IBzrLocalResource[] getBzrResourcesFor(IResource resources[]) {
		IBzrLocalResource[] bzrResources = new IBzrLocalResource[resources.length];
		for (int i = 0; i < resources.length; i++) {
			bzrResources[i] = getBzrResourceFor(resources[i]);
		}
		return bzrResources;
	}

	/**
	 * get the branch for this project
	 * @return IBzrBranch
	 */
	public IBzrBranch getBranch() {
		return this.branch;
	}

	public static BzrWorkspaceRoot getBzrWorkspaceRootFor(IProject project) {
		BazaarTeamProvider provider = getProviderFor(project);
		if(provider == null)
			return null;
		return provider.getBzrWorkspaceRoot();
	}

	public static BazaarTeamProvider getProviderFor(IProject project) {
		BazaarTeamProvider provider = (BazaarTeamProvider) RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID());
		return provider;
	}

	public IWorkspaceRoot getWorkspaceRoot() {
		return this.getBranch().getProject().getWorkspace().getRoot();
	}

	public IBazaarClient getClient() throws BazaarException {
		if (client == null) {
			client = getClient(this.branch);
		}
		return client;
	}
	
	public static IBazaarClient getClient(IBzrBranch branch) throws BazaarException {
		try {
			IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			if(branch.isLocal())
				client.setWorkDir(branch.getFile());
			return client;
		} catch (BazaarException e) {
			if(!EclipseBazaarCore.getDefault().isClientAvailable()) {
				throw new BazaarException(new BzrStatus(BzrStatus.ERROR, "Bazaar client not available." + System.getProperty("line.separator") + " Please check the bazaar and bzr-xmloutput plugin installation", e));
			}
			throw e;
		}
	}

	/**
	 * This check if the project contains a .bzr folder, if true set as teamPrivateMember  
	 * @param project
	 * @throws CoreException
	 */
	private void ensureMetaFolderIsTeamPrivate(IProject project) throws CoreException {
		if (project != null) {
			IResource metaFolder = project.findMember(new Path(IBzrBranch.DIR_META_FOLDER));
			if (metaFolder != null && metaFolder.exists())
				metaFolder.setTeamPrivateMember(true);
			metaFolder = project.findMember(new Path(IBzrBranch.DIR_META_FOLDER+".backup"));
			if (metaFolder != null && metaFolder.exists())
				metaFolder.setTeamPrivateMember(true);
		}
	}
	
	public IResource getResourceForRelativePath(String relativePath) {
		IPath branchPath = branch.getPath();
		IPath projectPath = branch.getProject().getLocation();
		IResource resource = null;  
		if(branchPath.segmentCount() == projectPath.segmentCount()) {
			resource = branch.getProject().findMember(new Path(relativePath));
		} else if(projectPath.segmentCount() > branchPath.segmentCount() ) {
			resource = branch.getProject().findMember(new Path(relativePath).removeFirstSegments(1));	
		}
		return resource;
	}

}
