package org.vcs.bazaar.eclipse;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.SafeRunner;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.BazaarClientPreferences;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarNotifyListener;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BazaarClientException.BazaarUncheckedException;
import org.vcs.bazaar.eclipse.core.BzrSaveParticipant;
import org.vcs.bazaar.eclipse.core.inventory.InventoryCacheManager;
import org.vcs.bazaar.eclipse.core.listeners.AbstractResourceChangeListener;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.FileModificationManager;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.listeners.PreferenceChangeListener;
import org.vcs.bazaar.eclipse.core.listeners.SyncFileChangeListener;
import org.vcs.bazaar.eclipse.core.log.LogCacheManager;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.repository.BranchLocationManager;
import org.vcs.bazaar.eclipse.core.status.StatusCacheManager;
import org.vcs.bazaar.eclipse.core.utils.TracingUtils;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * The main plugin class to be used in the desktop.
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 */
public class EclipseBazaarCore extends Plugin {

	public static final String ID = "org.vcs.bazaar.eclipse.core";
	
	// The shared instance.
	private static EclipseBazaarCore plugin;
	private static final List<IResourceStateListener> stateListeners = new ArrayList<IResourceStateListener>();
	
	private IBazaarNotifyListener consoleListener;
	private StatusCacheManager statusCacheManager;
	private InventoryCacheManager inventoryCacheManager;
	private LogCacheManager logCacheManager;
	private BazaarClientPreferences clientPreferences;
	private final PreferenceChangeListener preferenceListener = new PreferenceChangeListener();
	/** this listener will listen to modifications to files */
	private final AbstractResourceChangeListener fileModificationManager = new FileModificationManager();
	private final AbstractResourceChangeListener metaDirChangeListener = new SyncFileChangeListener();
	private boolean clientAvailable = false;

	/**
	 * The constructor.
	 */
	public EclipseBazaarCore() {
		plugin = this;
	}

	@Override
    public void start(BundleContext context) throws Exception {
		super.start(context);
		getPluginPreferences().addPropertyChangeListener(preferenceListener);
		inventoryCacheManager = new InventoryCacheManager();
		statusCacheManager = new StatusCacheManager();
		logCacheManager = new LogCacheManager();
		BranchLocationManager.getInstance().start();			
		ResourcesPlugin.getWorkspace().addResourceChangeListener(fileModificationManager, IResourceChangeEvent.POST_CHANGE);
		ResourcesPlugin.getWorkspace().addResourceChangeListener(metaDirChangeListener, IResourceChangeEvent.POST_CHANGE);
		// TODO: register as save participant
		// fileModificationManager.registerSaveParticipant();
		ResourcesPlugin.getWorkspace().addSaveParticipant(this, new BzrSaveParticipant());
		try {
			checkClient();
		} catch (BazaarException e) {
			log(e);
		}
	}

	/**
	 * Set bazaar java library preferences from Eclipse preferences.
	 */
	@SuppressWarnings(value = { "deprecation" })
	public void updateClientPreferences() {
		if (clientPreferences == null) {
            clientPreferences = BazaarClientPreferences.getInstance();
        }
		Preferences pluginPrefs = getPluginPreferences();
		String bzrExeName = pluginPrefs.getString(BazaarPreference.EXECUTABLE.name());
		if (bzrExeName != null && ! "".equals(bzrExeName)) {
		    // User has explicitly set a bazaar executable to use
		    clientPreferences.set(BazaarPreference.EXECUTABLE, bzrExeName);
		} else {
			// set the bzr executable we found in PATH
			pluginPrefs.setDefault(BazaarPreference.EXECUTABLE.name(), clientPreferences.getString(BazaarPreference.EXECUTABLE));
		}
		clientPreferences.set(BazaarPreference.BZR_EMAIL, pluginPrefs.getString(BazaarPreference.BZR_EMAIL.name()));
		clientPreferences.set(BazaarPreference.BZR_XMLRPC_PORT, pluginPrefs.getString(BazaarPreference.BZR_XMLRPC_PORT.name()));
		String pluginPath = pluginPrefs.getString(BazaarPreference.BZR_PLUGIN_PATH.name());
		Boolean useBundled = pluginPrefs.getBoolean(IBazaarCoreConstants.USE_BUNDLED_PLUGINS);
		if(useBundled) {
			// set our default (bundled) plugins first, if the user didn't changed the default
			pluginPath = this.getBundledPluginsPath()+":"+pluginPath;
		}
		if (pluginPath != null && pluginPath.length() > 0) {
			clientPreferences.set(BazaarPreference.BZR_PLUGIN_PATH, pluginPath);
			// update the plugin path, so the user can see it
			pluginPrefs.setDefault(BazaarPreference.BZR_PLUGIN_PATH.name(), pluginPath);
		} else {
			clientPreferences.unset(BazaarPreference.BZR_PLUGIN_PATH);
		}
	}
	
	private String getBundledPluginsPath() {
	    Bundle clientBundle = null;
	    for (Bundle bundle : this.getBundle().getBundleContext().getBundles()) {
	        if("org.vcs.bazaar.eclipse.client".equals(bundle.getSymbolicName().trim())) {
	            clientBundle = bundle;
	            break;
	        }
	    }
	    try {
	        URL entry = FileLocator.resolve(FileLocator.findEntries(clientBundle, new Path("plugins"))[0]);
			return new File(entry.getPath()).getPath();
	    } catch (IOException e) {
	        IStatus status = new Status(IStatus.ERROR, this.getClass().getPackage().getName(), e.getMessage(), e);
	        this.getLog().log(status);
	        return null;
	    }
	}

	@Override
    public void stop(BundleContext context) throws Exception {
		BranchLocationManager.getInstance().stop();
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(fileModificationManager);
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(metaDirChangeListener);
		getPluginPreferences().removePropertyChangeListener(preferenceListener);
		synchronized (stateListeners) {
			stateListeners.clear();
		}
		ResourcesPlugin.getWorkspace().removeSaveParticipant(this);
		super.stop(context);
		plugin = null;
	}

	/**
	 * @return plugin Returns the shared instance.
	 */
	public static EclipseBazaarCore getDefault() {
		return plugin;
	}

	public static final String getPluginId() {
		return ID;
	}

	public static String getProviderID() {
		return BazaarTeamProvider.ID;
	}

	/**
	 * Create and returns a new IBazaarClient instance.<br>
	 * If the client can't be created, a exception is thrown.
	 * 
	 * @return
	 * @throws BazaarException
	 */
	public IBazaarClient createClient() throws BazaarException {
		if(!clientAvailable) {
			checkClient();
		}
		IBazaarClient client = null;
		try {
			client = BazaarClientFactory.createClient(BazaarClientFactory.getPreferredClientType());
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} catch (BazaarUncheckedException e) {
			log(e.getMessage(), e);
			throw BazaarException.wrapException(e);
		}
		if (client != null) {
			// to prevent adding a null listener (this happen when the core
			// plugin is initialized before the ui plugin)
			if (getConsoleListener() != null) {
                client.addNotifyListener(getConsoleListener());
            }
		}
		return client;
	}

	public Boolean checkClient() throws BazaarException {
		updateClientPreferences();
		try {
			BazaarClientFactory.setupBestAvailableBackend(true);
			clientAvailable = true;
			return true;
		} catch (BazaarClientException e) {
			clientAvailable = false;
			throw BazaarException.wrapException(e);
		}
	}
	
	public String[] getAvailableClientTypes() {
		 return BazaarClientFactory.getClientTypes();
	}

	public boolean isClientAvailable() {
		return clientAvailable;
	}

	public void setConsoleListener(IBazaarNotifyListener consoleListener) {
		this.consoleListener = consoleListener;
	}

	public IBazaarNotifyListener getConsoleListener() {
		return consoleListener;
	}

	public static void addResourceStateListener(final IResourceStateListener listener) {
		synchronized (stateListeners) {
			if(!stateListeners.contains(listener)){
				stateListeners.add(listener);
			}
		}
	}

	public static void removeResourceStateListener(final IResourceStateListener listener) {
		synchronized (stateListeners) {
			stateListeners.remove(listener);
		}
	}

	/**
	 * get the resource status cache
	 * @return statusCacheManager the {@link StatusCacheManager} singleton 
	 */
	public StatusCacheManager getStatusCacheManager() {
		return statusCacheManager;
	}
	
	public InventoryCacheManager getInventoryCacheManager() {
		return inventoryCacheManager;
	}
	
	public LogCacheManager getLogCacheManager() {
		return logCacheManager;
	}

	public static void broadcastModificationStateChanges(final BzrChangeStateEvent event) {
		EclipseBazaarCore.trace("", "broadcastModificationStateChanges: ", event);
		final IResourceStateListener[] toNotify;
		synchronized (stateListeners) {
			toNotify = stateListeners.toArray(new IResourceStateListener[stateListeners.size()]);
		}

		for (int i = 0; i < toNotify.length; ++i) {
			final IResourceStateListener listener = toNotify[i];
			ISafeRunnable code = new ISafeRunnable() {
				public void run() throws Exception {
					listener.resourceStateChanged(event);
				}

				public void handleException(Throwable e) {
					// don't log the exception....it is already being logged in
					// Platform#run
				}
			};
			SafeRunner.run(code);
		}
	}
	
	
	/**
	 * Progress monitor helpers
	 * @param monitor
	 */
	public static void checkCanceled(IProgressMonitor monitor) {
		if (monitor != null && monitor.isCanceled()) {
            throw new OperationCanceledException();
        }
	}
	public static IProgressMonitor monitorFor(IProgressMonitor monitor) {
		if (monitor == null) {
            return new NullProgressMonitor();
        }
		return monitor;
	}
	
	/**
	 * creates a {@link SubProgressMonitor} with 1000 ticks
	 * @param monitor
	 * @return
	 */
	public static IProgressMonitor subProgressMonitorFor(IProgressMonitor monitor) {
		if (monitor == null) {
            return new NullProgressMonitor();
        }
		if (monitor instanceof NullProgressMonitor) {
            return monitor;
        }
		return new SubProgressMonitor(monitor, 1000);
	}
	
	// logging and tracing (taken from EclipseDarcsCore - @author The mighty PDE
	// wizard dskñsa)
	// ////////////////////

	public static void log(final String message, final int severity) {
		Status status = new Status(severity, getPluginId(), IStatus.OK, message, null);
		getDefault().getLog().log(status);
	}

	public static void log(final String message, final Throwable thr) {
		Status status = new Status(IStatus.ERROR, getPluginId(), IStatus.OK, message, thr);
		getDefault().getLog().log(status);
	}
	
	public static void log(CoreException ce) {
		getDefault().getLog().log(ce.getStatus());
	}

	public static boolean isTracing(final String optionId) {
		String option = getPluginId() + "/trace/" + optionId; //$NON-NLS-1$
		String value = Platform.getDebugOption(option);
		return "true".equals(value); //$NON-NLS-1$
	}

	public static void trace(final String optionId, final String what, Object...objects) {
		if (isTracing(optionId)) {
			TracingUtils.trace(what, objects);
		}
	}

	public static void traceException(final String optionId, final Exception ex) {
		if (isTracing(optionId)) {
			TracingUtils.trace("", ex);
		}
	}

}
