/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ILogOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResource;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class LogCommand extends BazaarCommand {

	private IBzrResource resource;
	private List<IBazaarLogMessage> logMessages;
	private Integer limit;

	/**
	 * limit the log size using the value of {@link IBazaarCoreConstants#LOG_REVISION_LIMIT} 
	 * in the plugin preferences.
	 *  
	 * @param bzrResource - the {@link IBzrResource} to fetch the log
	 */
	public LogCommand(IBzrResource bzrResource) {
		super();
		this.resource = bzrResource;
		final int logLimit = EclipseBazaarCore.getDefault().getPluginPreferences().getInt(IBazaarCoreConstants.LOG_REVISION_LIMIT);
		if(logLimit != 0) {
			limit = logLimit;
		}
	}

	/**
	 * 
	 * @param bzrResource - the {@link IBzrResource} to fetch the log
	 * @param limit - the number of revisions to fetch
	 */
	public LogCommand(IBzrResource bzrResource, Integer limit) {
		this.resource = bzrResource;
		this.limit = limit;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.vcs.bazaar.eclipse.core.commands.IBazaarCommand#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws BazaarException {
		try {
			if(resource == null)
				return;
			monitor.beginTask(CoreTexts.LogCommand_log, 1000);
			IBazaarClient client = getClient(resource.getBranch());
			client.getNotificationHandler().disableLog();
			List<Option> options = new ArrayList<Option>();
			if (limit != null) {
				options.add(ILogOptions.LIMIT.setArgument(limit.toString()));
			}
			
			if(resource instanceof IBzrBranch) {
				logMessages = client.log(((IBzrBranch)resource).getBranchLocation(), options.toArray(new Option[0]));
			} else if (resource instanceof IBzrLocalResource) {
				IBzrLocalResource localResource = ((IBzrLocalResource)resource);
				if(localResource.isProject()) {
					// if the resource is a project call log in the branch te get a full log and not only the project folder log
					logMessages = client.log(localResource.getBranch().getFile(), options.toArray(new Option[0]));
				} else {
					logMessages = client.log(localResource.getFile(), options.toArray(new Option[0]));
				}
			} else {
				logMessages = client.log(resource.getLocation(), options.toArray(new Option[0]));
			}
			client.getNotificationHandler().enableLog();
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

	public List<IBazaarLogMessage> getLogMessages() {
		if(logMessages == null) {
			logMessages = Collections.emptyList();
		}
		return logMessages;
	}
	
}
