/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.commandline.syntax.IResolveOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class ResolveCommand extends ResourceChangeCommand {

	private final IBzrLocalResource[] resources;
	private final IProject[] projects;
	private final boolean all;

	/**
	 * Resolve 
	 * @param branches
	 */
	public ResolveCommand(IProject[] projects, boolean all) {
		this.projects = projects;
		this.all = all;
		this.resources = null;
	}

	/**
	 * This constructor should to be used only on non-project resources (only files and folders)
	 * @param resources
	 */
	public ResolveCommand(Collection<IResource> resources) {
		this(BzrWorkspaceRoot.getBzrResourcesFor(resources.toArray(new IResource[0])));
	}
	
	/**
	 * @see #ResolveCommand(Collection)
	 * @param bzrResources
	 */
	public ResolveCommand(IBzrLocalResource[] bzrResources) {
		this.resources = bzrResources;
		this.projects = null;
		this.all = false;
	}

	public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			if(projects != null) {
				return resolveBranches(monitor);
			} else if(resources != null){
				return resolveResources(monitor);
			} else {
				return new IResource[0];
			}
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

	private IResource[] resolveBranches(IProgressMonitor monitor) throws BazaarException, BazaarClientException {
		Map<IBzrBranch, List<IProject>> map = getProjectsByBranch(projects);
		for (Entry<IBzrBranch, List<IProject>> entry : map.entrySet()) {
			IBzrBranch branch = entry.getKey();
			IBazaarClient client = getClient(branch);
			List<File> files = new ArrayList<File>(1);
			files.add(branch.getFile());
			if(all) {
				client.resolve(files, IResolveOptions.ALL);
			} else {
				client.resolve(files);
			}
				
		}
		return projects;
	}

	/**
	 * @param monitor
	 * @return {@link IResource}[] array of changed resources
	 * @throws BazaarException
	 * @throws BazaarClientException
	 */
	private IResource[] resolveResources(IProgressMonitor monitor) throws BazaarException, BazaarClientException {
		Map<IProject, List<File>> map = getFilesByProjectMap(resources);
		monitor.beginTask("Resolving", resources.length);
		for (IProject project : map.keySet()) {
			IBazaarClient client = getClient(project);
			client.resolve(map.get(project));
		}
		List<IResource> changed = new ArrayList<IResource>(resources.length);
		for (IResource resource : getIResources(resources)) {
			changed.add(resource);
			if(resource.getType() != IResource.PROJECT) {
				changed.add(resource.getParent());
			}
		}
		return changed.toArray(new IResource[changed.size()]);
	}

}
