/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.BazaarStatusKind;
import org.vcs.bazaar.client.BazaarTreeStatus;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.IBazaarStatus;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.BzrResourceStatus;

/**
 * @author Guillermo Gonzalez
 * 
 * TODO: extend from abstract BazaarCommand, and implement multi project
 * selection
 */
public class StatusCommand extends BazaarCommand {

	private final IProject project;
	private final IBzrLocalResource[] resources;
	private final IBzrLocalResource resource;
	private Map<IResource, IBzrResourceStatus> statusMap;
	private Set<BzrResourceStatus> deleted;
	private boolean noLog = false;
	private List<IBazaarLogMessage> pendingMerges;

	public StatusCommand(IProject project, IBzrLocalResource[] resources) {
		this(project, resources, null, false);
	}

	public StatusCommand(IProject project, IBzrLocalResource resource) {
		this(project, new IBzrLocalResource[0], resource, true);
	}

	public StatusCommand(IProject project, IBzrLocalResource[] resources, boolean noLog) {
		this(project, resources, null, noLog);
	}
	
	public StatusCommand(IProject project, IBzrLocalResource[] resources, IBzrLocalResource resource, boolean noLog) {
		super();
		this.project = project;
		this.resources = resources;
		this.resource = resource;
		this.noLog = noLog;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.vcs.bazaar.eclipse.core.commands.IBazaarCommand#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws BazaarException {
		// TODO Auto-generated method stub
		IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
		try {
			if ((resources == null || resources.length == 0) && project != null && project.isAccessible() && !subMonitor.isCanceled()) {
				IBazaarClient client = getClient(project);
				updateInventory(project, subMonitor);
				executeAndFilterLogOutput(client, new File[] { project.getLocation().toFile() });
			} else {
				Map<IProject, List<File>> map = getFilesByProjectMap(resources);
				for (IProject project : map.keySet()) {
					if(project.isAccessible() && !subMonitor.isCanceled()) {
						IBazaarClient client = getClient(project);;
						updateInventory(project, subMonitor);
						executeAndFilterLogOutput(client, getFiles());
					}
				}
			}
			if(statusMap == null)
				statusMap = new HashMap<IResource, IBzrResourceStatus>(0);

		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e, project);
		} catch (BazaarException e) {
			throw e;
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			subMonitor.done();
		}
	}

	public IBzrResourceStatus[] getStatus() {
		List<IBzrResourceStatus> list = new ArrayList<IBzrResourceStatus>(statusMap.values().size()+deleted.size());
		Collections.addAll(list, statusMap.values().toArray(new IBzrResourceStatus[0]));
		Collections.addAll(list, deleted.toArray(new IBzrResourceStatus[0]));
		return list.toArray(new IBzrResourceStatus[0]);
	}

	public List<IBazaarLogMessage> getPendingMerges() {
		return pendingMerges;
	}

	// //////////////////////////////////////////////////////////////////
	// Helpers
	private File[] getFiles() {
		File[] files = new File[resources.length];
		for (int i = 0; i < resources.length; i++) {
			files[i] = resources[i].getIResource().getLocation().toFile();
		}
		return files;
	}

	/**
	 * This method is a workaround to prevent a lot of output when is called
	 * from DecoratorStatus
	 * @param client {@link IBazaarClient}
	 * @param files {@link File}[]
	 * @throws BazaarClientException
	 * @throws CoreException
	 */
	private void executeAndFilterLogOutput(IBazaarClient client, File[] files) throws BazaarClientException, CoreException {
		if (noLog)
			client.getNotificationHandler().disableLog();
		BazaarTreeStatus treeStatus = client.status(files);
		pendingMerges = treeStatus.getPendingMerges();
		IBazaarStatus[] statuses = treeStatus.getStatusAsArray();
		client.getNotificationHandler().enableLog();
		// little hack to collect status per file and take care of folders
		// without a status
		// TODO: check if is this is needed. I think this is fixed in the parser
		statusMap = new HashMap<IResource, IBzrResourceStatus>(statuses.length);
		deleted = new HashSet<BzrResourceStatus>();
		for (IBazaarStatus aStatus : statuses) {
			IResource resource = findMember(project, aStatus);
			if (resource != null) {
				IBzrResourceStatus resourceStatus = new BzrResourceStatus(BzrWorkspaceRoot.getBzrResourceFor(resource), aStatus);
				statusMap.put(resource, resourceStatus);
			} else if(aStatus.contains(BazaarStatusKind.DELETED)) {
				resource = ResourceUtil.getIFile(project, aStatus);
				if(resource != null) {
					deleted.add(new BzrResourceStatus(BzrWorkspaceRoot.getBzrResourceFor(resource), aStatus));
				}
			}
		}
		if (resource != null) {
			IContainer container = null;
			if (resource.getIResource().getType() == IResource.PROJECT)
				container = project;
			else if (resource.getIResource().getType() == IResource.FILE)
				container = resource.getIResource().getParent();
			else if (resource.getIResource().getType() == IResource.FOLDER)
				container = (IContainer) resource.getIResource();
			createStatusesFor(container);
			if (!statusMap.containsKey(container))
				statusMap.put(container, getFolderStatus(container));
		}
	}

	private BzrWorkspaceRoot root;
	private IResource findMember(IProject project, IBazaarStatus status) {
		if(root == null)  {
			BazaarTeamProvider provider = (BazaarTeamProvider) RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID());
			root = provider.getBzrWorkspaceRoot();
		}
		return root.getResourceForRelativePath(status.getPath());
	}

	private void createStatusesFor(final IContainer resourceToUpdate) throws CoreException {
			// create the statuses for the child resources (only files)
			EclipseBazaarCore.trace(ITracingOptions.STATUS_COMMAND, "createStatusesFor#filesUpdates: " + resourceToUpdate.getName());
			resourceToUpdate.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					if (statusMap.containsKey(resource))
						return true;
					else if (resource.getType() == IResource.FILE) {
						IBzrResourceStatus status = null;
						try {
							if (isInInventory(resource)) 
								status = BzrResourceStatus.unchanged(resource);
							else // TODO: check for ignored and otherwise set it to unknown
								status = BzrResourceStatus.ignored(resource);
						} catch(Throwable thr) {
							EclipseBazaarCore.log(thr.getMessage(), thr);
						} 
						if (status != null) {
							statusMap.put(resource, status);
						} else {
							statusMap.put(resource, BzrResourceStatus.unknown(resource));
						}
					}
					return true;
				}
			}, IResource.DEPTH_INFINITE, IFolder.EXCLUDE_DERIVED);

			for (IBzrResourceStatus removed : deleted) {
				IContainer parent = removed.getResource().getIResource().getParent();				
				if(parent != null && parent.getType() != IResource.PROJECT) {
					statusMap.put(parent, BzrResourceStatus.modified(parent));	
				}
			}
			// now traverse the folder tree, and get the status of folders only
			EclipseBazaarCore.trace(ITracingOptions.STATUS_COMMAND, "createStatusesFor#foldersUpdates: " + resourceToUpdate.getName());
			resourceToUpdate.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					if ((resource.getType() == IResource.FOLDER || resource.getType() == IResource.PROJECT) && !statusMap.containsKey(resource)) {
						IBzrResourceStatus status = getFolderStatus((IContainer) resource);
						if (status != null) {
							statusMap.put(resource, status);
							return false;
						}
					}
					return true;
				}

			}, IResource.DEPTH_INFINITE, IFolder.EXCLUDE_DERIVED);
			// calculate the project status
			EclipseBazaarCore.trace(ITracingOptions.STATUS_COMMAND, "createStatusesFor#projectUpdate: " + resourceToUpdate.getName());
			IBzrResourceStatus status = getProjectStatus(resourceToUpdate.getProject());
			if (status != null) {
				statusMap.put(status.getResource().getIResource(), status);
			}
	}

	private IBzrResourceStatus getProjectStatus(IProject container) throws CoreException {
		for (IBzrResourceStatus status : deleted) {
			if(status != null && container.equals(status.getResource().getIResource().getProject())) {
				return BzrResourceStatus.modified(container);
			}
		}
		for (IResource member : container.members()) {
			IBzrResourceStatus status = statusMap.get(member);
			if (status != null && isModifiedStatus(status)) {
				return BzrResourceStatus.modified(container);
			}
		}
		if (!isInInventory(container)) {
			IPath projectPath = container.getLocation();
			if(resource.getBranch().getPath().matchingFirstSegments(projectPath) == projectPath.segmentCount())
				return BzrResourceStatus.unchanged(container);
			return BzrResourceStatus.ignored(container);
		} else if (isInInventory(container)) {
			return BzrResourceStatus.unchanged(container);
		}
		return BzrResourceStatus.unknown(container);
	}

	private IBzrResourceStatus getFolderStatus(IContainer container) throws CoreException {
		IBzrResourceStatus status = null;
		for (IResource member : container.members()) {
			if (member.getType() == IResource.FOLDER && !statusMap.containsKey(member)) {
				statusMap.put(member, getFolderStatus((IContainer) member));
			}
			IBzrResourceStatus memberStatus = statusMap.get(member);
			if (memberStatus != null && isModifiedStatus(memberStatus)) {
				status = BzrResourceStatus.modified(container);
			}
		}
		if(status != null)
			return status;
		if (!isInInventory(container)) {
			IPath projectPath = container.getLocation();
			if(resource.getBranch().getPath().matchingFirstSegments(projectPath) == projectPath.segmentCount())
				return BzrResourceStatus.unchanged(container);
			return BzrResourceStatus.ignored(container);
		} else if (isInInventory(container)) {
			return BzrResourceStatus.unchanged(container);
		}
		return BzrResourceStatus.unknown(container);
	}

	private boolean isModifiedStatus(IBzrResourceStatus status) {
		if (status.isHasConflicts() || status.isKindChange() || status.isModified() || status.isRenamed() || status.isUnknown() ||
				status.isXBitChanged() || status.isAdded() || status.isDeleted()) {
			return true;
		}
		return false;
	}
	
	private boolean isInInventory(IResource resource) throws CoreException {
		if(resource.isLinked(IResource.CHECK_ANCESTORS) 
				|| resource.isDerived() 
				|| !resource.isAccessible())
			return false;
		return EclipseBazaarCore.getDefault().getInventoryCacheManager().isInInventory(resource);
	}

	private void updateInventory(IProject project, IProgressMonitor monitor) throws BazaarException {
		EclipseBazaarCore.getDefault().getInventoryCacheManager().refreshCache(project, monitor);
	}
}
