/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.inventory;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.jobs.ILock;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.AbstractCacheManager;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * Provides a method to get if a resource is in the inventory. <br>
 * It is much more efficient to get this of a set a resources than only
 * one resource at a time. For that we use a
 * 
 * @link InventoryUpdater<br>
 * 
 * @author Guillermo Gonzalez
 * 
 * @basedOn StatusCacheManager - cedric chabanois (cchab at tigris.org)
 * 
 */
public class InventoryCacheManager extends AbstractCacheManager {
	
	private final static int INTERESTING_CHANGES = BzrChangeStateEvent.WT_STATE_MODIFY
			| BzrChangeStateEvent.RESOURCE_MODIFY;

	/**
	 * Name used for identifying Bazaar synchronization data in
	 * Resource>ResourceInfo#syncInfo storage
	 */
	public static final QualifiedName BZR_SYNC_KEY = new QualifiedName(EclipseBazaarCore.getPluginId(), "bzr-inventory-sync-key");

	private IInventoryCache inventoryCache;
	private InventoryUpdater inventoryUpdater;

	public InventoryCacheManager() {
		ResourcesPlugin.getWorkspace().getSynchronizer().add(InventoryCacheManager.BZR_SYNC_KEY);
		inventoryCache = new NaiveCache();
		inventoryUpdater = new InventoryUpdater();
	}
	
	/**
	 * update the cache using the given Resources
	 * 
	 * @param IResource[] resources
	 * @param rule
	 *            the scheduling rule to use when running this operation
	 */
	protected List<IResource> updateCache(final List<IBzrLocalResource> resources, ISchedulingRule rule) throws CoreException {
		final List<IResource> result = new ArrayList<IResource>(resources.size());
		if (ResourcesPlugin.getWorkspace().isTreeLocked()) {
			for (IBzrLocalResource bzrResource : resources) {
				result.add(updateCache(bzrResource));
			}
		} else {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					for (IBzrLocalResource bzrResource : resources) {
						result.add(updateCache(bzrResource));
					}
				}
			}, rule, IWorkspace.AVOID_UPDATE, null);
		}
		return result;
	}

	/**
	 * update the cache using the given resource
	 * 
	 * @param resource
	 */
	protected IResource updateCache(IBzrLocalResource resource) {
		if(resource.getIResource().getType() == IResource.ROOT) {
			return resource.getIResource();
		}
		synchronized (BZR_SYNC_KEY) {
			inventoryCache.remove(resource.getIResource());
			return inventoryCache.add(resource.getIResource());	
		}
	}
	
	/**
	 * Returns if the given resource is in the Inventory. If is not present in
	 * cache, it will be retrieved using the actual updateStrategy. 
	 * 
	 * @param resource
	 *            whose status is required.
	 * @throws CoreException
	 */
	public Boolean isInInventory(IResource resource) throws CoreException {
		return isInInventory(resource, inventoryUpdater);	
	}

	/**
	 * Returns if the given resource is in the Inventory. If is not present in
	 * cache, it will be retrieved using the actual updateStrategy. 
	 * 
	 * @throws CoreException
	 */
	private Boolean isInInventory(IResource resource, InventoryUpdater strategy) throws CoreException {
		if (!resource.exists() && !resource.isPhantom()) {
			return null;
		}
		Boolean status = Boolean.FALSE;
		IProject project = resource.getProject();
		if(resource.getType() == IResource.PROJECT) {
			project = (IProject)resource;
		}
		ILock lock = getLock(project);
		boolean acquired = false;
		try {
			acquired = lock.acquire(3000L);
			status = inventoryCache.isInInventory(resource);
			if (status == null || (!status && inventoryCache.needRefresh(resource))) {
				status = basicGetInventory(resource, strategy);
			}
		} catch(InterruptedException ie) { 
			EclipseBazaarCore.log(ie.getMessage(), ie);
		} finally {
			if(acquired) {
				lock.release();
			}
		}
		return status;
	}

	/**
	 * refresh the cache and returns if the resource is in it.
	 * 
	 * @param resource
	 * @param strategy
	 * @return
	 * @throws CoreException
	 */
	private Boolean basicGetInventory(IResource resource, InventoryUpdater strategy) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.INVENTORY_CACHE, "LogCacheManager.basicGetInventory: " + resource.getName());
		if (!resource.isAccessible())
			return null;
		// update the cache
		updateInventory(strategy.resourceToUpdate(resource, new NullProgressMonitor()), resource.getType() == IResource.FILE?resource.getParent():resource);
		return inventoryCache.isInInventory(resource);
	}

	/**
	 * The cache manager handles itself the inventory retrieving. However this
	 * method can be used to update the cache for specific resources
	 * 
	 * @param resources
	 * @param rule
	 *            the scheduling rule to use when running this operation
	 */
	private void updateInventory(List<IBzrLocalResource> resources, IResource rule) throws BazaarException {
		EclipseBazaarCore.trace(ITracingOptions.INVENTORY_CACHE, "LogCacheManager.updateInventory: " + rule.getName());
		try {
			if(resources == null || resources.size() == 0) {
				return;
			}
			// we don't need a rule here, we only require read-access to resources
			updateCache(resources, null);
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		}
	}

	/**
	 * Refresh the status of the cache using the given resource to the give depth. The depth can
	 * be deeper in case of phantom resources. These have to be traversed to
	 * infinite always ...
	 * 
	 * @param resource
	 * @param recursive
	 * @return array of resources which were refreshed (including all phantoms
	 *         and their children)
	 * @throws BazaarException
	 */
	public IResource[] refreshCache(final IContainer resource, IProgressMonitor monitor) throws BazaarException {
		IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
		subMonitor.subTask("Refreshing inventory cache");
		if (!shouldHandle(resource)) {
			return new IResource[0];
		}
		try {
			List<IResource> refreshedResources = updateCache(inventoryUpdater.resourceToUpdate(resource, monitor), resource);
			Set<IResource> resourcesToRefresh = resourcesToRefresh(resource, IContainer.INCLUDE_PHANTOMS, refreshedResources.size());
			for (Iterator<IResource> iter = refreshedResources.iterator(); iter.hasNext();) {
				resourcesToRefresh.remove(iter.next());
			}
			subMonitor.worked(resourcesToRefresh.size());
			// Resources which were not refreshed above (e.g. deleted resources)
			// We do it with depth = infinite, so the whole deleted trees are
			// refreshed.
			for (Iterator<IResource> it = resourcesToRefresh.iterator(); it.hasNext();) {
				IResource res = (IResource) it.next();
				if ((res.getType() != IResource.FILE) && res.isPhantom()) {
					Set<IResource> children = resourcesToRefresh(res, IContainer.INCLUDE_PHANTOMS | IContainer.INCLUDE_TEAM_PRIVATE_MEMBERS, 0);
					for (Iterator<IResource> iter = children.iterator(); iter.hasNext();) {
						IResource child = (IResource) iter.next();
						inventoryCache.remove(child);
						refreshedResources.add(child);
					}
				}
				inventoryCache.remove(res);
				refreshedResources.add(res);
			}
			return (IResource[]) refreshedResources.toArray(new IResource[refreshedResources.size()]);
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			subMonitor.done();
		}
	}

	private Set<IResource> resourcesToRefresh(IResource resource, int flags, int expectedSize) throws CoreException {
		if (!resource.exists() && !resource.isPhantom()) {
			return new HashSet<IResource>(0);
		}
		final Set<IResource> resultSet = (expectedSize != 0) ? new HashSet<IResource>(expectedSize) : new HashSet<IResource>();
		resource.accept(new IResourceVisitor() {
			public boolean visit(IResource aResource) throws CoreException {
				resultSet.add(aResource);
				return true;
			}
		}, IResource.DEPTH_INFINITE, flags);
		return resultSet;
	}

	/**
	 * Purge the inventory information from the cache
	 * 
	 * @param root
	 * @param deep
	 * @throws CoreException
	 */
	public void purgeCache(IContainer root, boolean deep) throws CoreException {
		inventoryCache.purgeCache(root, deep);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {
//		inventoryCache.flushPendingStatuses();
	}

	public void resourceStateChanged(BzrChangeStateEvent event) {
		if((event.getType() & INTERESTING_CHANGES) > 0) {
			for (IResource resource : event.getResources()) {
				if(resource.isAccessible() && !resource.isLinked()) {
					try {
						basicGetInventory(resource, inventoryUpdater);
					} catch (CoreException e) {
						EclipseBazaarCore.log(e);
					}
				}
			}
		}
	}

}
