/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.inventory;

import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 * 
 */
public class NaiveCache implements IInventoryCache {

	private Map<IProject, Set<IResource>> projectMap = new ConcurrentHashMap<IProject, Set<IResource>>();
	private Map<IProject, Long> lastUpdateMap = new ConcurrentHashMap<IProject, Long>();
	private static Long expireTime = 5000L;

	public IResource add(IResource resource) {
		Set<IResource> resourceSet = projectMap.get(resource.getProject());
		if(resourceSet == null) {
			resourceSet = new HashSet<IResource>();
			projectMap.put(resource.getProject(), resourceSet);
		}
		resourceSet.add(resource);
		lastUpdateMap.put(resource.getProject(), new Date().getTime());
		return resource;
	}

	public Boolean isInInventory(IResource resource) {
		IProject project = resource.getProject();
		if(resource.getType() == IResource.PROJECT) {
			project = (IProject)resource;
		}
		Set<IResource> resourceSet = projectMap.get(project);
		if (resourceSet != null && resourceSet.size() > 0) {
			EclipseBazaarCore.trace(ITracingOptions.INVENTORY_CACHE, "inventory query+delta: " + (new Date().getTime() - lastUpdateMap.get(resource.getProject()) <= expireTime), resource);
			return (resourceSet.contains(resource) && (new Date().getTime() - lastUpdateMap.get(project) <= expireTime));
		}
		return null;
	}
	
	public Boolean needRefresh(IResource resource) {
		Long creationTime = lastUpdateMap.get(resource.getProject());
		if(creationTime != null) {
			EclipseBazaarCore.trace(ITracingOptions.INVENTORY_CACHE, "inventory refresh delta: " + (new Date().getTime() - creationTime), resource);
			return !((new Date().getTime() - creationTime) < expireTime);
		}
		return true;
	}

	public void purgeCache(IContainer root, boolean deep) throws CoreException {
		if(root != null) {
			root.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					remove(resource);
					return true;
				}
			});
		}
	}

	public IResource remove(IResource resource) {
		if(resource != null && resource.getProject() != null) {
			Set<IResource> resourceSet = projectMap.get(resource.getProject());
			if(resourceSet != null) {
				resourceSet.remove(resource);
			}
		}
		return resource;
	}

	
}
