package org.vcs.bazaar.eclipse.core.listeners;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.Job;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.utils.BzrJobStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

public abstract class AbstractResourceChangeListener implements IResourceChangeListener {

	private final Job createJob(final IProject project, final List<IResource> resources) {
		return new Job("[Bazaar] Refreshing " + project.getName() + " status ") {
			private final IProject family = project;
			@Override
			public IStatus run(IProgressMonitor monitor) {
				try {
					IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
					subMonitor.beginTask("Status refresh", 100000);
					if (project != null && !subMonitor.isCanceled()) {
						refreshStatus(project, resources, monitor);
					} else {
						return Status.CANCEL_STATUS;
					}
				} catch (Exception e) {
					return BzrJobStatus.ERROR(this, e, e.getMessage());
				} finally {
					if(monitor != null)
						monitor.done();
				}
				return Status.OK_STATUS;
			}
			
			@Override
			public boolean belongsTo(Object family) {
				if(family instanceof IProject) {
					return this.family.equals(family);
				}
				return super.belongsTo(family);
			}
		};
	}

	/**
	 * Refresh (reset/reload) the status of all the given resources.
	 * 
	 * @param resources
	 *            Array of IResources to refresh
	 * @param monitor 
	 */
	private final void refreshStatus(IProject project, List<IResource> resources, IProgressMonitor monitor) {
		try {
			IResource[] refreshed = EclipseBazaarCore.getDefault().getStatusCacheManager().refreshStatus(project, monitor);
			resources.addAll(Arrays.asList(refreshed));
			monitor.subTask("Broadcasting changes");
			BzrChangeStateEvent event = new BzrChangeStateEvent(this, resources, BzrChangeStateEvent.RESOURCE_MODIFY);
			EclipseBazaarCore.broadcastModificationStateChanges(event);
		} catch (BazaarException e) {
			EclipseBazaarCore.log(e.getStatus().getMessage(), e.getStatus().getSeverity());
		} finally {
			monitor.done();
		}
	}

	protected final void refreshResources(final Collection<IResource> modifiedResources) {
		Map<IProject, List<IResource>> projects = new HashMap<IProject, List<IResource>>();
		for (IResource resource : modifiedResources) {
			List<IResource> resources = projects.get(resource.getProject());
			if(resources == null) {
				resources = new ArrayList<IResource>();
				projects.put(resource.getProject(), resources);
			}
			resources.add(resource);
		}
		for (IProject project : projects.keySet()) {
			Job[] jobs = Job.getJobManager().find(project);
			// if a job of the same "family" (project) is scheluded, pass
			if(jobs != null && jobs.length > 0)
				continue;
			Job job = createJob(project, projects.get(project));
			job.setPriority(Job.DECORATE);
			job.setSystem(false);
			ISchedulingRule rule = ResourcesPlugin.getWorkspace().getRuleFactory().refreshRule(project);
			job.setRule(rule);
			job.schedule(600);
		}
	}

	public AbstractResourceChangeListener() {
		super();
	}
	
}