/*******************************************************************************
 * Copyright (c) 2003, 2006 Subclipse project and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Subclipse project committers - initial API and implementation
 ******************************************************************************/
package org.vcs.bazaar.eclipse.core.listeners;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.Team;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;

/**
 * This class performs several functions related to determining the modified
 * status of files under Bazaar control. First, it listens for change
 * delta's for files and folders, and brodcasts them to all listeners.
 * 
 * @modifiedBy Guillermo Gonzalez
 */
public class FileModificationManager extends AbstractResourceChangeListener {
	protected int INTERESTING_CHANGES = IResourceDelta.CONTENT //
			| IResourceDelta.MOVED_FROM // 
			| IResourceDelta.MOVED_TO //
			| IResourceDelta.OPEN //
			| IResourceDelta.REPLACED //
			| IResourceDelta.TYPE;

	public void resourceChanged(IResourceChangeEvent event) {
		try {
			if(event.getBuildKind() != 0) {
				return;
			}
			
			final List<IResource> modifiedResources = new ArrayList<IResource>();
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "FileModificationManager#resourceChanged");
			event.getDelta().accept(new IResourceDeltaVisitor() {
				public boolean visit(IResourceDelta delta) throws CoreException {
					final IResource resource = delta.getResource();
					EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, " DeltaVisitor: ", resource);
					if(Team.isIgnoredHint(resource))
						return false;
					if(resource.getType() != IResource.ROOT && !shouldHandle(resource)) {
						return false;
					}
					if(resource.getType() == IResource.ROOT) {
						return true;
					}
					if (resource.getType() == IResource.FILE) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  File");						
						return handleFile(modifiedResources, delta, resource);
					} else if (resource.getType() == IResource.FOLDER) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Folder");
						return handleFolder(modifiedResources, delta, resource);
					} else if (resource.getType() == IResource.PROJECT && resource.exists()) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Project");
						return handleProject(modifiedResources, delta, (IProject)resource);
					}
					return true;
				}

			}, IFolder.EXCLUDE_DERIVED);
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "FileModificationManager#collectedChanges: ", (Object[])modifiedResources.toArray(new IResource[0]));
			// we refresh all changed resources and broadcast the changes to all
			// listeners (ex : BazaarLightweightDecorator)
			if(modifiedResources.size() > 0) {
				refreshResources(modifiedResources);
			}
			
		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getStatus().getMessage(), e.getStatus().getSeverity());
		}
	}
	

	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 */
	private boolean handleFile(final List<IResource> modifiedResources, IResourceDelta delta, final IResource resource) {
		if((delta.getFlags() & IResourceDelta.MARKERS) != 0) {
			return false;
		}
		if(resource.exists() && (delta.getFlags() & INTERESTING_CHANGES) != 0) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   InterestingChanges: ", resource);
			modifiedResources.add(resource.getParent());
			modifiedResources.add(resource);
		} else if ((delta.getKind() == IResourceDelta.CHANGED || delta.getKind() == IResourceDelta.ADDED) && resource.exists()) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   added/changed: ", resource);
			modifiedResources.add(resource.getParent());
			modifiedResources.add(resource);
		} else if (delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Removed: ", resource);
			// provide notifications for deletions since they
			// may not have been managed
			// The move/delete hook would have updated the
			// parent counts properly
			modifiedResources.add(resource.getParent());
		}
		return true;
	}


	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 * @return
	 * @throws CoreException
	 */
	private boolean handleFolder(final List<IResource> modifiedResources, IResourceDelta delta, final IResource resource)
			throws CoreException {
		if (delta.getKind() == IResourceDelta.ADDED || delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Added/Removed: ", resource);
			modifiedResources.add(resource);
			return false;
		} else if (resource.exists() && delta.getKind() == IResourceDelta.CHANGED) {
			return !(resource.isLinked(IResource.CHECK_ANCESTORS) || resource.isDerived());
		}
		return true;
	}

	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 * @throws CoreException
	 */
	private boolean handleProject(final List<IResource> modifiedResources, IResourceDelta delta, final IProject project)
			throws CoreException {
		if (project.isAccessible() && RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID()) == null) {
			return false; // not a bazaar handled project
		}
		if (!project.isAccessible()) {
			EclipseBazaarCore.getDefault().getStatusCacheManager().purgeCache(project, true);
			EclipseBazaarCore.getDefault().getInventoryCacheManager().purgeCache(project, true);
			EclipseBazaarCore.getDefault().getLogCacheManager().purgeCache(project);
			return false;
		} else if ((delta.getFlags() & IResourceDelta.OPEN) != 0) {
			EclipseBazaarCore.getDefault().getStatusCacheManager().purgeCache(project, true);
			EclipseBazaarCore.getDefault().getLogCacheManager().purgeCache(project);
			modifiedResources.add(project);
			return false;
		}
		if (delta.getKind() == IResourceDelta.ADDED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Added: ", project);
			modifiedResources.add(project);
			return false;
		} else if (delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Removed", project);
			modifiedResources.add(project);
			return false;
		}
		return true;
	}

}
