/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.listeners;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;

/**
 * Listens to .bzr meta-dir changes.<br>
 * When a change occurs the status of the branch/project is refreshed
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 */
public class SyncFileChangeListener extends AbstractResourceChangeListener {

	protected int INTERESTING_CHANGES = IResourceDelta.CONTENT 
										| IResourceDelta.MOVED_FROM 
										| IResourceDelta.MOVED_TO 
										| IResourceDelta.REPLACED
										| IResourceDelta.TYPE;

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {
		try {
			final List<IResource> affectedResources = new ArrayList<IResource>();
			final Set<IResource> closedProjects = new HashSet<IResource>();

			EclipseBazaarCore.trace(ITracingOptions.METADIR_CHANGE_LISTENER, "SyncFileChangeListener#collectingChanges/eventkind:" + event.getType());
			event.getDelta().accept(new IResourceDeltaVisitor() {
				public boolean visit(IResourceDelta delta) {
					IResource resource = delta.getResource();

					if (resource.getType() == IResource.ROOT) {
						return true;
					} else if (resource.getType() == IResource.PROJECT) {
						if (!resource.isAccessible()) {
							closedProjects.add(resource);
							return false;
						}
						return shouldHandle(resource);
					}

					String name = resource.getName();
					IContainer toBeNotified = null;

					if (IBzrBranch.DIR_META_FOLDER.equals(name)) {
						int kind = delta.getKind();
						toBeNotified = handleBzrDir((IContainer) resource, kind);
					} else if (isInsideMetaFolder(resource)) {
						toBeNotified = resource.getParent();
					} else {
						return false;
					}

					if (toBeNotified != null && !affectedResources.contains(toBeNotified)) {
						affectedResources.add(toBeNotified);
						return false;
					} else {
						return true;
					}
				}

			}, IContainer.INCLUDE_TEAM_PRIVATE_MEMBERS);

			if(closedProjects.size() > 0) {
				BzrChangeStateEvent bzrEvent = new BzrChangeStateEvent(this, closedProjects, BzrChangeStateEvent.RESOURCE_MODIFY);
				EclipseBazaarCore.broadcastModificationStateChanges(bzrEvent);
			}
			if(affectedResources.size() > 0)
				refreshResources(affectedResources);

		} catch (CoreException e) {
			EclipseBazaarCore.log(e);
		}
	}

	private boolean isInsideMetaFolder(IResource resource) {
		IProject project = resource.getProject();
		return project.getFolder(new Path(".bzr")).getFullPath().isPrefixOf(resource.getFullPath());
	}

	/*
	 * If it's a new .bzr directory then mark it as team-private.
	 */
	protected IContainer handleBzrDir(IContainer bzrDir, int kind) {
		if ((kind & INTERESTING_CHANGES) != 0) {
			if (!bzrDir.isTeamPrivateMember() && kind == IResourceDelta.ADDED) {
				try {
					bzrDir.setTeamPrivateMember(true);
				} catch (CoreException e) {
					EclipseBazaarCore.log(e);
				}
			}
			return bzrDir;
		}
		return null;
	}

}
