/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.log;

import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.jobs.ILock;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.AbstractCacheManager;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.IBzrResource;

/**
 * A basic/dumb Log/history cache manager.
 * ATM it's just a façade, there is no cache working.
 * TODO: need to workout a solution about how (and when)refresh the cache (and how to force a refresh)   
 * 
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 * 
 */
public class LogCacheManager extends AbstractCacheManager {

	private final static int INTERESTING_CHANGES = BzrChangeStateEvent.BRANCH_STATE_MODIFY;
	
	/**
	 * Name used for identifying Bazaar synchronization data in
	 * Resource>ResourceInfo#syncInfo storage
	 */
	public static final QualifiedName BZR_SYNC_KEY = new QualifiedName(EclipseBazaarCore.getPluginId(), "bzr-log-sync-key");

	private ILogCache logCache;
	private LogUpdater logUpdater;

	public LogCacheManager() {
		ResourcesPlugin.getWorkspace().getSynchronizer().add(LogCacheManager.BZR_SYNC_KEY);
		logCache = new NaiveCache();
		logUpdater = new LogUpdater();
	}
	
	/**
	 * Returns the last log of the given resource. If is not present in
	 * cache, it will be retrieved using the actual strategy. 
	 * 
	 * @param resource whose last log is required.
	 * @throws CoreException
	 */
	public IBazaarLogMessage getLastLog(IResource resource) throws CoreException {
		return getLogs(resource, logUpdater).get(0);	
	}
	
	
	/**
	 * Returns the logs of the given resource. If is not present in
	 * cache, it will be retrieved using the actual strategy. 
	 * 
	 * @param resource whose last log is required.
	 * @return a List<IBazaarLogMessage> for the corresponding resource 
	 * (the size is dependent on the limit setup in the preferences)
	 * @throws CoreException
	 */
	public List<IBazaarLogMessage> getLogs(IResource resource) throws CoreException {
		return getLogs(resource, logUpdater);	
	}

	/**
	 * see {@link #getLogs(IResource)}
	 */
	public List<IBazaarLogMessage> getLogs(IBzrResource resource) throws CoreException {
		return getLogs(resource, logUpdater, getLock(resource));
	}

	private List<IBazaarLogMessage> getLogs(final IResource resource, final LogUpdater strategy) throws CoreException {
		if (!resource.isAccessible() || resource.isPhantom() || resource.isLinked()) {
			return Collections.emptyList();
		}
		IProject project = resource.getProject();
		if(resource.getType() == IResource.PROJECT) {
			project = (IProject)resource;
		}
		final ILock lock = getLock(project);
		return getLogs(resource, strategy, lock);
	}
	
	private List<IBazaarLogMessage> getLogs(final Object resource, final LogUpdater strategy, final ILock lock) throws CoreException {
		List<IBazaarLogMessage> logs = null;
		boolean acquired = false;
		try {
			acquired = lock.acquire(3000L);
			// TODO: for the moment the cache don't detect when it need a refresh.
			// so I leave this without the cache.
//			logs = logCache.get(resource);
			if (logs == null || logs.size() == 0) {
				if(resource instanceof IBzrResource) {
					logs = basicGetLog((IBzrResource)resource, strategy);
				} else {
					logs = basicGetLog((IResource)resource, strategy);
				}
			}
		} catch(InterruptedException ie) { 
			EclipseBazaarCore.log(ie.getMessage(), ie);
		} finally {
			if(acquired) {
				lock.release();
			}
		}
		return logs;
	}

	/**
	 * refresh the cache and returns if the resource is in it.
	 */
	private List<IBazaarLogMessage> basicGetLog(IBzrResource resource, LogUpdater strategy) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.LOG_CACHE, "LogCacheManager.basicGetlog: " + resource.getName());
		// update the cache
		return updateCache(resource, strategy.fetchLog(resource, new NullProgressMonitor()));
	}
	
	private List<IBazaarLogMessage> basicGetLog(IResource resource, LogUpdater strategy) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.LOG_CACHE, "LogCacheManager.basicGetlog: " + resource.getName());
		// update the cache
		return updateCache(resource, strategy.fetchLog(resource, new NullProgressMonitor()));
	}

	private List<IBazaarLogMessage> updateCache(Object resource, List<IBazaarLogMessage> logs) {
		return logCache.add(resource, logs);
	}

	/**
	 * Purge the cache of the spacified IProject from the cache
	 */
	public void purgeCache(IProject root) throws CoreException {
		logCache.purgeCache(root);
	}

	public void resourceChanged(final IResourceChangeEvent event) {
		
	}

	public void resourceStateChanged(final BzrChangeStateEvent event) {
		if((event.getType() & INTERESTING_CHANGES) > 0) {
			for (IResource resource : event.getResources()) {
				if(resource.isAccessible() && !resource.isLinked()) {
					try {
						basicGetLog(resource, this.logUpdater);
					} catch (CoreException e) {
						EclipseBazaarCore.log(e);
					}
				}
			}
		}
	}
}
