/**
 * LICENCSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.repository;

import static org.vcs.bazaar.client.core.BranchLocation.LP_SCHEME;
import static org.vcs.bazaar.client.core.BranchLocation.REMOTE_SCHEMES;

import java.io.File;
import java.net.URI;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Assert;
import org.vcs.bazaar.client.utils.StringUtil;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.internal.core.model.LocalBranch;
import org.vcs.bazaar.eclipse.internal.core.model.LpBranch;
import org.vcs.bazaar.eclipse.internal.core.model.ProjectBranch;
import org.vcs.bazaar.eclipse.internal.core.model.RemoteBranch;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class BranchFactory {

	private static final BranchFactory instance = new BranchFactory();

	private BranchFactory(){}
	public static BranchFactory getInstance() {
		return instance;
	}

	/**
	 * @param URI
	 * 
	 * @return IBzrBranch for the given URI (or null if there is no branch)
	 * TODO: this method should only return RemoteBranch instances?
	 */
	public IBzrBranch findOrCreateFor(URI uri) {
		if (uri != null) {
			if(uri.getScheme() == null) {
				// asume it's a raw path
				File file = new File(uri.getSchemeSpecificPart());
				if(file.exists() || file.getParentFile().exists()) {				
					return new LocalBranch(uri);
				}
			}
            if (uri.getScheme() != null && "file".equals(uri.getScheme())) {
				return new LocalBranch(uri);
			}
			for (String remoteScheme : REMOTE_SCHEMES) {
				if (uri.getScheme() != null && uri.getScheme().equals(remoteScheme)) {
					return new RemoteBranch(uri);
				}
			}
		}
		return null;
	}

	/**
	 * @param URI
	 *            as String
	 * @return IBzrBranch for the given URI (or null if there is no branch)
	 */
	public IBzrBranch findOrCreateFor(String uri) {
		Assert.isTrue(validateBranch(uri));
		if (uri != null && uri.startsWith(LP_SCHEME)) {
			return new LpBranch(uri);
		}
		return findOrCreateFor(StringUtil.getAbsoluteURI(uri));
	}

	/**
	 * @param IProject
	 *            which this branch belongs
	 * @return a LocalBranch that for the localProject, with it's .bzr folder in
	 *         the project root folder or it parent
	 */
	public IBzrBranch findOrCreateFor(IProject project) {
		if (project != null) {
			return new ProjectBranch(project);
		}
		return null;
	}

	/**
	 * Check if the URI contains a valid branch
	 * 
	 * @param uri
	 * @return boolean
	 */
	public static boolean validateBranch(String uri) {
		return URIValidator.validateBranchLocation(uri);
	}

	private static class URIValidator {

		private static boolean validateBranchLocation(String uri) {
			try {
				if (uri == null || uri.trim().length() == 0) {
					return false;
				}
	            if (uri != null && uri.startsWith(LP_SCHEME)) {
					return true;
				}
				URI locationURI = StringUtil.getAbsoluteURI(uri);
				try {
					if(locationURI != null) {
						// try to create a branch from the URI
						getInstance().findOrCreateFor(locationURI);
					}
					// if a IBzrBranchcan be created, it's ok
					return true;
				} catch(Exception e) {
					return false;
				}
			} catch (Exception e) {
				// this is a malformed uri or a local file
				// now we try to check if this is a valid folder location
				File file = new File(uri);
				if(file.exists()) {
					return true;
				}
				return false;
			}
		}
	}

}
