/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.core.resources;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.team.IMoveDeleteHook;
import org.eclipse.core.resources.team.IResourceTree;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFolder;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class BazaarMoveDeleteHook implements IMoveDeleteHook {

	public boolean deleteFile(IResourceTree tree, IFile file, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(file))
			return false;

		IBzrLocalResource resource = BzrWorkspaceRoot.getBzrFileFor(file);
		try {
			if (!isUnderBazaarControl(resource) || !resource.isManaged())
				return false;

			monitor.beginTask("Removing file(s)", 1000);
			resource.delete();
			tree.deletedFile(file);
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean deleteFolder(IResourceTree tree, IFolder folder, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(folder))
			return false;

		IBzrLocalResource resource = BzrWorkspaceRoot.getBzrFileFor(folder);
		try {
			if (!isUnderBazaarControl(resource) || !resource.isManaged())
				return false;

			monitor.beginTask("Removing folder(s)", 1000);
			resource.delete();
			tree.deletedFolder(folder);
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean deleteProject(IResourceTree tree, IProject project, int updateFlags, IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean moveFile(IResourceTree tree, IFile source, IFile destination, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(source))
			return false;

		IBzrLocalResource sourceResource = BzrWorkspaceRoot.getBzrFileFor(source);
		try {
			if (!isUnderBazaarControl(sourceResource) || !sourceResource.isManaged())
				return false;
			if (!BzrWorkspaceRoot.getBzrResourceFor(destination.getParent()).getBranch().equals(sourceResource.getBranch())) {
				return false;
			}
			monitor.beginTask("Moving file(s)", 1000);
			IContainer parentDest = destination.getParent();
			IBzrLocalResource parentDestResource = BzrWorkspaceRoot.getBzrFolderFor(parentDest);
			// if the parent of the destination is not versioned we must add it
			// to Bazaar first
			if (parentDestResource == null || !parentDestResource.isManaged()) {
				parentDestResource.getBzrWorkspaceRoot().getClient().add(new File[] { parentDest.getLocation().toFile() });
			}
			sourceResource.move(destination.getLocation());
			tree.movedFile(source, destination);
			destination.refreshLocal(IResource.DEPTH_ZERO, monitor);
		} catch (BazaarClientException e) {
			tree.failed(BazaarException.wrapException(e).getStatus());
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} catch (CoreException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean moveFolder(IResourceTree tree, IFolder source, IFolder destination, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(source))
			return false;

		// FIXME: change this when Bug #107967 get fixed
		// if I try to move source into source/newlocation
		// bazaar tells: bzr: ERROR: Could not move source => newlocation:
		// Invalid argument
		// so for the moment we don't take care of this case
		if (source.getLocation().isPrefixOf(destination.getLocation())) {
			return false;
		}

		IBzrLocalResource sourceResource = BzrWorkspaceRoot.getBzrFolderFor(source);
		try {

			if (sourceResource == null || !isUnderBazaarControl(sourceResource))
				return false;
			IBzrLocalFolder bzrDest = BzrWorkspaceRoot.getBzrFolderFor(destination.getParent());
			if (bzrDest == null || !bzrDest.getBranch().equals(sourceResource.getBranch())) {
				return false;
			}
			monitor.beginTask("Removing folder(s)", 1000);
			sourceResource.move(destination.getLocation());
			tree.movedFolderSubtree(source, destination);
			destination.refreshLocal(IResource.DEPTH_INFINITE, monitor);
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} catch (CoreException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean moveProject(IResourceTree tree, IProject source, IProjectDescription description, int updateFlags, IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		return false;
	}

	private static boolean isUnderBazaarControl(IBzrLocalResource resource) {
		try {
			if(resource != null) {
				return resource.isManaged();
			}
		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
		}
		return false;
	}
}
