/*******************************************************************************
 * Copyright (c) 2004, 2006 Subclipse project and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Subclipse project committers - initial API and implementation
 ******************************************************************************/
package org.vcs.bazaar.eclipse.core.status;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.jobs.ILock;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.AbstractCacheManager;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.commands.RevnoCommand;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * Provides a method to get the status of a resource. <br>
 * It is much more efficient to get the status of a set a resources than only
 * one resource at a time. For that we use a
 * 
 * @link org.tigris.subversion.subclipse.core.status.StatusUpdateStrategy<br>
 * 
 * We use a tree (@link
 * org.tigris.subversion.subclipse.core.status.StatusCacheComposite) to keep the
 * status of the resources
 * 
 * @author cedric chabanois (cchab at tigris.org)
 * 
 * @modifiedBy Guillermo Gonzalez
 */
public class StatusCacheManager extends AbstractCacheManager {
	
	private final static int INTERESTING_CHANGES = BzrChangeStateEvent.BRANCH_STATE_MODIFY
			| BzrChangeStateEvent.WT_STATE_MODIFY
			| BzrChangeStateEvent.RESOURCE_MODIFY;

	/**
	 * Name used for identifying Bazaar synchronization data in
	 * Resource>ResourceInfo#syncInfo storage
	 */
	public static final QualifiedName BZR_SYNC_KEY = new QualifiedName(EclipseBazaarCore.getPluginId(), "bzr-status-sync-key");
	private IStatusCache statusCache;
	private StatusUpdater statusUpdater;

	public StatusCacheManager() {
		ResourcesPlugin.getWorkspace().getSynchronizer().add(StatusCacheManager.BZR_SYNC_KEY);
		statusCache = new NaiveCache();
		statusUpdater = new StatusUpdater();
	}

	/**
	 * @param resource
	 * @return true when the resource's status is present in cache
	 */
	public boolean hasCachedStatus(IResource resource) {
		return statusCache.hasCachedStatus(resource);
	}

	/**
	 * update the cache using the given statuses
	 * 
	 * @param statuses
	 * @param rule
	 *            the scheduling rule to use when running this operation
	 */
	protected List<IResource> updateCache(final IBzrResourceStatus[] statuses, ISchedulingRule rule) throws CoreException {
		final List<IResource> result = new ArrayList<IResource>(statuses.length);
		if (ResourcesPlugin.getWorkspace().isTreeLocked()) {
			for (int i = 0; i < statuses.length; i++) {
				result.add(updateCache(statuses[i]));
			}
		} else {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					for (int i = 0; i < statuses.length; i++) {
						result.add(updateCache(statuses[i]));
					}
				}
			}, rule, IWorkspace.AVOID_UPDATE, null);
		}
		return result;
		
	}

	/**
	 * update the cache using the given statuses
	 * 
	 * @param status
	 * @param workspaceRoot
	 */
	protected IResource updateCache(IBzrResourceStatus status) {
		return statusCache.addStatus(status);
	}

	/**
	 * Get the status of the given resource. If the status is not present in
	 * cache, it will be retrieved using the actual updateStrategy. If recursive
	 * startegy is being used also all child resources would be updated,
	 * otherwise only direct childern
	 * 
	 * @param resource
	 *            whose status is required.
	 * @throws CoreException
	 */
	public IBzrResourceStatus getStatus(IResource resource) throws CoreException {
		return getStatus(resource, statusUpdater);
	}

	/**
	 * The cached statuses do not provide revision numbers. This method
	 * allow to query for the revision of the resource explicitely.
	 * TODO: add the revision to the cache
	 * 
	 * @param resource
	 * @return
	 * @throws BazaarException
	 */
	public BazaarRevision getResourceRevision(IResource resource) throws BazaarException {
		if (resource == null)
			return null;
		RevnoCommand cmd = new RevnoCommand(resource);
		cmd.run(new NullProgressMonitor());
		BazaarRevision rev = cmd.getRevision();
		return rev;
	}

	/**
	 * get the status of the given resource
	 * 
	 * @throws CoreException
	 */
	private IBzrResourceStatus getStatus(IResource resource, StatusUpdater strategy) throws CoreException {
		if (!resource.exists() && !resource.isPhantom()) {
			return null;
		}
		IBzrResourceStatus status = null;
		ILock lock = getLock(resource.getProject());
		boolean acquired = false;
		try {
			acquired = lock.acquire(3000L);
			status = statusCache.getStatus(resource);
			if (status == null) {
					status = basicGetStatus(resource, strategy);
			}
		} catch(InterruptedException ie) { 
			EclipseBazaarCore.log(ie.getMessage(), ie);
		} finally {
			if(acquired) {
				lock.release();
			}
		}
		return status;
	}

	/**
	 * Get the statuse(s)
	 * 
	 * @param resource
	 * @param strategy
	 * @return
	 * @throws CoreException
	 */
	private IBzrResourceStatus basicGetStatus(IResource resource, StatusUpdater strategy) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.STATUS_CACHE, "StatusCacheManager.getBasicStatus: " + resource.getName());
		if (!resource.isAccessible())
			return null;
		// update the cache
		setStatuses(strategy.statusesToUpdate(resource, new NullProgressMonitor()), resource.getType() == IResource.FILE?resource.getParent():resource);
		return statusCache.getStatus(resource);
	}

	/**
	 * The cache manager handles itself the status retrieving. However this
	 * method can be used to update the statuses of some resources
	 * 
	 * @param statuses
	 * @param rule
	 *            the scheduling rule to use when running this operation
	 */
	private void setStatuses(IBzrResourceStatus[] statuses, ISchedulingRule rule) throws BazaarException {
		try {
			if(statuses == null || statuses.length == 0) {
				return;
			}
			// we don't need a rule here, we only require read-access to resources
			updateCache(statuses, null);
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		}
	}

	/**
	 * Refresh the status of the given resource to the give depth. The depth can
	 * be deeper in case of phantom resources. These have to be traversed to
	 * infinite always ...
	 * 
	 * @param resource
	 * @param monitor the progress monitor to use for reporting progress to the user. It is the caller's responsibility
	 *        to call done() on the given monitor. Accepts <code>null</code>, indicating that no progress should be
	 *        reported and that the operation cannot be cancelled
	 * @param recursive
	 * @return array of resources which were refreshed (including all phantoms
	 *         and their children)
	 * @throws BazaarException
	 */
	public IResource[] refreshStatus(final IContainer resource, IProgressMonitor monitor) throws BazaarException {
		IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
		subMonitor.subTask("Refreshing status cache");
		if (!shouldHandle(resource)) {
			return new IResource[0];
		}
		try {
			List<IResource> refreshedResources = updateCache(statusUpdater.statusesToUpdate(resource, subMonitor), resource);
			Set<IResource> resourcesToRefresh = resourcesToRefresh(resource, IContainer.INCLUDE_PHANTOMS, refreshedResources.size());
			for (Iterator<IResource> iter = refreshedResources.iterator(); iter.hasNext();) {
				resourcesToRefresh.remove(iter.next());
			}
			// Resources which were not refreshed above (e.g. deleted resources)
			// We do it with depth = infinite, so the whole deleted trees are
			// refreshed.
			for (Iterator<IResource> it = resourcesToRefresh.iterator(); it.hasNext();) {
				IResource res = (IResource) it.next();
				if(resource.isLinked(IResource.CHECK_ANCESTORS) 
						|| resource.isDerived())
					continue;
				if ((res.getType() != IResource.FILE) && res.isPhantom()) {
					Set<IResource> children = resourcesToRefresh(res, IContainer.INCLUDE_PHANTOMS | IContainer.INCLUDE_TEAM_PRIVATE_MEMBERS, 0);
					for (Iterator<IResource> iter = children.iterator(); iter.hasNext();) {
						IResource child = (IResource) iter.next();
						statusCache.removeStatus(child);
						refreshedResources.add(child);
					}
				}
				statusCache.removeStatus(res);
				refreshedResources.add(res);
			}
			return (IResource[]) refreshedResources.toArray(new IResource[refreshedResources.size()]);
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			subMonitor.done();
		}
	}

	private Set<IResource> resourcesToRefresh(IResource resource, int flags, int expectedSize) throws CoreException {
		if (!resource.exists() && !resource.isPhantom()) {
			return new HashSet<IResource>(0);
		}
		final Set<IResource> resultSet = (expectedSize != 0) ? new HashSet<IResource>(expectedSize) : new HashSet<IResource>();
		resource.accept(new IResourceVisitor() {
			public boolean visit(IResource aResource) throws CoreException {
				resultSet.add(aResource);
				return true;
			}
		}, IResource.DEPTH_INFINITE, flags);
		return resultSet;
	}

	/**
	 * Purge the status information from the cache
	 * 
	 * @param root
	 * @param deep
	 * @throws CoreException
	 */
	public void purgeCache(IContainer root, boolean deep) throws CoreException {
		statusCache.purgeCache(root, deep);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 * 
	 * When a resource changes this method will be called in a PRE_BUILD to
	 * allow to flush all changes which were not saved during previous
	 * operations when the workspace was locked.
	 * 
	 */
	public void resourceChanged(IResourceChangeEvent event) {
		statusCache.flushPendingStatuses();
	}

	public void resourceStateChanged(BzrChangeStateEvent event) {
		if((event.getType() & INTERESTING_CHANGES) > 0) {
			for (IResource resource : event.getResources()) {
				if(resource.isAccessible() && !resource.isLinked()) {
					try {
						basicGetStatus(resource, this.statusUpdater);
					} catch (CoreException e) {
						EclipseBazaarCore.log(e);
					}
				}
			}
		}
	}

}
