/**
 * LICENSE + COPYRIGHT 
 */
package org.vcs.bazaar.eclipse.core.utils;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.IBazaarStatus;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;

/**
 * <p>
 * provides static helping functionality for working with resources.
 * </p>
 * 
 * @author Guillermo Gonzalez
 * @basedOn ResourceUtil from eclipsedarcs by Leif Frenzel 
 * 
 */
public class ResourceUtil {

	public static boolean isUnderBazaarScope(final IProject project) {
		IPath projectLocation = project.getLocation();
		return findRootLocation(projectLocation.toFile()) != null;
	}

	/**
	 * find the root location of the repository that contains the given project;
	 * if the project itself is a repository (contains the <code>.bzr</code>
	 * metafolder), the path to the project is returned. May be null, if no
	 * repository is found.
	 */
	public static IPath findRootLocation(final IProject project) {
		File projectFile = project.getLocation().toFile();
		File location = findRootLocation(projectFile);
		return location != null?new Path(location.toString()):null;
	}

	/**
	 * FIXME: support for multiple project per branch 
	 */
	public static void setMetaFolderTeamPrivate(final IProject project, final boolean teamPrivate) {
		try {
			IFolder folder = project.getFolder(IBzrBranch.DIR_META_FOLDER);
			if (folder.exists()) {
				folder.setTeamPrivateMember(teamPrivate);
			}
		} catch (CoreException e) {
			EclipseBazaarCore.getDefault().getLog().log(e.getStatus());
		}
	}


	/**
	 * returns the path to the repository in which the specified resource is
	 * shared, or null, if the resource is not in a shared project.
	 */
	public static IPath getRepositoryPath(final IResource resource) {
		IPath result = null;
		if (resource == null)
			return null;
		IProject project = resource.getProject();
		if (RepositoryProvider.isShared(project)) {
			RepositoryProvider prov = RepositoryProvider.getProvider(project);
			if (prov instanceof BazaarTeamProvider) {
				BzrWorkspaceRoot bzrWorkspaceRoot = ((BazaarTeamProvider) prov).getBzrWorkspaceRoot();
				if(bzrWorkspaceRoot != null)
					result = bzrWorkspaceRoot.getBranch().getPath();
				else {
					result = findRootLocation(project);
				}
			}
		} else {
			result = findRootLocation(project);
		}
		return result;
	}

	/**
	 * <p>
	 * returns true, if the project directory is a Bazaar branch, i.e.
	 * contains the <code>.bzr/</code> directory.
	 * </p>
	 * 
	 * <p>
	 * Note: this returns false if the project is not already shared.
	 * </p>
	 */
	public static boolean isRepoDirInProject(final IProject project) {
		IPath repoPath = getRepositoryPath(project);
		return project.getLocation().equals(repoPath);
	}

	/**
	 * Set dirty properties in resource (or it's project, branch, etc)
	 * 
	 * @param lastResource
	 * @param resource
	 * @return the resource
	 */
	public static void markAsDirty(IResource resource) {
		IProject project = resource.getProject();
		IBzrBranch branch = BzrWorkspaceRoot.getBzrWorkspaceRootFor(project).getBranch();
		try {
			Object dirty = project.getSessionProperty(IBazaarCoreConstants.DIRTY);
			if(branch.isProjectBranch() && (dirty == null || !(Boolean)dirty)) {
				project.setSessionProperty(IBazaarCoreConstants.DIRTY, Boolean.TRUE);
			}
		} catch (CoreException e) {
			// silently ignore
		} 
		
	}
	
	public static void cleanCache(IResource[] resources) {
		for (IResource resource : resources) {
			IContainer container = null;
			if(resource.getType() == IResource.FILE)
				container = resource.getParent();
			else  
				container = (IContainer)resource;
			try {
				EclipseBazaarCore.getDefault().getStatusCacheManager().purgeCache(container, true);
				EclipseBazaarCore.getDefault().getInventoryCacheManager().purgeCache(container, true);
				EclipseBazaarCore.getDefault().getLogCacheManager().purgeCache(container.getProject());
			} catch (CoreException e) {
				EclipseBazaarCore.log(e);
			}
		}
	}
	
	/**
	 * @see IWorkspaceRoot#getFileForLocation(IPath)
	 * 
	 * @param absolutePath
	 * @return {@link IFile}
	 */
	public static IFile getIFileForLocation(String absolutePath) {
		return ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(absolutePath));
	}
	
	/**
	 * @see IWorkspaceRoot#getContainerForLocation(IPath)
	 * 
	 * @param absolutePath
	 * @return {@link IContainer}
	 */
	public static IContainer getIConatinerForLocation(String absolutePath) {
		return ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(absolutePath));
	}
	
	
	public static IResource getIFile(IProject project, IBazaarStatus aStatus) {
		IResource resource = getIFileForLocation(aStatus.getAbsolutePath());
		if(resource == null) {
			resource = ResourcesPlugin.getWorkspace().getRoot().getFile(getRelativePathToWorkspaceRoot(project, aStatus));
		} 
		return resource;
	}

	
	private static IPath getRelativePathToWorkspaceRoot(IProject project, IBazaarStatus status) {
		String lastSegment = project.getLocation().lastSegment();
		IPath absoluteFilePath = new Path(status.getAbsolutePath());
		String[] segments = absoluteFilePath.segments();
		int segIdx = 0;
		for (segIdx = 0; segIdx < segments.length; segIdx++) {
			if(segments[segIdx].equals(lastSegment)) {
				break;
			}
		}
		IPath relativePath = absoluteFilePath.removeFirstSegments(segIdx+1);
		return project.getFile(relativePath).getFullPath();
	}
	
	public static boolean shouldHandle(IResource resource) {
		boolean shouldHandle = !(resource.isDerived() || resource.isLinked(IResource.CHECK_ANCESTORS));
		IProject project = null;
		if(resource.getType() == IResource.PROJECT) {
			project = (IProject)resource;
		} else {
			project = resource.getProject();
		}
		return shouldHandle &&  project != null && RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID()) != null;
	}

	// helper methods
	// ////////////////

	private static File findRootLocation(final File file) {
		File metaFolder = new File(file, IBzrBranch.DIR_META_FOLDER);
		File result = null;
		if (metaFolder.exists() && metaFolder.isDirectory() && new File(metaFolder, "branch").exists()) {
			result = metaFolder.getParentFile();
		} else {
			File parent = file.getParentFile();
			// reached the root, no '.bzr' in sight, else recurse until reach read-only parent
			if(parent != null && parent.canWrite())
				result = (parent == null) ? null : findRootLocation(parent);
			else 
				result = null;
		}
		return result;
	}

}
