package org.vcs.bazaar.eclipse.internal.core;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.team.core.TeamException;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.utils.BzrStatus;

/**
 * A checked expection representing a failure in Bazaar.
 * <p>
 * Bazaar exceptions contain a status object describing the cause of the
 * exception.
 * </p>
 * 
 * @see IStatus
 */
public class BazaarException extends TeamException {

	private static final long serialVersionUID = 1L;
	private String message;

	public BazaarException(CoreException e) {
		super(e);
		setStackTrace(e.getStackTrace());
	}

	public BazaarException(int severity, int code, String message, Throwable e, IResource resource) {
		super(new BzrStatus(severity, code, e.getMessage(), e, resource));
		setStackTrace(getStatus().getException().getStackTrace());
		this.message = message;
	}

	public BazaarException(int severity, int code, String message, Throwable e) {
		this(severity, code, message, e, null);
	}

	public BazaarException(int severity, int code, String message) {
		this(severity, code, message, null);
	}

	public BazaarException(String message) {
		super(new BzrStatus(IStatus.ERROR, UNABLE, message, null));
	}

	public BazaarException(String message, Throwable e) {
		this(IStatus.ERROR, UNABLE, message, e);
	}

	public BazaarException(IStatus status) {
		super(status);
// setStackTrace(status.getException().getStackTrace());
		message = status.getMessage();
	}

	/*
	 * Static helper methods for creating exceptions
	 */
	public static BazaarException wrapException(IResource resource, String message, IOException e) {
		// NOTE: we should record the resource somehow
		// We should also inlcude the IO message
		return new BazaarException(new BzrStatus(IStatus.ERROR, IO_FAILED, message, e));
	}

	/*
	 * Static helper methods for creating exceptions
	 */
	public static BazaarException wrapException(IResource resource, String message, InterruptedException e) {
		// NOTE: we should record the resource somehow
		return new BazaarException(new BzrStatus(IStatus.ERROR, UNABLE, message, e));
	}

	/*
	 * Static helper methods for creating exceptions
	 */
	public static BazaarException wrapException(IResource resource, String message, CoreException e) {
		return new BazaarException(new BzrStatus(IStatus.ERROR, e.getStatus().getCode(), message, e));
	}

	/*
	 * Static helper methods for creating exceptions
	 */
	public static BazaarException wrapException(Exception e) {
		Throwable t = e;
		if (e instanceof InvocationTargetException) {
			Throwable target = ((InvocationTargetException) e).getTargetException();
			if (target instanceof BazaarException) {
				return (BazaarException) target;
			}
			t = target;
		}
		return new BazaarException(new BzrStatus(IStatus.ERROR, UNABLE, t.getMessage() != null ? t.getMessage() : "", t)); //$NON-NLS-1$
	}

	public static BazaarException wrapException(CoreException e) {
		if (e instanceof BazaarException) {
			return (BazaarException) e;
		}
		return new BazaarException(e);
	}

	public static BazaarException wrapException(BazaarClientException e) {
		String msg = CoreTexts.Unexpected_error;
		if (e.isCommandLine())
			msg = CoreTexts.bind(CoreTexts.Command_error, e.getCommand());
		return new BazaarException(BzrStatus.ERROR, UNABLE, msg, e);
	}

	public static BazaarException wrapException(BazaarClientException e, IResource resource) {
		String msg = CoreTexts.Unexpected_error;
		if (e.isCommandLine())
			msg = CoreTexts.bind(CoreTexts.Command_error, e.getCommand());
		return new BazaarException(new BzrStatus(BzrStatus.ERROR, UNABLE, msg, e, resource));
	}

	/**
	 * @param aCheckedException
	 * @return
	 */
	public static RuntimeException makeUnChecked(CoreException e) {
		return new RuntimeException(e);
	}

	@Override
	public Throwable getCause() {
		return getStatus().getException();
	}

	@Override
	public String getLocalizedMessage() {
		return getMessage();
	}

	@Override
	public String getMessage() {
		return message;
	}

}
