/**
 * LICENCSE + COPYRIGHT
 * Copyright (c) 2004-2006 by Leif Frenzel - see http://leiffrenzel.de
 */
package org.vcs.bazaar.eclipse.internal.core.model;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarInfo;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrMissingRevisions;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.core.repository.BranchLocationManager;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * <p>
 * Implementation of a local branch, which corresponds to a folder on the local disk (.bzr) 
 * </p>
 * 
 * @author Guillermo Gonzalez
 * @basedOn LocalDarcsRL by Leif Frenzel
 * 
 */
public class LocalBranch extends BzrBranch implements IBzrLocalBranch {

	private static final long serialVersionUID = -6363525810607144655L;
	private final URI location;


	public LocalBranch(final URI location) {
		this.location = location;
		// make sure the URI scheme is "file"
		assert ("file".equals(getLocation().getScheme()));
	}
	

	public boolean isLocal() {
		return true;
	}

	public IProject getProject() {
		return null;
	}
	
	public URI getLocation() {
		return location;
	}
	
	public File getFile() {
		return getPath().toFile();
	}
	
	@Override
	public IPath getPath() {
		return new Path(location.toString());
	}
	
	public String getNick() {
		try {
			IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			client.setWorkDir(getFile());
			return client.nick(null);
		} catch (BazaarException e) {
			throw BazaarException.makeUnChecked(e);
		} catch (BazaarClientException e) {
			throw BazaarClientException.makeUnChecked(e);
		}
	}
	
	public List<IBzrBranch> getPullLocations() throws BazaarException {
		List<IBzrBranch> locations = new ArrayList<IBzrBranch>();
		IBzrBranch parent = getParent();
		if(parent != null)
			locations.add(parent);
		locations.addAll(BranchLocationManager.getInstance().getAllBranches());
		return locations;
	}

	public List<IBzrBranch> getPushLocations() throws BazaarException {
		List<IBzrBranch> locations = new ArrayList<IBzrBranch>();
		IBzrBranch push = getPushBranch();
		if(push != null)
			locations.add(push);
		for (IBzrBranch branch : BranchLocationManager.getInstance().getAllBranches()) {
			if(branch!=null && branch.getBranchLocation() != null)
				locations.add(branch);
		}
		return locations;
	}

	public IBzrBranch getParent() {
		IBazaarInfo info = null;
		if(info == null) {
			info = getInfo();
		}
		if(info == null || info.getRelatedBranches() == null || info.getRelatedBranches().getParentBranch() == null) 
			return null;
		return BranchFactory.getInstance().findOrCreateFor(info.getRelatedBranches().getParentBranch());
	}
	
	public IBzrBranch getPushBranch() {
		IBazaarInfo info = null;
		if(info == null) {
			info = getInfo();
		}
		if(info == null || info.getRelatedBranches() == null) 
			return null;
		if(info.getRelatedBranches().getPushBranch() != null && !"".equals(info.getRelatedBranches().getPushBranch().trim())) {
			return BranchFactory.getInstance().findOrCreateFor(info.getRelatedBranches().getPushBranch());
		} else {
			return null;
		}
	}
	
	public IBazaarInfo getInfo() {
		try {
			IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			client.setWorkDir(getFile());
			return client.info(getFile());
		} catch (BazaarException e) {
			throw BazaarException.makeUnChecked(e);
		} catch (BazaarClientException e) {
			throw BazaarClientException.makeUnChecked(e);
		}
	}
	
	/**
	 * FIXME: support for multiple project per branch
	 */
	@Override
	public boolean isBound() {
		IBazaarInfo info = getInfo();
		IBazaarInfo.Locations locations = info.getLocations();
		return !(locations.getCheckoutOfBranch() == null && locations.getCheckoutRoot() == null 
				&& locations.getLightCheckoutRoot() == null && locations.getBoundToBranch() == null
				&& locations.getRepositoryCheckoutRoot() == null);
	}

	public IBzrMissingRevisions getMissing(IBzrBranch otherBranch) throws BazaarException {
		return new BzrMissingRevisions(new HashMap<String, List<IBazaarLogMessage>>());
	}

	public IResource getIResource() {
		return null;
	}

	public IBzrResourceStatus getStatus() throws CoreException {
		return null;
	}
	
}