/**
 * LICENCSE + COPYRIGHT
 * Copyright (c) 2004-2006 by Leif Frenzel - see http://leiffrenzel.de
 */
package org.vcs.bazaar.eclipse.internal.core.model;

import java.net.URI;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarInfo;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.core.commands.InfoCommand;
import org.vcs.bazaar.eclipse.core.commands.MetaModifyCommand;
import org.vcs.bazaar.eclipse.core.commands.NickCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrMissingRevisions;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * <p>
 * Implementation of a branch, which corresponds to a project in the workspace. 
 * </p>
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 * 
 */
public class ProjectBranch extends LocalBranch  {

	private static final long serialVersionUID = 6725957483386436257L;
	private IProject project;
	/** keep a cached info and a dirty flag which is set by {@link MetaModifyCommand} */
	private IBazaarInfo info;

	public ProjectBranch(final IProject project) {
		super(null);
		this.project = project;
		// make sure the URI scheme is "file"
		assert ("file".equals(getLocation().getScheme()));
	}

	/**
	 * @return the project
	 */
	@Override
	public IProject getProject() {
		return project;
	}
	
	@Override
	public URI getLocation() {
		return getPath().toFile().toURI();
	}
	
	@Override
	public IPath getPath() {
		return ResourceUtil.findRootLocation(project);
	}
	
	@Override
	public String getNick() {
		try {
			NickCommand cmd = new NickCommand(this.project);
			cmd.run(new NullProgressMonitor());
			return cmd.getNick();
		} catch (BazaarException e) {
			throw BazaarException.makeUnChecked(e);
		}
	}
	
	@Override
	public IBazaarInfo getInfo() {
		try {
			Object isDirty = project.getSessionProperty(IBazaarCoreConstants.DIRTY);
			if(info == null || isDirty == null || ((Boolean)isDirty)) {
				InfoCommand cmd = new InfoCommand(project, false);
				cmd.run(new NullProgressMonitor());
				info = cmd.getInfo();
				project.setSessionProperty(IBazaarCoreConstants.DIRTY, Boolean.FALSE);
			}
		} catch (BazaarException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
			info = null;
		} catch (CoreException e) {
			EclipseBazaarCore.log(e);
		}
		return info;
	}
	
	@Override
	public IBzrMissingRevisions getMissing(IBzrBranch otherBranch) throws BazaarException {
		return new BzrMissingRevisions(calculateMissing(otherBranch));
	}
	
	protected Map<String, List<IBazaarLogMessage>> calculateMissing(IBzrBranch otherBranch) throws BazaarException {
		Map<String, List<IBazaarLogMessage>> result = null;
		IBazaarClient client = null;
		try {
			client = BzrWorkspaceRoot.getProviderFor(project).getBzrWorkspaceRoot().getClient();
			client.getNotificationHandler().disableLog();
			result = client.missing(project.getLocation().toFile(), otherBranch.getBranchLocation());
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			if (client != null)
				client.getNotificationHandler().enableLog();
		}
		return result;
	}
	
	public boolean isProjectBranch() {
		return true;
	}
	
	@Override
	public IResource getIResource() {
		return getProject();
	}
	
	@Override
	public IBzrResourceStatus getStatus() throws CoreException {
		return EclipseBazaarCore.getDefault().getStatusCacheManager().getStatus(getIResource());
	}

}