/**
 * 
 */
package org.vcs.bazaar.eclipse.internal.core.model.local;

import java.io.File;
import java.net.URI;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.commandline.syntax.IRemoveOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.inventory.InventoryCacheManager;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResource;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.BzrResource;

/**
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public abstract class BzrLocalResource extends BzrResource implements IBzrLocalResource {

	private final IResource resource;

	protected BzrLocalResource(final IResource resource, final IBzrBranch branch) {
		super(branch);
		this.resource = resource;
	}
	
	public boolean exists() {
		return getIResource().exists();
	}
	
	public boolean isFolder() {
		return false;
	}

	public IResource getIResource() {
		return resource;
	}
	
	/**
	 * Return the parent of this resource or null if this resource is the
	 * workspaceRoot (or this should be the branch?)
	 * 
	 * @return a handle to the parent of this resource.
	 */
	public IBzrResource getParent() {
		final IResource parent = resource.getParent();
		if (parent != null && parent.getType() == IResource.FOLDER) {
			return new BzrLocalFolder((IFolder) parent, getBranch());
		} else {
			// In the case that this resource is the workspaceroot
			return null;
		}
	}
	
	public IPath getBranchRelativePath() {
		final IPath location = resource.getLocation();
		final IPath branchPath = getBranch().getPath();
		Assert.isTrue(branchPath.isPrefixOf(location));
		final IPath tail = location.removeFirstSegments(branchPath.segmentCount());
		IPath result = new Path("/"); //$NON-NLS-1$
		for (int i = 0; i < tail.segmentCount(); i++) {
			result = result.append(tail.segment(i));
		}
		return result.makeRelative();
	}
	
	public boolean isDirty() {
		boolean result = false;
		if (this instanceof IContainer) {
			try {
				IResource[] ress = ((IContainer) resource).members();
				for (int i = 0; !result && i < ress.length; i++) {
					result = BzrWorkspaceRoot.getBzrResourceFor(ress[i]).isDirty();
					if(result) {
						break;
					}
				}
			} catch (final CoreException cex) {
				String msg = "Problem checking dirty state"; //$NON-NLS-1$
				EclipseBazaarCore.log(msg, cex);
			}
		} else if (resource instanceof IFile) {
			result = BzrWorkspaceRoot.getBzrFileFor(resource).isDirty();
		}
		return result;
	}
	
	public void delete() throws BazaarException {
		try {
			getBzrWorkspaceRoot().getClient().remove(new File[] { getIResource().getLocation().toFile() }, IRemoveOptions.FORCE);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e, getIResource());
		}
	}
	
	public BzrWorkspaceRoot getBzrWorkspaceRoot() {
		final BazaarTeamProvider provider = (BazaarTeamProvider) RepositoryProvider.getProvider(resource.getProject(), EclipseBazaarCore.getProviderID());
		if (provider == null) {
			return null;
		}
		return provider.getBzrWorkspaceRoot();
	}

	public void move(IPath destination) throws BazaarException {
		try {
			getBzrWorkspaceRoot().getClient().move(getIResource().getLocation().toFile(), destination.toFile());
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e, getIResource());
		}
	}

	public boolean isManaged() throws BazaarException {
		try {
			if(isProjectAndBranchRoot()) {
				return true;
			}
			final InventoryCacheManager inventoryCache = EclipseBazaarCore.getDefault().getInventoryCacheManager();
			return inventoryCache.isInInventory(getIResource());
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		}
	}
	
	private boolean isProjectAndBranchRoot() {
		return getIResource().getType() == IResource.PROJECT && getIResource().getLocation().equals(getBranch().getPath());
	}

	public void unmanage(IProgressMonitor monitor) throws BazaarException {
		try {
			getBzrWorkspaceRoot().getClient().remove(new File[] { getIResource().getLocation().toFile() }, IRemoveOptions.KEEP);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e, getIResource());
		}
	}
	
	public File getFile() {
		return getIResource().getLocation().toFile();
	}
	
	public boolean isProject() {
		return getIResource().getType() == IResource.PROJECT;
	}
	
	public URI getLocation() {
		return getIResource().getLocationURI();
	}
	
	public String getName() {
		return getIResource().getName();
	}
	
	public IBzrResourceStatus getStatus() throws CoreException {
		return EclipseBazaarCore.getDefault().getStatusCacheManager().getStatus(this.getIResource());
	}
	
	@Override
	public int hashCode() {
		return getIResource().hashCode();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		if (obj instanceof IBzrLocalResource) {
			final IBzrLocalResource bzrRes = (IBzrLocalResource) obj;
			return this.getIResource().equals(bzrRes.getIResource()) && this.getBranch().equals(bzrRes.getBranch());
		}
		return super.equals(obj);
	}
	
	public boolean isModified(IProgressMonitor monitor) throws BazaarException {
		throw new BazaarException(IStatus.INFO, IStatus.INFO, "Not Implemented"); //$NON-NLS-N$
	}
	
}
