package org.vcs.bazaar.eclipse.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.console.BzrOutputConsole;
import org.vcs.bazaar.eclipse.ui.decorator.BazaarLightweightDecorator;
import org.vcs.bazaar.eclipse.ui.dialogs.UpdateInfoDialog;
import org.vcs.bazaar.eclipse.ui.markers.NewFileMarker;
import org.vcs.bazaar.eclipse.ui.preferences.PreferenceStoreChangeListener;
import org.vcs.bazaar.eclipse.ui.team.BazaarUtilities;
import org.vcs.bazaar.eclipse.ui.team.RefreshExecutionListener;

/**
 * The main plugin class to be used in the desktop.
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 */
public class EclipseBazaarUI extends AbstractUIPlugin {

	private static final String LOGVIEW_ID = "org.vcs.bazaar.eclipse.ui.views.LogView";
	private static final String MISSINGVIEW_ID = "org.vcs.bazaar.eclipse.ui.views.MissingView";

	/**
	 * The shared instance.
	 */
	private static EclipseBazaarUI plugin;
	
	private BzrOutputConsole console;
	private final IPropertyChangeListener preferenceChangeListener = new PreferenceStoreChangeListener();
	private BundleListener bundleListener;
	private RefreshExecutionListener refreshExecListener;
	private final NewFileMarker newFileMarker = new NewFileMarker();
	
	/**
	 * The constructor.
	 */
	public EclipseBazaarUI() {
		plugin = this;		
	}

    /**
     * This method is called upon plug-in activation.
     */
    @Override
    public void start(BundleContext context) throws Exception {
		bundleListener = new BundleListener() {
			public void bundleChanged(BundleEvent event) {
                if (event.getBundle() == getBundle() 
                		&& event.getType() == BundleEvent.STARTED 
                		&& getBundle().getState() == Bundle.ACTIVE) {
                    try {
                    	String version = (String)getBundle().getHeaders().get(UITexts.Dependency_Version);
            			if(version != null && !version.equals(EclipseBazaarUI.this.getPluginPreferences().getString(UITexts.Dependency_Version))) {
            				EclipseBazaarUI.this.getPluginPreferences().setValue(UITexts.Dependency_Version, version);
            				getStandardDisplay().asyncExec(new Runnable() {
                        		public void run() {
                        			UpdateInfoDialog dialog = new UpdateInfoDialog(getStandardDisplay().getActiveShell());
        							dialog.setBlockOnOpen(true);
        							dialog.open();
                        		}
                        	});	
            			}
						EclipseBazaarCore.getDefault().checkClient();
					} catch (BazaarException e) {
						getStandardDisplay().asyncExec(new Runnable() {
                    		public void run() {
                    			BazaarUtilities.configureExecutable();
                    		}
                    	});
						
					}
                }
			}
		};
		context.addBundleListener(bundleListener);
		super.start(context);
		getPreferenceStore().addPropertyChangeListener(preferenceChangeListener);
		if(!getWorkbench().getDecoratorManager().getEnabled(BazaarLightweightDecorator.class.getName())) {
			getWorkbench().getDecoratorManager().setEnabled(BazaarLightweightDecorator.class.getName(), true);
		}
		refreshExecListener = new RefreshExecutionListener();
		refreshExecListener.addTo((ICommandService) this.getWorkbench().getService(ICommandService.class));
		EclipseBazaarCore.addResourceStateListener(newFileMarker);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	@Override
    public void stop(BundleContext context) throws Exception {
		refreshExecListener.removeFrom((ICommandService) this.getWorkbench().getService(ICommandService.class));
		getPreferenceStore().removePropertyChangeListener(preferenceChangeListener);
		getConsole().shutdown();
		context.removeBundleListener(bundleListener);
		EclipseBazaarCore.removeResourceStateListener(newFileMarker);
		super.stop(context);
		plugin = null;
	}

	/**
	 * @return the shared instance.
	 */
	public static EclipseBazaarUI getDefault() {
		return plugin;
	}

	public static String getPluginId() {
		return getDefault().getBundle().getSymbolicName();
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path.
	 * 
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(getPluginId(), path);
	}
	
	/**
	 * Convenience method to get the currently active workbench page. Note that
	 * the active page may not be the one that the user perceives as active in
	 * some situations so this method of obtaining the activate page should only
	 * be used if no other method is available.
	 * 
	 * @return the active workbench page
	 */
	public static IWorkbenchPage getActivePage() {
		final IWorkbenchWindow window = getDefault().getWorkbench().getActiveWorkbenchWindow();
		if (window == null) {
            return null;
        }
		return window.getActivePage();
	}
	
	/**
	 * Creates a busy cursor and runs the specified runnable.
	 * May be called from a non-UI thread.
	 * 
	 * @param parent the parent Shell for the dialog
	 * @param runnable the runnable
	 * 
	 * @exception InvocationTargetException when an exception is thrown from the runnable
	 * @exception InterruptedException when the progress monitor is canceled
	 */	
	public static void runWithProgress(Shell parent, final IRunnableWithProgress runnable) throws InvocationTargetException, InterruptedException {
		boolean createdShell = false;
		try {
			if (parent == null || parent.isDisposed()) {
				final Display display = getStandardDisplay();
				if (display == null) {
					// cannot provide progress (not in UI thread)
					runnable.run(new NullProgressMonitor());
					return;
				}
				// get the active shell or a suitable top-level shell
				parent = display.getActiveShell();
				if (parent == null) {
					parent = new Shell(display);
					createdShell = true;
				}
			}
			// pop up progress dialog after a short delay
			final Exception[] holder = new Exception[1];
			BusyIndicator.showWhile(parent.getDisplay(), new Runnable() {
				public void run() {
					try {
						runnable.run(new NullProgressMonitor());
					} catch (InvocationTargetException e) {
						holder[0] = e;
					} catch (InterruptedException e) {
						holder[0] = e;
					}
				}
			});
			if (holder[0] != null) {
				if (holder[0] instanceof InvocationTargetException) {
					throw (InvocationTargetException) holder[0];
				} else {
					throw (InterruptedException) holder[0];
				}
			}
			//new TimeoutProgressMonitorDialog(parent, TIMEOUT).run(true /*fork*/, cancelable, runnable);
		} finally {
			if (createdShell && parent != null) {
                parent.dispose();
            }
		}
	}	

	// logging and tracing
	// ////////////////////

	public static void log(final String message, final int severity) {
		final Status status = new Status(severity, getPluginId(), IStatus.OK, message, null);
		getDefault().getLog().log(status);
	}

	public static void log(final String message, final CoreException ex) {		
		final Status status = new Status(ex.getStatus().getSeverity(), getPluginId(), ex.getStatus().getCode(), message, ex);
		getDefault().getLog().log(status);
	}

	public static void log(final String message, final Throwable thr) {
		final Status status = new Status(IStatus.ERROR, getPluginId(), IStatus.OK, message, thr);
		getDefault().getLog().log(status);
	}

	public static void log(CoreException ex) {
		getDefault().getLog().log(ex.getStatus());
	}

	public static boolean isTracing(final String optionId) {
		final String option = getPluginId() + "/trace/" + optionId; //$NON-NLS-1$
		final String value = Platform.getDebugOption(option);
		return "true".equals(value); //$NON-NLS-1$
	}
	
	public static void traceException(final String optionId, final Exception ex) {
		if (isTracing(optionId)) {
			ex.printStackTrace();
		}
	}

	public static Color getInfoBgColor() {
		return getStandardDisplay().getSystemColor(SWT.COLOR_INFO_BACKGROUND);
	}

	public static Color getBgColor() {
		return getStandardDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND);
	}

	public static Display getStandardDisplay() {
		Display display = PlatformUI.getWorkbench().getDisplay();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	public synchronized BzrOutputConsole getConsole() {
		if(console == null){
			console = new BzrOutputConsole();
		}
		return console;
	}

	public static String getLogViewId() {
		return LOGVIEW_ID;
	}

	public static String getMissingViewId() {
		return MISSINGVIEW_ID;
	}
}
