/**
 * org.vcs.bazaar.eclipse (c) Vectrace Jan 31, 2006
 * Created by zingo
 */
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.IJobStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.AddCommand;
import org.vcs.bazaar.eclipse.core.commands.CommitCommand;
import org.vcs.bazaar.eclipse.core.commands.StatusCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.utils.BzrJobStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.dialogs.CommitDialog;
import org.vcs.bazaar.eclipse.ui.operations.PushOperation;

/**
 * @author zingo
 * @modifiedBy Guillermo Gonzalez
 * 
 * TODO: needs revision & test (maybe split into Operation and Action)
 */
public class CommitAction extends WorkbenchAction {

	private CommitDialog commitDialog = null;
	/**
	 * The action has been activated. The argument of the method represents the
	 * 'real' action sitting in the workbench UI.
	 * 
	 * @throws BazaarException
	 * @throws InterruptedException
	 * @throws InvocationTargetException
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */

	public void execute(final IAction action) throws BazaarException, InvocationTargetException, InterruptedException {
		if(commitDialog == null) {
			commitDialog = new CommitDialog(getShell());
		}
		UIJob job = new UIJob(this.getShell().getDisplay(), "Getting commitable resources...") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				monitor.beginTask("Getting commitable resources..", 100);
				try {
					CommitableResources resourceStauses = createCommitableResources(monitor, getEnclosingProjectsForSelectedResources());
					CommitableResources selectedResource = createCommitableResources(monitor, getSelectedResources());
					monitor.worked(1000);
					if (resourceStauses.statusByProject.size() == 0) {
						return BzrJobStatus.INFO(this, "Nothing to commit");
					}
					List<CommitableResources> results = new ArrayList<CommitableResources>(2);
					results.add(resourceStauses);
					results.add(selectedResource);
					return new BzrJobStatus<List<CommitableResources>>(IJobStatus.OK, this, results, "Commitable resources calculated");
				} catch (Exception e) {
					return UIJob.errorStatus(e);
				} finally {
					monitor.done();
				}
			}
		};
		job.setUser(true);
		job.setPriority(UIJob.LONG);
		job.addJobChangeListener(new JobChangeAdapter() {
			public void done(IJobChangeEvent event) {
				IStatus status = event.getResult();
				if(status instanceof BzrJobStatus) {
					if(status.getSeverity() == IStatus.INFO)
						MessageDialog.openInformation(getParentShell(), "Information", status.getMessage());
					else if(status.getSeverity() == IStatus.OK) {
						List<CommitableResources> result = ((BzrJobStatus<List<CommitableResources>>)status).getResult();
						int exit = commitDialog.open(result.get(0), result.get(1), true);
						if (exit == Window.OK && commitDialog.getCommitMessage() != null) {
							try {
								handleSelected(result.get(0));
							} catch (Exception e) {
								CommitAction.this.handle(e);
							} 
						}
					}
				}
				event.getJob().removeJobChangeListener(this);
			}
		});
		job.schedule();
	}
	
	/**
	 * Given an IResource[] returns the list of commitable resources as a list of IBzrResourceStatus.
	 *  
	 * @param monitor a IProgressMonitor
	 * @param resources an array of resources to check for inclusion in the commitable list
	 * @return List<IBzrResourceStatus> 
	 * @throws BazaarException
	 */
	protected static List<IBzrResourceStatus> getCommitableResources(IProgressMonitor monitor, IResource[] resources) throws BazaarException {
		return createCommitableResources(monitor, resources).getAllStatuses();
	}

	static CommitableResources createCommitableResources(IProgressMonitor monitor, IResource[] resources) throws BazaarException {
		CommitableResources commitableResource = new CommitableResources();
		Map<IProject, List<IResource>> map = getProjectMapping(resources);
		for (IProject project : map.keySet()) {
			List<IResource> filteredResources = map.get(project);
			IBzrLocalResource[] bzrResources = null;
			if(filteredResources != null && filteredResources.size() > 0) {
				bzrResources = BzrWorkspaceRoot.getBzrResourcesFor(filteredResources.toArray(new IResource[filteredResources.size()]));
			} else {
				bzrResources = BzrWorkspaceRoot.getBzrResourcesFor(new IResource[]{project});
			}
			StatusCommand cmd = new StatusCommand(project, bzrResources, true);
			cmd.run(monitor);
			IBzrResourceStatus[] result = cmd.getStatus();
			if (result.length > 0) {
				commitableResource.add(project, Arrays.asList(result));
			}
			commitableResource.addPendingMerge(project, cmd.getPendingMerges());
			
		}
		return commitableResource;
	}
	
	private void handleSelected(final CommitableResources commitableResources) throws BazaarException, InvocationTargetException, InterruptedException {

		final List<IResource> filesToAdd = commitDialog.getFilesToAdd();
		final List<IBzrLocalResource> filesToCommit = commitDialog.getFilesToCommit();
		final String commitText = commitDialog.getCommitMessage();
		final boolean pushAfterCommit = commitDialog.isPushAfterCommit();
		final List<Option> options = commitDialog.getOptions();
		Job job = new Job("Running commit") {
			@Override
			public IStatus run(IProgressMonitor monitor) {
				try {
					monitor.beginTask(CoreTexts.CommitCommand_commit, 1000); //$NON-NLS-1$
					// first we execute a 'bzr add'
					if (filesToAdd != null && filesToAdd.size() > 0) {
						// TODO: maybe this should be refactored to a
						// AddOperation (because it's also used in AddAction)
						Map<BazaarTeamProvider, List<IResource>> map = getProviderMapping(filesToAdd.toArray(new IResource[0]));
						for (BazaarTeamProvider provider : map.keySet()) {
							List<IResource> resources = map.get(provider);
							AddCommand op = new AddCommand(provider.getBzrWorkspaceRoot(), resources);
							op.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
						}
					}
					// now the commit
					if (filesToCommit != null && filesToCommit.size() > 0) {
						CommitCommand commitCmd = null;
						if(commitableResources.getAllPendingMerges().size() > 0) {
							// commit all modified files (including unknowns)
							// currently we do this, by committing without passing any file to the commit cmd
							commitCmd = new CommitCommand(filesToCommit, commitText, options, true);
						} else {
							commitCmd = new CommitCommand(filesToCommit, commitText, options);
						}
						commitCmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
					}
					// handle push after commit
					if(pushAfterCommit && filesToCommit != null && filesToCommit.size() > 0) {
						Set<IProject> projects = getProjectMapping(filesToCommit);
						for (IProject project: projects) {
							new PushOperation(project).run(EclipseBazaarCore.subProgressMonitorFor(monitor));
						}
					}
				} catch (CoreException e) {
					EclipseBazaarUI.log(e.getMessage(), e);
					return e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(e.getMessage(), e);
					return BzrJobStatus.ERROR(this, e, "Error while executing commit");
				} finally {
					monitor.done();
				}
				return BzrJobStatus.OK(this, filesToCommit,"New revision committed");
			}
		};
		job.setUser(true);
		job.setPriority(UIJob.LONG);
		job.schedule();
	}
	
	private Set<IProject> getProjectMapping(List<IBzrLocalResource> filesToCommit) {
		Set<IProject> projects = new HashSet<IProject>();
		for (IBzrLocalResource bzrResource : filesToCommit) {
			projects.add(bzrResource.getIResource().getProject());
		}
		return projects;
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}

	public static class CommitableResources {
		
		List<IBazaarLogMessage> pendingMerges;
		List<IBzrResourceStatus> statuses;
		Map<IProject, List<IBzrResourceStatus>> statusByProject;
		private HashMap<IProject, List<IBazaarLogMessage>> pendingMergesByProject;
		
		public CommitableResources() {
			statusByProject = new HashMap<IProject, List<IBzrResourceStatus>>();
			pendingMergesByProject = new HashMap<IProject, List<IBazaarLogMessage>>();
		}
		
		public void addPendingMerge(IProject project, List<IBazaarLogMessage> pendingMerges) {
			pendingMergesByProject.put(project, pendingMerges);
		}

		public void add(IProject project, List<IBzrResourceStatus> resourceStatus) {
			statusByProject.put(project, resourceStatus);
		}

		public List<IBzrResourceStatus> getAllStatuses() {
			if(statuses == null) {
				statuses = new ArrayList<IBzrResourceStatus>();
				for (IProject project : statusByProject.keySet()) {
					statuses.addAll(statusByProject.get(project));
				}
			}
			return statuses;
		}
		
		public List<IBazaarLogMessage> getAllPendingMerges() {
			if(pendingMerges == null) {
				pendingMerges = new ArrayList<IBazaarLogMessage>();
				for (IProject project : pendingMergesByProject.keySet()) {
					pendingMerges.addAll(pendingMergesByProject.get(project));
				}
			}
			return pendingMerges;
		}
	}
}
