/**
 * LICENCSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.actions;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.team.core.TeamException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;

/**
 * <p>
 * Abstract class that represent an Action executed in the workspace
 * </p>
 * 
 * @author Guillermo Gonzalez
 * 
 * <p>
 * <b>Note:</b> The idea of this abstract class is taken from Subclipse plugin
 * </p>
 */
public abstract class WorkspaceAction extends BzrAction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.internal.ui.actions.TeamAction#isEnabled()
	 */
	@Override
	protected boolean isEnabled() throws TeamException {
		IResource[] resources = getSelectedResources();
		if(!EclipseBazaarCore.getDefault().isClientAvailable()) {
			return false;
		}
		if (resources.length == 0) {
			// don't enable if there no selection
			return false;
		}
		if (!isEnabledForMultipleResources() && resources.length != 1) {
			return false;
		}
		if (!isEnabledForMultipleEclosingProjects() && getEnclosingProjectsForSelectedResources().length != 1) {
			return false;
		}

		// we should check each resource in the selection?. for the moment we do
		for (IResource resource : resources) {
			// we don't handle linked resources
			if (!shouldHandle(resource)) {
				return false;
			}

			if (!isEnabledByType(resource)) {
				return false;
			}

			// ensure that resource management state matches what the action
			// requires
			IBzrLocalResource bzrResource = BzrWorkspaceRoot.getBzrResourceFor(resource);
			if (!isEnabledForBzrResource(bzrResource)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * tell if is enbaled filtering by type (IFolder, IFile) subclasses should
	 * override this method
	 * 
	 * @param resource
	 * @return boolean
	 */
	protected boolean isEnabledByType(IResource resource) {
		// enabled for all types
		return true;
	}

	/**
	 * Tell if this action should be available for multiple resource selection
	 * 
	 * @return boolean
	 */
	protected boolean isEnabledForMultipleResources() {
		return true;
	}

	/**
	 * Tell if this action should be available for multiple project selection
	 * 
	 * @return boolean
	 */
	protected boolean isEnabledForMultipleEclosingProjects() {
		return false;
	}

	protected boolean isEnabledForManagedResources() {
		return true;
	}

	protected boolean isEnabledForUnmanagedResources() {
		return true;
	}

	protected boolean isEnabledForAddedResources() {
		return true;
	}

	protected boolean isEnabledForRenamedResources() {
		return true;
	}
	
	protected boolean isEnabledForModifiedResources() {
		return true;
	}
	
	protected boolean isEnabledForUnmodifiedResources() {
		return true;
	}
	
	protected boolean isEnabledForConflictedResources() {
		return false;
	}

	private boolean isEnabledForBzrResource(IBzrLocalResource bzrResource) {
		boolean managed = false;
		boolean added = false;
		boolean renamed = false;
		boolean modified = false;
		boolean conflicts = false;
		
		try {
			managed = bzrResource.isManaged();
			if (!managed && !isEnabledForUnmanagedResources())
				return false;
		} catch (CoreException e) {
			return false;
		}
		IBzrResourceStatus status = null;
		try {
			status = bzrResource.getStatus();
		} catch (CoreException e) {
			return false;
		}
		if (status != null && !status.isIgnored()) {
			renamed = status.isRenamed();
			added = status.isAdded();
			managed = !status.isUnknown();
			modified = status.isHasConflicts() || status.isKindChange() || status.isModified() || status.isRenamed() || status.isUnknown() ||
					status.isXBitChanged() || status.isAdded() || status.isDeleted();
			conflicts = status.isHasConflicts();
		}
		if(conflicts && isEnabledForConflictedResources()) {
			return true;
		}
		if (managed && !isEnabledForManagedResources())
			return false;
		if (!managed && !isEnabledForUnmanagedResources())
			return false;
		if (renamed && !isEnabledForRenamedResources()) {
			return false;
		} else if (added && !isEnabledForAddedResources()) {
			return false;
		} 
		if(modified && !isEnabledForModifiedResources()) {
			return false;
		} else if(!modified && !isEnabledForUnmodifiedResources()) {
			return false;
		}
		return true;
	}

}
