/*******************************************************************************
 * Copyright (c) ?
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Charles O'Farrell - implementation (based on subclipse)
 *     StefanC           - jobs framework, code cleenup
 *******************************************************************************/

package org.vcs.bazaar.eclipse.ui.annotaions;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.text.IInformationControl;
import org.eclipse.jface.text.IInformationControlCreator;
import org.eclipse.jface.text.revisions.Revision;
import org.eclipse.jface.text.revisions.RevisionInformation;
import org.eclipse.jface.text.source.LineRange;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.ui.TeamOperation;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.texteditor.AbstractDecoratedTextEditor;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.AnnotateCommand;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileRevision;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFile;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.team.BzrQuickDiffReferenceProvider;

/**
 * <p>Annotate operation</p> 
 * 
 * <p><i>based on MercurialEclipse</i></p>
 *  
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class ShowAnnotationOperation extends TeamOperation {
	
	private final class BazaarAnnotateRevision extends Revision {
		private final CommitterColors colors;

		private final IBazaarLogMessage log;

		private final AnnotateBlock block;

		private BazaarAnnotateRevision(CommitterColors colors, IBazaarLogMessage log, AnnotateBlock block) {
			this.colors = colors;
			this.log = log;
			this.block = block;
		}

		@Override
		public Object getHoverInfo() {
			String date = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT).format(getDate(block.getDate()));
			return block.getUser()
					+ " " + block.getRevision() + " " + date + "\n\n" + //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					(log != null ? log.getMessage() : ""); //$NON-NLS-1$
		}

		@Override
		public String getAuthor() {
			return block.getUser();
		}

		@Override
		public String getId() {
			return block.getRevision();
		}

		private Date getDate(String date) {
			String year = block.getDate().substring(0, 4);
			String month = block.getDate().substring(5, 6);
			String day = block.getDate().substring(7, 8);
			Calendar cal = Calendar.getInstance();
			cal.set(Integer.valueOf(year), Integer.valueOf(month), Integer.valueOf(day));
			return cal.getTime();	
		}
		
		@Override
		public Date getDate() {
//			return getDate(block.getDate());
			return log.getDate();
		}

		@Override
		public RGB getColor() {
			return colors.getCommitterRGB(getAuthor());
		}
	}

	private static final String DEFAULT_TEXT_EDITOR_ID = EditorsUI.DEFAULT_TEXT_EDITOR_ID;
	private final IBzrLocalFile remoteFile;

	public ShowAnnotationOperation(IWorkbenchPart part, IBzrLocalFile remoteFile) {
		super(part);
		this.remoteFile = remoteFile;
	}

	public void run(IProgressMonitor monitor) throws InvocationTargetException,
			InterruptedException {
		monitor.beginTask(null, 100);

		try {
			AnnotateCommand cmd = new AnnotateCommand(remoteFile);
			cmd.run(new NullProgressMonitor());
			final AnnotateBlocks annotateBlocks = new AnnotateBlocks(cmd.getAnnotation());

			// this is not needed if there is no live annotate
			final RevisionInformation information = createRevisionInformation(annotateBlocks, monitor);

			// We aren't running from a UI thread
			new UIJob("Bazaar Annotate") {
				@Override
				public IStatus runInUIThread(IProgressMonitor monitor) {
					monitor.beginTask("Applying Annotation",
							IProgressMonitor.UNKNOWN);
					final AbstractDecoratedTextEditor editor = getEditor();
					if (editor != null) {
						editor.showRevisionInformation(information, BzrQuickDiffReferenceProvider.ID);

					}
					monitor.done();
					return Status.OK_STATUS;
				}
			}.schedule();

		} catch (CoreException e) {
			EclipseBazaarUI.log(e);
		} finally {
			monitor.done();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.TeamOperation#getGotoAction()
	 */
	@Override
	protected IAction getGotoAction() {
		return super.getGotoAction();
	}

	private AbstractDecoratedTextEditor getEditor() {
		final IWorkbench workbench = PlatformUI.getWorkbench();
		final IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IEditorReference[] references = window.getActivePage()
				.getEditorReferences();
		IResource resource = remoteFile.getIResource();
		if (resource == null) {
			return null;
		}

		for (int i = 0; i < references.length; i++) {
			IEditorReference reference = references[i];
			try {
				if (resource != null
						&& resource.equals(reference.getEditorInput()
								.getAdapter(IFile.class))) {
					IEditorPart editor = reference.getEditor(false);
					if (editor instanceof AbstractDecoratedTextEditor) {
						return (AbstractDecoratedTextEditor) editor;
					}
					// editor opened is not a text editor - reopen file using
					// the
					// defualt text editor
					IEditorPart part = getPart().getSite().getPage()
							.openEditor(new FileEditorInput((IFile) resource),
									DEFAULT_TEXT_EDITOR_ID, true,
									IWorkbenchPage.MATCH_NONE);
					if (part != null
							&& part instanceof AbstractDecoratedTextEditor) {
						return (AbstractDecoratedTextEditor) part;
					}
				}
			} catch (PartInitException e) {
				// ignore
			}
		}

		// no existing editor references found, try to open a new editor for the
		// file
		if (resource instanceof IFile) {
			try {
				IEditorDescriptor descrptr = IDE
						.getEditorDescriptor((IFile) resource);
				// try to open the associated editor only if its an internal
				// editor
				if (descrptr.isInternal()) {
					IEditorPart part = IDE.openEditor(getPart().getSite()
							.getPage(), (IFile) resource);
					if (part instanceof AbstractDecoratedTextEditor) {
						return (AbstractDecoratedTextEditor) part;
					}

					// editor opened is not a text editor - close it
					getPart().getSite().getPage().closeEditor(part, false);
				}
				// open file in default text editor
				IEditorPart part = IDE.openEditor(
						getPart().getSite().getPage(), (IFile) resource,
						DEFAULT_TEXT_EDITOR_ID);
				if (part != null && part instanceof AbstractDecoratedTextEditor) {
					return (AbstractDecoratedTextEditor) part;
				}

			} catch (PartInitException e) {
			}
		}

		return null;
	}

	@SuppressWarnings("unchecked")
	private RevisionInformation createRevisionInformation(final AnnotateBlocks annotateBlocks, IProgressMonitor monitor) throws BazaarException {
		Map<String, BazaarFileRevision> logEntriesByRevision = new HashMap<String, BazaarFileRevision>();
		BazaarFileHistory fileHistory = new BazaarFileHistory((IFile)remoteFile.getIResource());
		IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
		try {
			fileHistory.refresh(subMonitor);
			logEntriesByRevision.putAll(getLogEntriesByRevision(fileHistory.getFileRevisions()));
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			subMonitor.done();
		}

		RevisionInformation info = new RevisionInformation();

		try {
			Class infoClass = info.getClass();
			Class[] paramTypes = { IInformationControlCreator.class };
			Method method1 = infoClass.getMethod("setHoverControlCreator", paramTypes);
			Method method2 = infoClass.getMethod("setInformationPresenterControlCreator", paramTypes);

			final class AnnotationControlCreator implements IInformationControlCreator {
				private final String statusFieldText;

				public AnnotationControlCreator(String statusFieldText) {
					this.statusFieldText = statusFieldText;
				}

				public IInformationControl createInformationControl(Shell parent) {
					return new SourceViewerInformationControl(parent, SWT.TOOL, SWT.NONE, JFaceResources.DEFAULT_FONT, statusFieldText);
				}
			}

			method1.invoke(info, new Object[] { new AnnotationControlCreator("Press F2 for focus.") });
			method2.invoke(info, new Object[] { new AnnotationControlCreator(null) });

		} catch (Exception e) {
			// ignore
		}

		final CommitterColors colors = CommitterColors.getDefault();

		HashMap<String, Revision> sets = new HashMap<String, Revision>();

		for (final AnnotateBlock block : annotateBlocks.getAnnotateBlocks()) {
			// TODO: handle nested revision (aka merges)
			BazaarFileRevision bzrRevision = logEntriesByRevision.get(block.getRevision());
			if(bzrRevision == null)
				continue;
			final IBazaarLogMessage logEntry = bzrRevision.getIBazaarLogMessage();

			Revision revision = sets.get(block.getRevision());
			if (revision == null) {
				revision = new BazaarAnnotateRevision(colors, logEntry, block);
				sets.put(block.getRevision(), revision);
				info.addRevision(revision);
			}
			revision.addRange(new LineRange(block.getStartLine(), block.getEndLine() - block.getStartLine() + 1));
		}
		return info;
	}

	private Map<String, BazaarFileRevision> getLogEntriesByRevision(IFileRevision[] fileRevisions) {
		Map<String, BazaarFileRevision> logEntriesByRevision = new HashMap<String, BazaarFileRevision>(fileRevisions.length); 
		for (IFileRevision fileRev : fileRevisions) {
			BazaarFileRevision bzrFileRev = (BazaarFileRevision)fileRev;
			if(bzrFileRev.getChildren() != null && bzrFileRev.getChildren().length > 0) {
				logEntriesByRevision.putAll(getLogEntriesByRevision(bzrFileRev.getChildren()));
			}
			logEntriesByRevision.put(fileRev.getContentIdentifier(), (BazaarFileRevision)fileRev);
		}
		return logEntriesByRevision;
	}
}
