/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.compare;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.IEditableContent;
import org.eclipse.compare.IResourceProvider;
import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.DiffNode;
import org.eclipse.compare.structuremergeviewer.ICompareInput;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.ui.synchronize.SaveableCompareEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * Compare editor input that is able to save the left side of the compare
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public class SaveableBazaarCompareEditorInput extends SaveableCompareEditorInput {
	
	private static class MyDiffNode extends DiffNode {
		public MyDiffNode(int kind, ITypedElement left, ITypedElement right) {
			super(kind, null, left, right);
		}
		
		public MyDiffNode(ITypedElement left, ResourceNode right) {
			super(left, right);
		}

		@Override
		protected void fireChange() {
			super.fireChange();
		}
	}
	
	private BazaarResourceNode left;
	private ResourceNode right;
	
	/**
	 * This constructor use a default configuration with both side read-only
	 * @param resource
	 * @param left
	 * @param right
	 */
	public SaveableBazaarCompareEditorInput(IResource resource, BazaarResourceNode left, ResourceNode right, IWorkbenchPage page) {
		this(new CompareConfiguration(), resource, left, right, page);
	}
	
	public SaveableBazaarCompareEditorInput(CompareConfiguration configuration,
			IResource resource, BazaarResourceNode left, ResourceNode right, IWorkbenchPage page) {
		super(configuration, page);
		this.left = left;
		this.right = right;
		setTitle(resource.getName());
		configuration.setLeftLabel(left.getName());
		configuration.setRightLabel(right.getName());
	}

	@Override
	protected void fireInputChange() {
		((MyDiffNode)getCompareInput()).fireChange();
	}

	@Override
	protected ICompareInput prepareCompareInput(IProgressMonitor monitor)
			throws InvocationTargetException, InterruptedException {
		ICompareInput input = createCompareInput();
		getCompareConfiguration().setLeftEditable(isLeftEditable(input));
		getCompareConfiguration().setRightEditable(false);
		initLabels(input);
		return input;
		
	}
	
	private ICompareInput createCompareInput() {
		ITypedElement left = createFileElement((IFile)this.left.getResource());
		MyDiffNode node = new MyDiffNode(left,right);
		return node;
	}
	
	private void initLabels(ICompareInput input) {
		CompareConfiguration cc = getCompareConfiguration();
		if (getLeftRevision() != null) {
			String leftLabel = getFileRevisionLabel(getLeftRevision());
			cc.setLeftLabel(leftLabel);
		} else if (getResource(input) != null) {
			String label = UITexts.bind("{0}", new Object[]{ input.getLeft().getName() });
			cc.setLeftLabel(label);
		}
		if (getRightRevision() != null) {
			String rightLabel = getFileRevisionLabel(getRightRevision());
			cc.setRightLabel(rightLabel);
		}
	}
	
	private boolean isLeftEditable(ICompareInput input) {
		Object left = input.getLeft();
		if (left instanceof IEditableContent) {
			return ((IEditableContent) left).isEditable();
		}
		return false;
	}
	
	private String getFileRevisionLabel(IFileRevision fileRevision) {
		return fileRevision.getName();
	}

	protected IFileRevision getLeftRevision() {
		if (left instanceof IFileRevision) {
			return (IFileRevision) left;
		}
		return null;
	}
	
	protected IFileRevision getRightRevision() {
		if (right instanceof IFileRevision) {
			return (IFileRevision) right;
		}
		return null;
	}
	
	private IResource getResource(ICompareInput input) {
		if (getLocalElement() instanceof IResourceProvider) {
			return ((IResourceProvider) getLocalElement()).getResource();
		}
		return null;
	}
	
	public IResource getLocalElement() {
		if (left != null) {
			return (IResource) left.getResource();
		}
		return null;
	}
}
