/*******************************************************************************
 * Copyright (c) 2004, 2006 Subclipse project and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Subclipse project committers - initial API and implementation
 * Modified by:
 *     Guillermo Gonzalez
 ******************************************************************************/
package org.vcs.bazaar.eclipse.ui.console;

import java.io.File;
import java.io.IOException;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.IOConsole;
import org.eclipse.ui.console.IOConsoleOutputStream;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.client.IConsoleListener;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.console.ConsoleDocument.ConsoleLine;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * Console that shows the output of Bazaar commands. It is shown as a page in
 * the generic console view. It supports coloring for message, command, and
 * error lines in addition the font can be configured.
 * 
 * @since 3.0
 */
public class BzrOutputConsole extends IOConsole implements IConsoleListener, IPropertyChangeListener {

	/**
	 * Constant used to define consoles that have the ability to output
	 * bazaar information
	 */
	public static final String BZR_CONSOLE_TYPE = "BZR";

	// streams for each command type - each stream has its own color
	private IOConsoleOutputStream commandStream;
	private IOConsoleOutputStream messageStream;
	private IOConsoleOutputStream errorStream;
	private IOConsoleOutputStream debugStream;

	// preferences for showing the Bazaar console when Bazaar output is provided
	private boolean showOnDebug;
	private boolean showOnMessage;
	private boolean showOnError;

	private ConsoleDocument document;

	/**
	 * Indicates whether the console is visible in the Console view
	 */
	private boolean visible = false;
	/**
	 * Indicates whether the console's streams have been initialized
	 */
	private boolean initialized = false;

	/**
	 * Used to notify this console of lifecycle methods <code>init()</code>
	 * and <code>dispose()</code>.
	 */
	class MyLifecycle implements org.eclipse.ui.console.IConsoleListener {
		public void consolesAdded(IConsole[] consoles) {
			for (IConsole console : consoles) {
				if (console == BzrOutputConsole.this) {
					init();
				}
			}

		}

		public void consolesRemoved(IConsole[] consoles) {
			for (IConsole console : consoles) {
				if (console == BzrOutputConsole.this) {
					ConsolePlugin.getDefault().getConsoleManager().removeConsoleListener(this);
					dispose();
				}
			}
		}
	}

	/**
	 * Constructor initializes preferences and colors but doesn't create the
	 * console page yet.
	 */
	public BzrOutputConsole() {
		super("Bazaar", BZR_CONSOLE_TYPE, EclipseBazaarUI.getImageDescriptor("icons/bzrlogo.png")); //$NON-NLS-1$
		// setup console showing preferences
		showOnMessage = EclipseBazaarUI.getDefault().getPreferenceStore().getBoolean(BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_MESSAGE);
		showOnError = EclipseBazaarUI.getDefault().getPreferenceStore().getBoolean(BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_ERROR);
		showOnDebug = EclipseBazaarUI.getDefault().getPreferenceStore().getBoolean(BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_DEBUG);
		document = new ConsoleDocument();
		EclipseBazaarCore.getDefault().setConsoleListener(BzrOutputConsole.this);
		EclipseBazaarUI.getDefault().getPreferenceStore().addPropertyChangeListener(BzrOutputConsole.this);
	}

	@Override
    protected void init() {
		// Called when console is added to the console view
		super.init();

		initLimitOutput();

		// Ensure that initialization occurs in the ui thread
		EclipseBazaarUI.getStandardDisplay().asyncExec(new Runnable() {
			public void run() {
				JFaceResources.getFontRegistry().addListener(BzrOutputConsole.this);
				initializeStreams();
				dump();
			}
		});
	}

	private void initLimitOutput() {
		IPreferenceStore store = EclipseBazaarUI.getDefault().getPreferenceStore();
		if (store.getBoolean(BazaarPreferenceConstants.PREF_CONSOLE_LIMIT_OUTPUT)) {
			setWaterMarks(1000, store.getInt(BazaarPreferenceConstants.PREF_CONSOLE_HIGH_WATER_MARK));
		} else {
			setWaterMarks(-1, 0);
		}
	}

	/*
	 * Initialize thre streams of the console. Must be called from the UI
	 * thread.
	 */
	private void initializeStreams() {
		synchronized (document) {
			if (!initialized) {
				commandStream = newOutputStream();
				errorStream = newOutputStream();
				messageStream = newOutputStream();
				debugStream = newOutputStream();
				// set the font
				setFont(JFaceResources.getFontRegistry().get(BazaarPreferenceConstants.PREF_CONSOLE_FONT));
				initialized = true;
				// set the colors
				Color newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_COMMAND_COLOR);
				commandStream.setColor(newColor);
				newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_MESSAGE_COLOR);
				messageStream.setColor(newColor);
				newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_ERROR_COLOR);
				errorStream.setColor(newColor);
			}
		}
	}

	private void dump() {
		synchronized (document) {
			visible = true;
			ConsoleDocument.ConsoleLine[] lines = document.getLines();
			for (ConsoleLine line : lines) {
				appendLine(line.type, line.line);
			}
			document.clear();
		}
	}

	private void appendLine(int type, String line) {
		synchronized (document) {
			if (visible) {
				try {
					switch (type) {
					case ConsoleDocument.COMMAND:
						commandStream.write(line);
						commandStream.write(System.getProperty("line.separator"));
						break;
					case ConsoleDocument.MESSAGE:
						messageStream.write("  " + line); //$NON-NLS-1$
						messageStream.write(System.getProperty("line.separator"));
						break;
					case ConsoleDocument.ERROR:
						errorStream.write(System.getProperty("line.separator"));
						errorStream.write("  " + line); //$NON-NLS-1$	
						break;
					case ConsoleDocument.DEBUG:
						debugStream.write("  " + line); //$NON-NLS-1$
						debugStream.write(System.getProperty("line.separator"));
						break;
					case ConsoleDocument.DELIMITER:
						messageStream.write(line); //$NON-NLS-1$
						messageStream.write(System.getProperty("line.separator"));
						break;
					default:
						break;
					}
				} catch (IOException e) {
					EclipseBazaarUI.log(e.getMessage(), e);
				}
			} else {
				document.appendConsoleLine(type, line);
			}
		}
	}

	private void bringConsoleToFront() {
		IConsoleManager manager = ConsolePlugin.getDefault().getConsoleManager();
		if (!visible) {
			manager.addConsoles(new IConsole[] { this });
		}
		manager.showConsoleView(this);
	}

	@Override
    protected void dispose() {
		// Here we can't call super.dispose() because we actually want the
		// partitioner to remain
		// connected, but we won't show lines until the console is added to the
		// console manager
		// again.

		// Called when console is removed from the console view
		synchronized (document) {
			visible = false;
			JFaceResources.getFontRegistry().removeListener(this);
		}
	}

	/**
	 * Clean-up created fonts.
	 */
	public void shutdown() {
		// Call super dispose because we want the partitioner to be
		// disconnected.
		super.dispose();
		EclipseBazaarUI.getDefault().getPreferenceStore().removePropertyChangeListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getProperty();
		// colors
		if (visible) {
			if (BazaarPreferenceConstants.PREF_CONSOLE_COMMAND_COLOR.equals(property)) {
				Color newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_COMMAND_COLOR);
				commandStream.setColor(newColor);
			} else if (BazaarPreferenceConstants.PREF_CONSOLE_MESSAGE_COLOR.equals( property )) {
				Color newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_MESSAGE_COLOR);
				messageStream.setColor(newColor);
			} else if (BazaarPreferenceConstants.PREF_CONSOLE_ERROR_COLOR.equals( property )) {
				Color newColor = createColor(EclipseBazaarUI.getStandardDisplay(), BazaarPreferenceConstants.PREF_CONSOLE_ERROR_COLOR);
				errorStream.setColor(newColor);
				// font
			} else if (BazaarPreferenceConstants.PREF_CONSOLE_FONT.equals( property )) {
				setFont(JFaceResources.getFontRegistry().get(BazaarPreferenceConstants.PREF_CONSOLE_FONT));
			} else if (BazaarPreferenceConstants.PREF_CONSOLE_LIMIT_OUTPUT.equals( property )) {
				initLimitOutput();
			}
		}
		// show preferences
		if (BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_MESSAGE.equals(property)) {
			Object value = event.getNewValue();
			if (value instanceof String) {
				showOnMessage = Boolean.getBoolean((String) event.getNewValue());
			} else {
				showOnMessage = ((Boolean) value).booleanValue();
			}
		}

		// Show on error
		if (BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_ERROR.equals(property)) {
			Object value = event.getNewValue();
			if (value instanceof String) {
				showOnError = Boolean.getBoolean((String) event.getNewValue());
			} else {
				showOnError = ((Boolean) value).booleanValue();
			}
		}

		// Show on debug
		if (BazaarPreferenceConstants.PREF_CONSOLE_SHOW_ON_DEBUG.equals(property)) {
			Object value = event.getNewValue();
			if (value instanceof String) {
				showOnDebug = Boolean.getBoolean((String) event.getNewValue());
			} else {
				showOnDebug = ((Boolean) value).booleanValue();
			}
		}
	}

	/**
	 * Returns a color instance based on data from a preference field.
	 */
	private Color createColor(Display display, String preference) {
		RGB rgb = PreferenceConverter.getColor(EclipseBazaarUI.getDefault().getPreferenceStore(), preference);
		return new Color(display, rgb);
	}

	public void logCommandLine(String commandLine) {
		if (showOnMessage) {
			bringConsoleToFront();
		}
		appendLine(ConsoleDocument.DELIMITER, UITexts.BazaarConsole_delimiter);
		appendLine(ConsoleDocument.COMMAND, commandLine);
	}

	public void logMessage(String message) {
		if (showOnMessage) {
			bringConsoleToFront();
		}
		appendLine(ConsoleDocument.MESSAGE, "  " + message); //$NON-NLS-1$
	}

	public void logRevision(long revision, String path) {
	}

	public void logCompleted(String message) {
		if (showOnMessage) {
			bringConsoleToFront();
		}
		appendLine(ConsoleDocument.MESSAGE, "  " + message); //$NON-NLS-1$
	}

	public void logError(String message) {
		if (showOnMessage || showOnError) {
			bringConsoleToFront();
		}
		appendLine(ConsoleDocument.ERROR, "  " + message); //$NON-NLS-1$
	}

	public void logCommandOutput(String message) {
		if (showOnDebug) {
			bringConsoleToFront();
			appendLine(ConsoleDocument.DEBUG, "  " + message); //$NON-NLS-1$
		}
	}

	public void onNotify(File path) {
	}

	public void setCommand(int command) {
	}

}