/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.decorator;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.ui.PlatformUI;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.ITracingOptions;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 * 
 */
public class BazaarLightweightDecorator extends LabelProvider implements ILightweightLabelDecorator, IResourceStateListener {

	public static final SimpleDateFormat dateFormat = new SimpleDateFormat("MM-dd-yyyy HH:mm:ss");
	private final IPreferenceStore store = EclipseBazaarUI.getDefault().getPreferenceStore();

	public BazaarLightweightDecorator() {
		super();
		// ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
		EclipseBazaarCore.addResourceStateListener(this);
	}

	public static void refresh() {
		EclipseBazaarUI.getDefault().getWorkbench().getDecoratorManager().update(BazaarLightweightDecorator.class.getName());
	}

	public static void update(IContainer container) {
		BazaarLightweightDecorator decorator = (BazaarLightweightDecorator) EclipseBazaarUI.getDefault().getWorkbench().getDecoratorManager()
				.getBaseLabelProvider(BazaarLightweightDecorator.class.getName());
		decorator.refresh(container);
	}

	public void refresh(IContainer container) {
		final List<IResource> resources = new ArrayList<IResource>();
		try {
			container.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) {
					resources.add(resource);
					return true;
				}
			});
			fireLabelChangeEvent(new LabelProviderChangedEvent(this, resources.toArray()));
		} catch (CoreException e) {
			EclipseBazaarUI.log(e);
		}
	}

	public void decorate(Object element, IDecoration decoration) {
		IResource resource = (IResource) element;
		if(resource.isAccessible() && !resource.isLinked(IResource.CHECK_ANCESTORS) && !resource.isDerived() && !resource.isTeamPrivateMember()) {
			IProject project = resource.getProject();
			if (project != null && RepositoryProvider.isShared(project) && getRepositoryProvider(resource) != null) {
				if(EclipseBazaarCore.getDefault().isClientAvailable())
					decorate(resource, project, decoration);
			}
		}
	}

	private void decorate(IResource objectResource, IProject objectProject, IDecoration decoration) {
		String fullPath = (objectResource.getLocation()).toString();
		// Do not decorate the stuff under .bzr
		if (fullPath.indexOf(IBzrBranch.DIR_META_FOLDER) == -1 && objectResource.isAccessible()) {
			try {
				IBzrLocalResource resource = BzrWorkspaceRoot.getBzrResourceFor(objectResource);
				decorateByType(resource, decoration);
			} catch (CoreException e) {
				EclipseBazaarUI.log(e);
			} catch (RuntimeException e) {
				// I can get a runtimeException if the project is closed when
				// I'm still computing decorations
				EclipseBazaarUI.traceException(ITracingOptions.LABEL_DECORATOR, e);
				if (objectProject.isOpen())
					EclipseBazaarUI.log("Unexpected error while decorating: "+objectResource.getName(), e);
			}
		}
	}

	private void decorateByType(IBzrLocalResource bzrResource, IDecoration decoration) throws CoreException {
		// unknown resource
		boolean isCached = EclipseBazaarCore.getDefault().getStatusCacheManager().hasCachedStatus(bzrResource.getIResource());
		if (!isCached) {
			if (!bzrResource.isManaged()) {
				if (store.getBoolean(BazaarPreferenceConstants.STATUS_TEXT_DECORATOR)) {
					decoration.addPrefix(DecoratorTextLabels.notTrackedPrefix);
				} 
				if (store.getBoolean(BazaarPreferenceConstants.STATUS_OVERLAY_DECORATOR)) {
					decoration.addOverlay(DecoratorImages.notTrackedDescriptor);
				}
				return;
			}
		}
		if (bzrResource.isFolder()) {
			IBzrResourceStatus status = bzrResource.getStatus();
			// set the label (only if the resource is a project)
			if (bzrResource.getIResource().getType() == IResource.PROJECT && store.getBoolean(BazaarPreferenceConstants.PROJECT_TEXT_DECORATOR)) {
				String nick = bzrResource.getBranch().getNick();
				if (nick != null)
					decoration.addSuffix("  [" + nick + "]");
			}
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_TEXT_DECORATOR)) {
				decoration.addPrefix(DecoratorTextLabels.getStatusTextLabel(status));
			} 
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_OVERLAY_DECORATOR)) {
				decoration.addOverlay(DecoratorImages.getImageDescriptor(status));
			}
			return;
		} else {
			IBzrResourceStatus status = bzrResource.getStatus();
			IBazaarLogMessage logMessage = null;
			if (store.getBoolean(BazaarPreferenceConstants.FILE_TEXT_DECORATOR) && !status.isAdded() && !status.isUnknown() && !status.isIgnored())
				logMessage = getLastLogMessage(bzrResource.getIResource(), status);
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_TEXT_DECORATOR)) {
				decoration.addPrefix(DecoratorTextLabels.getStatusTextLabel(status));
			} 
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_OVERLAY_DECORATOR)) {
				decoration.addOverlay(DecoratorImages.getImageDescriptor(status));
			}
			if (logMessage != null)
				decoration.addSuffix("  " + logMessage.getRevision().getValue() + " " + logMessage.getCommiter() + " " +
						dateFormat.format(logMessage.getDate()));
		}
	}

	public void dispose() {
		super.dispose();
		EclipseBazaarCore.removeResourceStateListener(this);
	}

	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	private void fireLabelChangeEvent(final LabelProviderChangedEvent event) {
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
			public void run() {
				fireLabelProviderChanged(event);
			}
		});

	}

	public void resourceStateChanged(final BzrChangeStateEvent event) {
		Set<IResource> resources = new HashSet<IResource>();
		for (IResource resource : event.getResources()) {
			if (resource != null && resource.getProject().isAccessible()) {
				resources.add(resource);
			}
		}
		fireLabelChangeEvent(new LabelProviderChangedEvent(this, resources.toArray()));
	}

	private static BazaarTeamProvider getRepositoryProvider(IResource resource) {
		RepositoryProvider repoProvider = RepositoryProvider.getProvider(resource.getProject());
		if (!(repoProvider instanceof BazaarTeamProvider)) {
			// Resource not unsing this plugin
			return null;
		}
		return (BazaarTeamProvider) repoProvider;
	}

	private static IBazaarLogMessage getLastLogMessage(IResource resource, IBzrResourceStatus status) throws CoreException {
		return EclipseBazaarCore.getDefault().getLogCacheManager().getLogs(resource).get(0);
	}

}
