/*******************************************************************************
 * Copyright (c) 2006 Software Balm Consulting Inc.
 * 
 * This software is licensed under the zlib/libpng license.
 * 
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it freely,
 * subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not 
 *            claim that you wrote the original software. If you use this 
 *            software in a product, an acknowledgment in the product 
 *            documentation would be appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *            misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source distribution.
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.BazaarStatusKind;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ICommitOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.actions.CommitAction.CommitableResources;
import org.vcs.bazaar.eclipse.ui.history.HistoryTree;
import org.vcs.bazaar.eclipse.ui.history.HistoryTree.Column;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;

/**
 * <br>
 * A commit dialog box allowing choosing of what files to commit and a commit
 * message for those files. Untracked files may also be chosen. <br>
 * 
 * @author Peter Hunnisett <peter_hge at softwarebalm dot com> <br>
 * @modifiedBy Guillermo Gonzalez
 * 
 */
public class CommitDialog extends TrayDialog {

    private static String DEFAULT_COMMIT_MESSAGE = UITexts.CommitDialog_defaultCommitMessage;

    private class CommittableFilesFilter extends ViewerFilter {

        public CommittableFilesFilter() {
            super();
        }

        /**
         * Filter out uncommittable files (i.e. ! -> deleted but still tracked)
         */
        public boolean select(Viewer viewer, Object parentElement, Object element) {
            if (element instanceof IBzrResourceStatus) {
                IBzrResourceStatus status = ((IBzrResourceStatus) element);
                IBzrLocalResource bzrResource = status.getResource();
                if (status.isUnchanged()
                    || status.isIgnored()
                    || (!status.isDeleted() && bzrResource.getIResource().getType() == IResource.PROJECT))
                {
                    return false;
                }
            }
            return true;
        }
    }

    public class AdaptableCommitList implements IAdaptable, IWorkbenchAdapter {
        private IBzrResourceStatus[] resources;

        public AdaptableCommitList(List<IBzrResourceStatus> resources) {
            this.resources = resources.toArray(new IBzrResourceStatus[0]);
        }

        public Object[] getChildren(Object o) {
            return resources;
        }

        public ImageDescriptor getImageDescriptor(Object object) {
            return null;
        }

        public String getLabel(Object o) {
            if (o instanceof IBzrResourceStatus) {
                return ((IBzrResourceStatus) o).getStatus().toString();
            }
            return o == null ? "" : o.toString();//$NON-NLS-1$
        }

        public Object getParent(Object o) {
            return null;
        }

        public Object getAdapter(Class adapter) {
            if (adapter == IWorkbenchAdapter.class) {
                return this;
            }
            return null;
        }
    }

    private static Column[] columns =
        new Column[] { Column.REVNO, Column.COMMITTER, Column.TIMESTAMP, Column.MESSAGE };

    private StyledText commitTextBox;

    private Label commitTextLabel;

    private Button showUntrackedFilesButton;

    private Button pushAfterCommitButton;

    private Button localCommitButton;

    private CommitableResources commitableResources;

    private List<IResource> filesToAdd;

    private List<IBzrLocalResource> filesToCommit;

    private String commitMessage;

    private MouseListener commitMouseListener;

    private KeyListener commitKeyListener;

    private boolean pushAfterCommit = false;

    private CommitableResources selectedResources;

    private Text authorText;

    private BzrResourceSelectionComposite resourceSelectionComposite;

    private Text fixesText;

    private ArrayList<Option> options = new ArrayList<Option>(0);

    private Button authorButton;

    private Button fixesButton;

    private boolean pendingMerge;

    public CommitDialog(Shell shell, CommitableResources commitableResources,
        CommitableResources selectedResources)
    {
        this(shell);
        this.commitableResources = commitableResources;
        this.selectedResources = selectedResources;
    }

    public CommitDialog(Shell shell) {
        super(shell);
        setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.MAX | SWT.BORDER | SWT.TITLE);
        setBlockOnOpen(false);
    }

    public List<IBzrLocalResource> getFilesToCommit() {
        return filesToCommit;
    }

    public String getCommitMessage() {
        return commitMessage;
    }

    public List<IResource> getFilesToAdd() {
        return filesToAdd;
    }

    protected Control createDialogArea(Composite parent) {
        Composite container = (Composite) super.createDialogArea(parent);
        getShell().setText(UITexts.CommitDialog_dialogTitle);
        container.setLayout(new GridLayout(1, false));
        container.setLayoutData(new GridData(GridData.FILL_BOTH));

        commitTextLabel = new Label(container, SWT.NONE);
        commitTextLabel.setText(UITexts.CommitDialog_commentsTitle);

        commitTextBox = new StyledText(container, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER | SWT.WRAP);
        GridData textBoxData = new GridData(GridData.FILL_HORIZONTAL);
        textBoxData.heightHint = 150;
        commitTextBox.setLayoutData(textBoxData);

        createAuthorTextBox(container);
        createFixesTextBox(container);

        resourceSelectionComposite = new BzrResourceSelectionComposite(!pendingMerge) {
            private HistoryTree logTree;

            @Override
            protected void createOption(Composite container) {
                Group optionGroup = new Group(container, SWT.NONE);
                optionGroup.setText("Options");
                optionGroup.setLayout(new GridLayout(3, false));

                GridData data;
                data = new GridData();
                data.horizontalAlignment = GridData.FILL;
                data.verticalAlignment = GridData.CENTER;
                data.grabExcessHorizontalSpace = true;
                data.grabExcessVerticalSpace = true;
                optionGroup.setLayoutData(data);

                if (!pendingMerge) {
                    showUntrackedFilesButton = new Button(optionGroup, SWT.CHECK);
                    showUntrackedFilesButton.setText(UITexts.CommitDialog_checkboxUntracked);
                }

                pushAfterCommitButton = new Button(optionGroup, SWT.CHECK);
                pushAfterCommitButton.setText(UITexts.CommitDialog_pushAfterCommit);

                localCommitButton = new Button(optionGroup, SWT.CHECK);
                localCommitButton.setText(UITexts.CommitDialog_localCommit);
            }

            @Override
            protected void createUpperControls(Composite container) {
                if (!pendingMerge) {
                    Label commitFilesLabel = new Label(container, GridData.FILL_BOTH);
                    commitFilesLabel.setText("Select Files:");
                }
            }

            @Override
            protected void createMiddleControls(Composite container) {
                if (pendingMerge) {
                    TabFolder folder = new TabFolder(container, SWT.NONE);
                    folder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
                    TabItem fileListItem = new TabItem(folder, SWT.NONE);
                    fileListItem.setText("File list");
                    super.createMiddleControls(folder);
                    fileListItem.setControl(super.getTableViewer().getTable());
                    TabItem logItem = new TabItem(folder, SWT.NONE);
                    logTree = new HistoryTree(folder, columns);
                    logTree.getTree().getColumn(Column.REVNO.ordinal()).setText("RevId");
                    logItem.setControl(logTree);
                    logItem.setText("Pending merges");
                    GridData data;
                    data = new GridData();
                    data.horizontalAlignment = GridData.FILL;
                    data.verticalAlignment = GridData.FILL;
                    data.grabExcessHorizontalSpace = true;
                    logTree.setLayoutData(data);
                    logTree.populate(commitableResources.getAllPendingMerges());
                } else {
                    super.createMiddleControls(container);
                }
            }
        };
        Composite listComposite = resourceSelectionComposite.createControls(container);
        resourceSelectionComposite.getTableViewer().addFilter(new CommittableFilesFilter());
        resourceSelectionComposite.addUntrackedFilesFilter();

        resourceSelectionComposite.setResources(this.commitableResources.getAllStatuses());
        if (pendingMerge) {
            List<IBzrResourceStatus> filteredResources =
                new ArrayList<IBzrResourceStatus>(this.commitableResources.getAllStatuses().size());
            for (IBzrResourceStatus status : this.commitableResources.getAllStatuses()) {
                if (!status.isUnknown()) {
                    filteredResources.add(status);
                }
            }
            resourceSelectionComposite.setSelection(filteredResources);
            resourceSelectionComposite.addUntrackedFilesFilter();
        } else {
            for (IBzrResourceStatus status : selectedResources.getAllStatuses()) {
                if (status.isUnknown() && status.getResource() != null
                    && !status.getResource().isFolder())
                {
                    resourceSelectionComposite.removeUntrackedFilesFilter();
                    break;
                }
            }
            resourceSelectionComposite.setSelection(this.selectedResources.getAllStatuses());
        }
        resourceSelectionComposite.refreshTableViewer();

        listComposite.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));

        makeActions();
        return container;
    }

    private void createAuthorTextBox(Composite parent) {
        final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(3, false));
        container.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
        authorButton = new Button(container, SWT.CHECK);
        GridData data = new GridData();
        data.horizontalAlignment = GridData.BEGINNING;
        data.grabExcessHorizontalSpace = false;
        data.grabExcessVerticalSpace = false;
        authorButton.setLayoutData(data);
        final Label author = new Label(container, SWT.HORIZONTAL);
        author.setText("Author:");
        authorText = new Text(container, SWT.SINGLE | SWT.WRAP);
        authorText.setText(EclipseBazaarUI.getDefault().getPreferenceStore().getString(
            BazaarPreference.BZR_EMAIL.name()));
        authorText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        authorText.setEditable(false);
        authorButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                authorText.setEditable(authorButton.getSelection());
                authorText.setEnabled(authorButton.getSelection());
            }
        });
        container.addControlListener(new ControlAdapter() {
            @Override
            public void controlResized(ControlEvent e) {
                Point currSize = authorText.getSize();
                currSize.x = container.getSize().x;
                authorText.setSize(currSize);
            }
        });
    }

    private void createFixesTextBox(Composite parent) {
        final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(3, false));
        container.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
        fixesButton = new Button(container, SWT.CHECK);
        GridData data = new GridData();
        data.horizontalAlignment = GridData.BEGINNING;
        data.grabExcessHorizontalSpace = false;
        data.grabExcessVerticalSpace = false;
        fixesButton.setLayoutData(data);
        final Label author = new Label(container, SWT.HORIZONTAL);
        author.setText("Fixes:");
        fixesText = new Text(container, SWT.SINGLE | SWT.WRAP);
        fixesText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        fixesText.setEditable(false);
        fixesButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                fixesText.setEditable(fixesButton.getSelection());
                fixesText.setEnabled(fixesButton.getSelection());
            }
        });
        container.addControlListener(new ControlAdapter() {
            public void controlResized(ControlEvent e) {
                Point currSize = fixesText.getSize();
                currSize.x = container.getSize().x;
                fixesText.setSize(currSize);
            }
        });
    }

    private void makeActions() {

        commitTextBox.setCapture(true);
        if (!pendingMerge) {
            showUntrackedFilesButton.setSelection(!resourceSelectionComposite.isFilterUntracked()); // Start unselected.
        }
        pushAfterCommitButton.setSelection(false); // Start unselected.
        localCommitButton.setSelection(false); // Start unselected
        localCommitButton.setVisible(false);

        // TODO: for the moment only one branch is supported in commit so we can do this
        for (IBzrResourceStatus commitable : commitableResources.getAllStatuses()) {
            if (!commitable.isDeleted() && commitable.getResource().getBranch().isBound()) {
                localCommitButton.setVisible(true);
                break;
            }
        }
        if (!pendingMerge) {
            showUntrackedFilesButton.addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    if (showUntrackedFilesButton.getSelection()) {
                        resourceSelectionComposite.removeUntrackedFilesFilter();
                        for (TableColumn column : resourceSelectionComposite.getTableViewer()
                            .getTable().getColumns())
                        {
                            column.pack();
                        }
                    } else {
                        resourceSelectionComposite.addUntrackedFilesFilter();
                    }
                    resourceSelectionComposite.getTableViewer().refresh(true);
                }
            });
        }

        // set Ctrl + Return listener (fix Bug #128071)
        commitTextBox.addTraverseListener(new TraverseListener() {
            public void keyTraversed(TraverseEvent e) {
                if (e.detail == SWT.TRAVERSE_RETURN && (e.stateMask & SWT.CTRL) != 0) {
                    okPressed();
                }
            }
        });

        setupDefaultCommitMessage();
    }

    private void setupDefaultCommitMessage() {
        commitTextBox.setText(DEFAULT_COMMIT_MESSAGE);
        commitMouseListener = new MouseListener() {

            public void mouseDown(MouseEvent e) {
                // On the first mouse down in the area clear the default commit
                // message.
                commitTextBox.setText("");
                commitTextBox.removeMouseListener(commitMouseListener);
                commitTextBox.removeKeyListener(commitKeyListener);
            }

            public void mouseDoubleClick(MouseEvent e) {
                // Nothing
            }

            public void mouseUp(MouseEvent e) {
                // Nothing
            }
        };

        commitKeyListener = new KeyListener() {

            public void keyPressed(KeyEvent e) {
                // On the first key press, deleted the default commit message
                // and then remove the handlers.
                commitTextBox.setText("");
                commitTextBox.removeMouseListener(commitMouseListener);
                commitTextBox.removeKeyListener(commitKeyListener);
            }

            public void keyReleased(KeyEvent e) {
                // Nothing
            }
        };
        commitTextBox.addMouseListener(commitMouseListener);
        commitTextBox.addKeyListener(commitKeyListener);
    }

    private List<IBzrLocalResource> getCommitList(List<IBzrResourceStatus> list) {
        List<IBzrLocalResource> filtered = new ArrayList<IBzrLocalResource>();
        for (IBzrResourceStatus status : list) {
            filtered.add(status.getResource());
        }
        return filtered;
    }

    private List<IResource> getToAddList(List<IBzrResourceStatus> list) {
        List<IResource> filtered = new ArrayList<IResource>();
        for (IBzrResourceStatus status : list) {
            if (status.contains(BazaarStatusKind.UNKNOWN)) {
                filtered.add(status.getResource().getIResource());
            }
        }
        return filtered;
    }

    /**
     * Override the OK button pressed to capture the info we want first and then
     * call super.
     */
    protected void okPressed() {
        filesToAdd = getToAddList(resourceSelectionComposite.getSelection());
        filesToCommit = getCommitList(resourceSelectionComposite.getSelection());
        commitMessage = commitTextBox.getText();
        pushAfterCommit = pushAfterCommitButton.getSelection();
        String author = authorText.getText();
        String fixes = fixesText.getText();
        this.options = new ArrayList<Option>(0);
        if (localCommitButton.getSelection()) {
            options.add(ICommitOptions.LOCAL);
        }
        if (authorButton.getSelection() && author != null && !"".equals(author.trim())) {
            options.add(ICommitOptions.AUTHOR.setArgument(author));
        }
        if (fixesButton.getSelection() && fixes != null && !"".equals(fixes.trim())) {
            options.add(ICommitOptions.FIXES.setArgument(fixes));
        }
        if (commitMessage == null || commitMessage.trim().length() == 0
            || commitMessage.equals(UITexts.CommitDialog_defaultCommitMessage))
        {
            MessageDialog.openInformation(this.getParentShell(), "Empty Commit Text",
                "You must complete the comment");
        } else {
            super.okPressed();
        }
    }

    protected Point getInitialSize() {
        return new Point(480, 700);
    }

    public boolean isPushAfterCommit() {
        return pushAfterCommit;
    }

    public ArrayList<Option> getOptions() {
        return options;
    }

    public int open(CommitableResources commitableResources,
        CommitableResources selectedResources,
        boolean shouldBlock)
    {
        setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.MAX | SWT.BORDER | SWT.TITLE);
        setBlockOnOpen(shouldBlock);
        this.commitableResources = commitableResources;
        this.selectedResources = selectedResources;
        this.pendingMerge = commitableResources.getAllPendingMerges().size() > 0;
        return open();
    }

}
