/**
* Copyright (C) 2009 Canonical Ltd
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * 
 * @author Javier Der Derian
 * @BasedOn RevertDialog from BzrEclipse 
 */
public class IgnoredDialog extends Dialog {

	private Map<IResource, String> ignored;
	private TableViewer tableViewer;
	private List<IResource> unchecked;
	
	/**
	 * Create the dialog
	 * 
	 * @param parentShell
	 * @param selectedResources 
	 * @param commiteableResources 
	 */
	public IgnoredDialog(Shell parentShell, Map<IResource, String> ignored) {
		super(parentShell);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		this.ignored = ignored;
	}
	
	private void createControls(Composite container)
	{
		Composite control = new Composite(container, SWT.NONE);
		control.setLayout(new GridLayout());
		
		final Label label = new Label(control, SWT.NONE);
		label.setText(UITexts.IgnoredDialog_label);
		
		int tableStyle;
		tableStyle = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI | SWT.CHECK | SWT.BORDER;
		
		Composite tablecontainer = new Composite(container, SWT.NONE);
        
		final Table table = new Table(tablecontainer, tableStyle);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumnLayout layout = new TableColumnLayout(); 
		tablecontainer.setLayout(layout);
		// File name
		final TableColumn file = new TableColumn(table, SWT.LEFT);
		file.setResizable(true);
		file.setText(UITexts.IgnoredDialog_ignoredfile);
		layout.setColumnData(file, new ColumnWeightData(60, true));

		final TableColumn pattern = new TableColumn(table, SWT.LEFT);
		pattern.setResizable(true);
		pattern.setText(UITexts.IgnoredDialog_match);
		layout.setColumnData(pattern, new ColumnWeightData(60, true));
		
		tableViewer = new CheckboxTableViewer(table);
		tableViewer.setContentProvider(new IStructuredContentProvider() {

		    public Object[] getElements(Object inputElement) {
		        Map<IResource, String> elements = (Map<IResource, String>)inputElement;
		        return elements.entrySet().toArray();
		    }

		    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		        // do nothing   
		    }

            public void dispose() {
                // do nothing                
            }
		});
		tableViewer.setLabelProvider(new IgnoredFilesLabelProvider());

		tableViewer.setInput(ignored);
		tableViewer.refresh(true);
		((CheckboxTableViewer)tableViewer).setAllChecked(true);

		
		GridDataFactory gf = GridDataFactory.fillDefaults().align(GridData.FILL, GridData.FILL).grab(true, true);
        gf.applyTo(tablecontainer);
        
	}
	
	/**
	 * Create contents of the dialog
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());
		createControls(container);
		return container;
	}
	
	/**
	 * Saves list of unchecked items
	 */
	protected void readUnChecked() {
		final List<IResource> files = new ArrayList<IResource>();
		for (Entry<IResource, String> element : ignored.entrySet()) {
			if(((CheckboxTableViewer)tableViewer).getChecked(element) == false) {
			    files.add(element.getKey());
			}
		}
		this.unchecked = files;
	}
	
	public List<IResource> getUnChecked() {
		return this.unchecked;
	}
	
	
	/**
	 * Create contents of the button bar
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(530, 500);
	}
	
	@Override
	protected void okPressed() { 
		readUnChecked();
		super.okPressed();
	}
	
	// this main is for test the dialog during development 
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		
		IgnoredDialog dialog = new IgnoredDialog(shell, new HashMap<IResource, String>());
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
