/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.net.URI;
import java.util.Collection;
import java.util.Collections;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class LocationInputDialog extends InputDialog {

	private Combo uriCombo;
	private Button remember;
	private Button force;
	private Collection<IBzrBranch> locations;
	private LocationDialogInfo info;
	private Boolean[] options;
	private Button useExistingDir;
	
	private static final int REMEMBER = 0;
	private static final int FORCE = 1;
	private static final int USE_EXISTING_DIR = 2;
	
	protected static class BaseURIValidator implements IInputValidator {
		public String isValid(String newText) {
			try {
				String url = "";
				if(newText != null && newText.startsWith("lp:")) {
					url = newText.trim();
				} else {
					URI uri = URI.create(newText);
					url = uri.toString();
				}
				if(url.toString().length() == 0) {
					return "No location specified." + System.getProperty("line.separator") + "Please enter a valid branch location";
				}
			} catch (IllegalArgumentException ex) {
				// TODO: show a proper error message
				return ex.getMessage();
			}
			return null;
		}
	};

	public static LocationInputDialog remoteBindLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo info) {
		return remoteLocationDialog(parentShell, locations, info, "Please enter the URI to bind: ", false);
	}
	
	public static LocationInputDialog remotePushLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo info) {
		return new LocationInputDialog(parentShell, "Remote location Dialog", "Please enter the URI to push to: ", "", new BaseURIValidator(), locations, info, true, false, true);
	}
	
	public static LocationInputDialog remotePullLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo info) {
		return remoteLocationDialog(parentShell, locations, info, "Please enter the URI to pull from: ", true);
	}
	
	public static LocationInputDialog missingLocationDialog(Shell parentShell, final Collection<IBzrBranch> locations, LocationDialogInfo info) {
		return new LocationInputDialog(parentShell, "Remote location Dialog", "Please enter the URI to look for missing revisions: ", "", new BaseURIValidator(), locations, info, false);
	}
	
	public static LocationInputDialog remoteMergeLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo info) {
		return remoteLocationDialog(parentShell, locations, info, "Please enter the URI of the branch to merge with: ", true);
	}
	
	/**
	 * 
	 * @param parentShell
	 * @param locations
	 * @param info
	 * @param dialogMessage
	 * @param rememberOption if true shows a checkbox fo remember option in pull/push actions
	 * @return 
	 * LocationInputDialog
	 */
	public static LocationInputDialog remoteLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo info, String dialogMessage, boolean rememberOption) {
		return new LocationInputDialog(parentShell, "Remote location Dialog", dialogMessage, "", new BaseURIValidator(), locations, info, rememberOption);
	}
	
	public static LocationInputDialog switchLocationDialog(Shell parentShell, Collection<IBzrBranch> locations, LocationDialogInfo dialogInfo) {
		return new LocationInputDialog(parentShell, "Switch location", "Please enter the URI to switch: ", "", new BaseURIValidator(), locations, dialogInfo, false, true);
	}

	public LocationInputDialog(Shell parentShell, String dialogTitle, String dialogMessage, String initialValue, IInputValidator validator, Collection<IBzrBranch> locations, LocationDialogInfo info, Boolean...options) {
		super(parentShell, dialogTitle, dialogMessage, initialValue, validator);
		this.locations = locations;
		this.info = info;
		this.options = options;
	}
	
	protected LocationInputDialog(Shell shell, String dialogTitle, String dialogMessage, String initialValue, BaseURIValidator validator) {
		super(shell, dialogTitle, dialogMessage, initialValue, validator);
		this.locations = Collections.emptyList();
		this.info = new LocationDialogInfo();
		this.options = new Boolean[0];
	}

	public void loadBranchesInCombo() {
		for (IBzrBranch branch : locations) {
			uriCombo.add(branch.getBranchLocation().toString());
		}
	}

	protected void createTextInput(Composite composite) {
		uriCombo = new Combo(composite, SWT.SINGLE | SWT.BORDER);
		uriCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		uriCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validateInput();
			}
		});
		loadBranchesInCombo();
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite)super.createDialogArea(parent);
		createOptionsArea(composite);
		return composite;
	}

	protected void createOptionsArea(Composite composite) {
		if(options.length >= 1 && options[REMEMBER]) {
			remember = new Button(composite, SWT.CHECK);
			remember.setText("Remember");
			remember.setSelection(false); // Start unselected.
		}
		if(options.length >= 2 && options[FORCE]) {
			force = new Button(composite, SWT.CHECK);
			force.setText("force");
			force.setToolTipText(UITexts.SwitchAction_force_tooltip);
			force.setSelection(false); // Start unselected.
		}
		if(options.length >= 3 && options[USE_EXISTING_DIR]) {
			useExistingDir = new Button(composite, SWT.CHECK);
			useExistingDir.setText("Use existing directory");
			useExistingDir.setToolTipText("Use existing directory");
			useExistingDir.setSelection(false); // Start unselected.
		}
	}

	@Override
	public Control getText() {
		return uriCombo;
	}

	@Override
	public String getTextInput() {
		return uriCombo.getText();
	}
	
	@Override
	public void setTextInput(String text) {
		uriCombo.setText(text);
	}
	
	protected Boolean isRemember() {
		return remember.getSelection();
	}
	
	protected Boolean isForce() {
		return force.getSelection();
	}
		
	@Override
	protected void okPressed() {
		if(options.length >= 1 && options[REMEMBER]) {
			info.setRemember(remember.getSelection());
		}
		if(options.length >= 2 && options[FORCE]) {
			info.setForce(force.getSelection());
		}
		if(options.length >= 3 && options[USE_EXISTING_DIR]) {
			info.setUseExistingDir(useExistingDir.getSelection());
		}
		info.setLocation(getValue());
		super.okPressed();
	}
	
	public static class LocationDialogInfo {
		boolean remember;
		boolean force;
		boolean useExistingDir;
		String location;
		
		public boolean isRemember() {
			return remember;
		}
		public void setRemember(boolean remember) {
			this.remember = remember;
		}
		public String getLocation() {
			return location;
		}
		public void setLocation(String location) {
			this.location = location;
		}
		public boolean isForce() {
			return force;
		}
		public void setForce(boolean force) {
			this.force = force;
		}
		public boolean isUseExistingDir() {
			return useExistingDir;
		}
		public void setUseExistingDir(boolean useExistingDir) {
			this.useExistingDir = useExistingDir;
		}
	}

}
