/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IMergeOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.ui.widgets.RevisionNumber;

/**
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail.com>
 * @BasedOn RevertDialog from MercurialEclipse 
 */
public class MergeDialog extends LocationInputDialog {

	private static final String TITLE = "Remote location Dialog";
	private static final String MESSAGE = "Please enter the URI of the branch to merge with: ";
	private RevisionNumber revision;
	private ArrayList<Option> options;
	private Button reprocess;
	private Button pull;
	private Button uncommitted;
	private MergeDialogInfo dialogInfo;

	/**
	 * Create the dialog
	 * 
	 * @param parentShell
	 * @param selectedResources 
	 * @param commiteableResources 
	 */
	public MergeDialog(Shell parentShell, Collection<IBzrBranch> locations, MergeDialogInfo dialogInfo) {
		super(parentShell, TITLE, MESSAGE, "", new BaseURIValidator(), locations, dialogInfo, true, true);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		this.dialogInfo = dialogInfo;
	}
	
	protected void createOptionsArea(Composite composite) {
		Composite container = new Composite(composite, SWT.NONE);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));
		
		Group checkBoxGroup = new Group(container, SWT.NONE);
		checkBoxGroup.setLayout(new GridLayout(2, true));
		super.createOptionsArea(checkBoxGroup);
		reprocess = new Button(checkBoxGroup, SWT.CHECK);
		reprocess.setText("Reprocess");
		reprocess.setSelection(false); // Start unselected.
		reprocess.setToolTipText("Reprocess to reduce spurious conflicts.");
		pull = new Button(checkBoxGroup, SWT.CHECK);
		pull.setText("Pull");
		pull.setSelection(false); // Start unselected.
		uncommitted = new Button(checkBoxGroup, SWT.CHECK);
		uncommitted.setText("Uncommitted");
		uncommitted.setSelection(false); // Start unselected.
		uncommitted.setToolTipText("Apply uncommitted changes from a working copy, instead of branch changes");
		
		Group revisionGroup = new Group(container, SWT.NONE);
		revisionGroup.setLayout(new GridLayout());
		revision = new RevisionNumber(revisionGroup);
		
		GridData data = new GridData(GridData.FILL, GridData.FILL, true, false);
		revisionGroup.setLayoutData(data);
		checkBoxGroup.setLayoutData(data);
		data = new GridData(GridData.FILL_BOTH);
		container.setLayoutData(data);
	
		revision.addShowLogButtonSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				showLog();
			}
		});
	}
	
	protected void showLog() {
		String url = super.getTextInput();
		if(url == null || "".equals(url.trim())) {
			return;
		}
		IBzrBranch branch = BranchFactory.getInstance().findOrCreateFor(url);
		if(branch != null) {
			ShowLogDialog dialog = new ShowLogDialog(getShell(), branch);
			if(dialog.open() == Dialog.OK && dialog.getSelectedRevision() != null) {
				revision.setRevision(dialog.getSelectedRevision().getValue());
			}
		}
	}
	
	/**
	 * Create contents of the button bar
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(530, 400);
	}
	
	@Override
	protected void okPressed() {
		this.options = new ArrayList<Option>(0);
		if(isRemember()) {
			options.add(IMergeOptions.REMEMBER);
		}
		if(isForce()) {
			options.add(IMergeOptions.FORCE);
		}
		if(pull.getSelection()) {
			options.add(IMergeOptions.PULL);
		}
		if(reprocess.getSelection()) {
			options.add(IMergeOptions.REPROCESS);
		}
		if(uncommitted.getSelection()) {
			options.add(IMergeOptions.UNCOMMITTED);
		}
		if(revision.isRevisionSelected()) {
			options.add(IMergeOptions.REVISION.setArgument(revision.getRevision()));
		}
		this.dialogInfo.options = options;
		super.okPressed();
	}

	public static class MergeDialogInfo extends LocationDialogInfo {
		List<Option> options;
		
		public List<Option> getOptions() {
			return options;
		}
	}
	
	// this main is for test the dialog during development 
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		MergeDialog dialog = new MergeDialog(shell, Collections.EMPTY_LIST, new MergeDialogInfo());
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
