/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IRevertOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;
import org.vcs.bazaar.eclipse.ui.widgets.RevisionNumber;

/**
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail.com>
 * @BasedOn RevertDialog from MercurialEclipse 
 */
public class RevertDialog extends Dialog {

	private Button forgetMerges, noBackup, wholeBranch;
	private RevisionNumber revision;
	private ArrayList<Option> options;
	private BzrResourceSelectionComposite resourcesComposite;
	private List<IBzrResourceStatus> selection;
	private boolean isWholeBranch;
	private boolean wholeBranchEnabled;

	/**
	 * Create the dialog
	 * 
	 * @param parentShell
	 * @param selectedResources 
	 * @param commiteableResources 
	 */
	public RevertDialog(Shell parentShell, List<IBzrResourceStatus> resources, List<IBzrResourceStatus> selectedResources) {
		super(parentShell);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		resourcesComposite = getBzrResourceSelectionComposite();
		resourcesComposite.setSelection(selectedResources);
		resourcesComposite.setResources(resources);
	}
	
	private BzrResourceSelectionComposite getBzrResourceSelectionComposite() {
		return new BzrResourceSelectionComposite() {

			@Override
			protected void createUpperControls(Composite container) {
				final Label label = new Label(container, SWT.NONE);
				label.setText("Checked resources will be reverted");
			}
			
			@Override
			protected void createOption(Composite optionComposite) {
				if(wholeBranchEnabled) {
					wholeBranch = new Button(optionComposite, SWT.CHECK);
					wholeBranch.setText("Revert the whole branch");
				}
				Group optionGroup = new Group(optionComposite, SWT.NONE);
				optionGroup.setText("Options");
				optionGroup.setLayout(new GridLayout(2, false));
				
				revision = new RevisionNumber(optionGroup);
				revision.showLogButton(false); // diable show log button
				
				Composite checkboxContainer = new Composite(optionGroup, SWT.NONE);
				checkboxContainer.setLayout(new GridLayout());
				forgetMerges = new Button(checkboxContainer, SWT.CHECK);
				forgetMerges.setText("Forget merges");
				noBackup = new Button(checkboxContainer, SWT.CHECK);
				noBackup.setText("No backup");
				GridData data;
		        data = new GridData();
		        data.horizontalAlignment = GridData.FILL;
		        data.verticalAlignment = GridData.CENTER;
		        data.grabExcessHorizontalSpace = true;
		        data.grabExcessVerticalSpace = false;
		        optionGroup.setLayoutData(data);
				checkboxContainer.setLayoutData(data);
				
				revision.addShowLogButtonSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						showLog();
					}
				});
			}
		};
	}


	protected void showLog() {
		IBzrBranch branch = this.resourcesComposite.getResources().get(0).getResource().getBranch();
		if(branch != null) {
			ShowLogDialog dialog = new ShowLogDialog(getShell(), branch);
			if(dialog.open() == Dialog.OK && dialog.getSelectedRevision() != null) {
				revision.setRevision(dialog.getSelectedRevision().getValue());
			}
		}
	}
	
	/**
	 * Create contents of the dialog
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());
		Composite control = resourcesComposite.createControls(container);
		control.setLayoutData(new GridData(GridData.FILL_BOTH));
		resourcesComposite.refreshTableViewer();
		resourcesComposite.addUntrackedFilesFilter();
		makeActions();
		return container;
	}

	private void makeActions() {
		if(wholeBranchEnabled) {
			wholeBranch.setSelection(false);
			wholeBranch.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					resourcesComposite.getTableViewer().getTable().setEnabled(!wholeBranch.getSelection());
				}
			});
		}
	}

	/**
	 * Create contents of the button bar
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(530, 500);
	}
	
	public Option[] getOptions() {
		return options.toArray(new Option[options.size()]);
	}

	@Override
	protected void okPressed() { 
		selection = resourcesComposite.getSelection();
		this.options = new ArrayList<Option>(0);
		if(forgetMerges.getSelection()) {
			options.add(IRevertOptions.FORGET_MERGES);
		}
		if(noBackup.getSelection()) {
			options.add(IRevertOptions.NO_BACKUP);
		}
		if(revision.isRevisionSelected()) {
			options.add(IRevertOptions.REVISION.setArgument(revision.getRevision()));
		}
		isWholeBranch = wholeBranchEnabled?wholeBranch.getSelection():false;
		super.okPressed();

	}

	/**
	 * 
	 * @return List<IBzrResourceStatus> of the selected resources
	 */
	public List<IBzrResourceStatus> getSelection() {
		return selection;
	}
	
	public boolean isWholeBranch() {
		return isWholeBranch;
	}

	public void setWholeBranchEnabled(boolean enable) {
		this.wholeBranchEnabled = enable;
	}

	// this main is for test the dialog during development 
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		RevertDialog dialog = new RevertDialog(shell, new ArrayList<IBzrResourceStatus>(0), new ArrayList<IBzrResourceStatus>(0));
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
