/**
* Copyright (C) 2009 Canonical Ltd
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ISendOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.widgets.RevisionNumber;

/**
 * 
 * @author Javier Der Derian <javierder AT gmail.com>
 * @BasedOn RevertDialog from BzrEclipse 
 */
public class SendDialog extends LocationInputDialog {

	private static final String TITLE = UITexts.BazaarSendDialogTitle;
	private static final String MESSAGE = UITexts.BazaarSendDialogMessage;
	private RevisionNumber revision;
	private ArrayList<Option> options;
	private Button nobundle;
	private Button nopatch;
	private Button send;
	private Button save;
	
	private SendDialogInfo dialogInfo;
	private Text submitLocationText;
	private Text messageText;
	private Text formatText; 
	private String mailErrorMessage;
	
	private Text errorMessageText;

	/**
	 * Create the dialog
	 * 
	 * @param parentShell
	 * @param selectedResources 
	 * @param commiteableResources 
	 */
	public SendDialog(Shell parentShell, Collection<IBzrBranch> locations, SendDialogInfo dialogInfo) {
		super(parentShell, TITLE, MESSAGE, "", new BaseURIValidator(), locations, dialogInfo, true, false);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		this.dialogInfo = dialogInfo;
	}
	
	protected void createOptionsArea(Composite composite) {
		
		final Label label = new Label(composite, SWT.NONE);
		label.setText(UITexts.SendDialog_EnterLocation);

		Group typeBoxGroup = new Group(composite, SWT.NONE);
		
		GridData typeBoxGrid = new GridData();
		
		typeBoxGrid.horizontalAlignment = SWT.FILL;		
		typeBoxGroup.setLayout(new GridLayout(2, true));
		typeBoxGroup.setLayoutData(typeBoxGrid);
		
		send = new Button(typeBoxGroup, SWT.RADIO);
		send.setText(UITexts.SendDialog_SendEmail);
		send.setSelection(true); // Start selected.
		send.setToolTipText(UITexts.SendDialog_SendEmailTooltip);
		save = new Button(typeBoxGroup, SWT.RADIO);
		save.setText(UITexts.SendDialog_SaveLocation);
		save.setSelection(false); // Start unselected.
		save.setToolTipText(UITexts.SendDialog_SaveLocationTooltip);
		
		final Composite submitComosite = new Composite(composite,SWT.None);
		submitComosite.setLayout(new GridLayout(2, false));
		
		GridData submitLocationGrid = new GridData();
		
		submitLocationGrid.horizontalAlignment = SWT.FILL;
		submitLocationGrid.widthHint = 440;
		submitLocationText = new Text(submitComosite, SWT.BORDER | SWT.SINGLE);
		submitLocationText.setLayoutData(submitLocationGrid);
		
		
		submitLocationText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if(send.getSelection()) {
					Pattern p = Pattern.compile(".+@.+\\.[a-z]+");
					Matcher m = p.matcher(getPath());
					boolean matchFound = m.matches();
					if (matchFound) {
						setMailErrorMessage(null);
					}
					else {
						setMailErrorMessage(UITexts.SendDialog_WrongEmail);
					}
				}
				
			}
		});
		
		errorMessageText = new Text(composite, SWT.READ_ONLY | SWT.WRAP);
		errorMessageText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		errorMessageText.setBackground(errorMessageText.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		
		GridData locationBoxGrid = new GridData();
		locationBoxGrid.horizontalAlignment = SWT.FILL;
		submitComosite.setLayoutData(locationBoxGrid);
		
		final Button browseButton = new Button(submitComosite, SWT.PUSH);
		browseButton.setText(UITexts.BazaarSendDialogBrowse);
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				final FileDialog dialog = new FileDialog(getShell());
				dialog.setText("Save As");
				final String dir = dialog.open();
				if (dir != null){
					submitLocationText.setText(dir);
				}
			}
		});
		browseButton.setEnabled(false);
		save.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				browseButton.setEnabled(true);
				setMailErrorMessage(null);
				super.widgetSelected(e);
			}
		});
		send.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				browseButton.setEnabled(false);
				submitLocationText.setText(submitLocationText.getText());
				super.widgetSelected(e);
			}
		});
		
		final Label label2 = new Label(composite, SWT.NONE);
		label2.setText("Message");
		
		GridData messageGrid = new GridData();
		
		messageGrid.horizontalAlignment = SWT.FILL;		
		messageText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		messageText.setLayoutData(messageGrid);
		
		Composite container = new Composite(composite, SWT.NONE);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));
		
		Group checkBoxGroup = new Group(container, SWT.NONE);
		checkBoxGroup.setLayout(new GridLayout(2, false));
		super.createOptionsArea(checkBoxGroup);
		nobundle = new Button(checkBoxGroup, SWT.CHECK);
		nobundle.setText(UITexts.SendDialog_NoBundle);
		nobundle.setSelection(false); // Start unselected.
		nobundle.setToolTipText(UITexts.SendDialog_NoBundleTooltip);
		nopatch = new Button(checkBoxGroup, SWT.CHECK);
		nopatch.setText(UITexts.SendDialog_NoPatch);
		nopatch.setSelection(false); // Start unselected.
		nopatch.setToolTipText(UITexts.SendDialog_NoPatchTooltip);
		
		
		Composite formatBoxGroup = new Composite(checkBoxGroup, SWT.NONE);
		
		GridData formatBoxGrid = new GridData();
		
		formatBoxGrid.horizontalAlignment = SWT.FILL;		
		formatBoxGroup.setLayout(new GridLayout(2, false));
		formatBoxGroup.setLayoutData(formatBoxGrid);
		
		final Label label3 = new Label(formatBoxGroup, SWT.NONE);
		label3.setText(UITexts.SendDialog_Format);
		formatText = new Text(formatBoxGroup, SWT.BORDER | SWT.SINGLE);
	
		Group revisionGroup = new Group(container, SWT.NONE);
		revisionGroup.setLayout(new GridLayout());
		revision = new RevisionNumber(revisionGroup);
		
		GridData data = new GridData(GridData.FILL, GridData.FILL, true, false);
		revisionGroup.setLayoutData(data);
		checkBoxGroup.setLayoutData(data);
		data = new GridData(GridData.FILL_BOTH);
		container.setLayoutData(data);
		
		revision.addShowLogButtonSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				showLog();
			}
		});
	}
	
	/**
	 * Sets or clears the error message. If not <code>null</code>, the OK
	 * button is disabled.
	 * 
	 * @param errorMessage
	 *            the error message, or <code>null</code> to clear
	 */
	public void setMailErrorMessage(String mailErrorMessage) {
		if (errorMessageText != null && !errorMessageText.isDisposed()) {
			errorMessageText.setText(mailErrorMessage == null ? System.getProperty("line.separator") : mailErrorMessage); //$NON-NLS-1$
			// Disable the error message text control if there is no error, or
			// no error text (empty or whitespace only). Hide it also to avoid
			// color change.
			// See https://bugs.eclipse.org/bugs/show_bug.cgi?id=130281
			boolean hasError = mailErrorMessage != null && (StringConverter.removeWhiteSpaces(mailErrorMessage)).length() > 0;
			errorMessageText.setEnabled(hasError);
			errorMessageText.setVisible(hasError);
			errorMessageText.getParent().update();
			// Access the ok button by id, in case clients have overridden
			// button creation.
			// See https://bugs.eclipse.org/bugs/show_bug.cgi?id=113643
			Control button = getButton(IDialogConstants.OK_ID);
			if (button != null) {
				button.setEnabled(mailErrorMessage == null);
			}
		}

	}
	
	protected void showLog() {
		String url = super.getTextInput();
		if(url == null || "".equals(url.trim())) {
			return;
		}
		IBzrBranch branch = BranchFactory.getInstance().findOrCreateFor(url);
		if(branch != null) {
			ShowLogDialog dialog = new ShowLogDialog(getShell(), branch);
			if(dialog.open() == Dialog.OK && dialog.getSelectedRevision() != null) {
				revision.setRevision(dialog.getSelectedRevision().getValue());
			}
		}
	}
	
	public String getPath()	{
		return submitLocationText.getText();
	}
	public String getMessage() {
		return messageText.getText();
	}
	
	public String getFormat() {
		return formatText.getText();
	}
	
	
	public Integer getType(){
		// 0 means send by email, 1 means save in file
		if(send.getSelection())
			return 0;
		else
			return 1;
	}
	
	protected Boolean isNoBundle() {
		return nobundle.getSelection();
	}
	
	protected Boolean isNoPatch() {
		return nopatch.getSelection();
	}
	
	public Option[] getOptions() {
		return options.toArray(new Option[options.size()]);
		
	}
	
	/**
	 * Create contents of the button bar
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(540, 530);
	}
	
	@Override
	protected void okPressed() {
		this.options = new ArrayList<Option>(0);
		
		if(getPath().length() > 0 && getValue().length() > 0 ) {
			if(getType() == 0){
				options.add(ISendOptions.MAIL_TO.setArgument(getPath()));
			}
			else {
				options.add(ISendOptions.OUTPUT.setArgument(getPath()));
			}
		}
		else {
			//FIXME: give some kind of message
			return;
		}
		
		options.add(ISendOptions.MESSAGE.setArgument(getMessage()));
		
		if(isRemember()) {
			options.add(ISendOptions.REMEMBER);
		}

		if(isNoBundle()) {
			options.add(ISendOptions.NO_BUNDLE);
		}
		
		if(isNoPatch()) {
			options.add(ISendOptions.NO_PATCH);
		}
		if(revision.isRevisionSelected()) {
			options.add(ISendOptions.REVISION.setArgument(revision.getRevision()));
		}
	
		if(getFormat().length() > 0) {
			options.add(ISendOptions.FORMAT.setArgument(getFormat()));
		}
		
		this.dialogInfo.options = options;
		super.okPressed();
	}

	public static class SendDialogInfo extends LocationDialogInfo {
		List<Option> options;
		
		public List<Option> getOptions() {
			return options;
		}
	}
	
	// this main is for test the dialog during development 
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		SendDialog dialog = new SendDialog(shell, Collections.EMPTY_LIST, new SendDialogInfo());
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
