/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.history;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.history.IFileHistory;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.ui.history.HistoryPage;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileRevision;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.resources.IStorageBazaarRevision;
import org.vcs.bazaar.eclipse.core.utils.BzrStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.actions.AbstractCompareAction;
import org.vcs.bazaar.eclipse.ui.actions.OpenBazaarRevisionAction;
import org.vcs.bazaar.eclipse.ui.operations.DiffOperation;

/**
 * 
 * @author Guillermo Gonzalez
 * @basedOn MercurialHistoryPage by zingo 
 * 
 */
public class BazaarHistoryPage extends HistoryPage implements IResourceStateListener {

	private TreeViewer viewer;
	private Composite control; 
	private LogTreeContentProvider logTreeContentProvider;
	private BazaarFileHistory bazaarFileHistory;
	IFileRevision[] entries;

	private RefreshBazaarHistory refreshFileHistoryJob;

	private class RefreshBazaarHistory extends Job {
		BazaarFileHistory bazaarFileHistory;

		public RefreshBazaarHistory() {
			super("Fetching Bazaar revisions..."); //$NON-NLS-1$
			this.setUser(true);
			this.setPriority(UIJob.LONG);
		}

		public void setFileHistory(BazaarFileHistory bazaarFileHistory) {
			this.bazaarFileHistory = bazaarFileHistory;
		}

		public IStatus run(IProgressMonitor monitor) {
			IStatus status = Status.OK_STATUS;
			if (bazaarFileHistory != null) {
				monitor.beginTask(getName(), 1000);
				try {
					bazaarFileHistory.refresh(monitor);
					if(viewer.getContentProvider() == null)
						setContentProvider();
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
					status = e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(BazaarException.wrapException(e));
					status = new BzrStatus(BzrStatus.ERROR, e.getMessage(), e);
				} finally {
					monitor.done();
				}
			}
			return status;
		}

	}

	class LogTreeContentProvider implements ITreeContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			entries = null;
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (entries != null)
				return entries;

			final IFileHistory fileHistory = (IFileHistory) parent;
			entries = fileHistory.getFileRevisions();

			return entries;
		}

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof BazaarFileRevision)
				return ((BazaarFileRevision) parentElement).getChildren();
			return null;
		}

		public Object getParent(Object element) {
			if (element instanceof BazaarFileRevision && ((BazaarFileRevision) element).hasParent()) {
				return ((BazaarFileRevision) element).getParent();
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof BazaarFileRevision) {
				BazaarFileRevision revision = (BazaarFileRevision) element;
				return revision.hasChildren();
			}
			return false;
		}

	}
	
	public BazaarHistoryPage(IResource resource) {
		super();
		setInput(resource);
		EclipseBazaarCore.addResourceStateListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.HistoryPage#inputSet()
	 */
	@Override
	public boolean inputSet() {
		if (isValidInput(getInput())) {
			bazaarFileHistory = new BazaarFileHistory(getIFile());
			refresh();
			return true;
		}
		return false;
	}
	
	public IFile getIFile() {
		return (IFile)getInput();
	}


	public void createControl(Composite parent) {
		control = new HistoryTree(parent);
		viewer = HistoryTreeHelper.createTreeViewer((HistoryTree)control);
		contributeActions();
	}
	
	private void setContentProvider() {
		if(logTreeContentProvider == null)
			logTreeContentProvider = new LogTreeContentProvider();
		viewer.setContentProvider(logTreeContentProvider);
	}

	private void contributeActions() {
		final OpenBazaarRevisionAction openAction = new OpenBazaarRevisionAction("Open");
		openAction.setPage(this);
		final AbstractCompareAction compareWithcurrentAction = new AbstractCompareAction("Compare with current") {
			@Override
			public void internalRun() {
				try {
					IStorageBazaarRevision storage = getStorage(0);
					DiffOperation.openSaveableEditor(storage.getResource(), storage.getRevision());
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
				}
			}
			@Override
			protected boolean shouldShow(IStructuredSelection selection) {
				return getInput() instanceof IFile && selection.size() == 1;
			}
		};
		final AbstractCompareAction abstractCompareAction = new AbstractCompareAction("Compare each Other") {
			@Override
			public void internalRun() {
				try {
					DiffOperation.openEditor(getStorage(1), getStorage(0));
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
				}
			}
			@Override
			public boolean shouldShow(IStructuredSelection selection) {
				return getInput() instanceof IFile && selection.size() == 2;
			}

		};
		viewer.getTree().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				openAction.selectionChanged((IStructuredSelection) viewer.getSelection());
				abstractCompareAction.selectionChanged((IStructuredSelection) viewer.getSelection());
				compareWithcurrentAction.selectionChanged((IStructuredSelection) viewer.getSelection());
			}
		});
		// Contribute actions to popup menu
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(viewer.getTree());
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menuMgr) {
				menuMgr.add(new Separator(IWorkbenchActionConstants.GROUP_FILE));
				menuMgr.add(openAction);
				menuMgr.add(abstractCompareAction);
				menuMgr.add(compareWithcurrentAction);
			}
		});
		menuMgr.setRemoveAllWhenShown(true);
		viewer.getTree().setMenu(menu);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#getControl()
	 */
	@Override
	public Control getControl() {
		return control;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#getDescription()
	 */
	public String getDescription() {
		return getIFile().getFullPath().toOSString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#getName()
	 */
	public String getName() {
		return getIFile().getFullPath().toOSString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#isValidInput(java.lang.Object)
	 */
	public boolean isValidInput(Object object) {
		if (object instanceof IResource && ((IResource) object).getType() == IResource.FILE) {
			RepositoryProvider provider = RepositoryProvider.getProvider(((IFile) object).getProject());
			if (provider instanceof BazaarTeamProvider) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#refresh()
	 */
	public void refresh() {
		if (isValidInput(getInput()) && getControl() != null) {
			if (refreshFileHistoryJob == null) {
				refreshFileHistoryJob = new RefreshBazaarHistory();
				refreshFileHistoryJob.addJobChangeListener(new JobChangeAdapter() {
					public void done(IJobChangeEvent event) {
						if(event.getResult().isOK())
							BazaarHistoryPage.this.getSite().getShell().getDisplay().asyncExec(new Runnable() {
								public void run() {
									viewer.setInput(bazaarFileHistory);	
								}
							});
					}
				});
			}

			boolean canceled = true;
			if (refreshFileHistoryJob.getState() != Job.NONE) {
				canceled = refreshFileHistoryJob.cancel();
			}
			if(canceled) {
				refreshFileHistoryJob.setFileHistory(bazaarFileHistory);
				refreshFileHistoryJob.setRule(getIFile().getProject());
				refreshFileHistoryJob.schedule();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * Select the revision in the receiver.
	 */
	public void selectRevision(BazaarRevision revision) {
		if (entries == null) {
			return;
		}

		BazaarFileRevision entry = findIFileRevisionBy(entries, revision);
		if(entry != null) {
			ITreeSelection selection = new TreeSelection(createTreePath(entry));
			viewer.expandToLevel(selection.getPaths(), 0);
			viewer.setSelection(selection, true);
		}

	}
	
	private BazaarFileRevision findIFileRevisionBy(IFileRevision[] entries, BazaarRevision revision) {
		for (IFileRevision aEntry : entries) {
			if (((BazaarFileRevision)aEntry).getRevision().equals(revision)) {
				return (BazaarFileRevision)aEntry;
			} else if(logTreeContentProvider.hasChildren(aEntry)) {
				BazaarFileRevision result = findIFileRevisionBy((BazaarFileRevision[])logTreeContentProvider.getChildren(aEntry), revision);
				if(result != null)
					return result;
			}
		}
		return null;
	}
	
	private TreePath createTreePath(BazaarFileRevision entry) {
		List<BazaarFileRevision> list = new ArrayList<BazaarFileRevision>();
		BazaarFileRevision current = entry;
		list.add(0, current);
		while(current.getParent() != null) {
			list.add(0, current.getParent());
			current = current.getParent();
		}
		return new TreePath(list.toArray());
	}

	public void resourceStateChanged(final BzrChangeStateEvent event) {
		if((event.getType() & BzrChangeStateEvent.RESOURCE_MODIFY) > 0) {
			return;
		}
		if(getIFile() != null && getIFile().exists()) {
			for (IResource resource : event.getResources()) {
				if(resource.isAccessible() && resource.equals(getInput())) {
					refresh();
				}
			}
		}
	}

	@Override
	public void dispose() {
		if (refreshFileHistoryJob.getState() != Job.NONE) {
			refreshFileHistoryJob.cancel();
		}
		EclipseBazaarCore.removeResourceStateListener(this);
		super.dispose();
	}
	
}
