/**
 * 
 */
package org.vcs.bazaar.eclipse.ui.history;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.vcs.bazaar.client.IBazaarLogMessage;

/**
 * A composite that holds a Tree as the only child.
 * This Tree is able to show {@link IBazaarLogMessage}
 * 
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 *
 */
public class HistoryTree extends Composite {
	
	public enum Column {
		REVNO, TAGS, COMMITTER, TIMESTAMP, BRANCH_NICK, MESSAGE, FILES;
	}
	
	private static Column[] defaultColumns = new Column[]{Column.REVNO, Column.COMMITTER, Column.TIMESTAMP, Column.BRANCH_NICK, Column.MESSAGE};	
	private Tree tree;
	private boolean onlyShowMainline;
	
	public HistoryTree(Composite parent, int style, Column...columns) {
		super(parent, SWT.NONE);
		GridLayout layout0 = new GridLayout();
		layout0.marginHeight = 0;
		layout0.marginWidth = 0;
		this.setLayout(layout0);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.grabExcessVerticalSpace = true;
		this.setLayoutData(data);

		if(columns == null || columns.length == 0) {
			columns = defaultColumns;
		}
		tree = HistoryTreeHelper.configureTree(this, new Tree(this, style), columns);
	}
	
	public HistoryTree(Composite parent, Column...columns) {
		this(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL, columns);
	}

	public void setOnlyShownMainline(boolean onlyMainline) {
		this.onlyShowMainline = onlyMainline;
	}

	public Tree getTree() {
		return tree;
	}
	
	public void populate(final List<IBazaarLogMessage> logs) {
		getTree().removeAll();
		if(logs == null)
			return;
		
		for (IBazaarLogMessage logMessage : logs) {
			TreeItem root = createItem();
			if(logMessage.getMerged() != null && logMessage.getMerged().size() > 0 && !onlyShowMainline) {
				List<IBazaarLogMessage> childs = logMessage.getMerged();
				int idx = 0; 
				for (IBazaarLogMessage child : childs) {
					setItemText(createChildItem(root, idx), child);
					idx++;
				}
			}
			setItemText(root, logMessage);
		}
	}
	
	private TreeItem createItem() {
		TreeItem item = new TreeItem(getTree(), SWT.NONE);
		return item;
	}
	
	private static TreeItem createChildItem(TreeItem parent, int index) {
		TreeItem item = new TreeItem(parent, SWT.NONE, index);
		return item;
	}
	
	protected void setItemText(TreeItem item, IBazaarLogMessage logMessage) {
		TreeColumn[] columns = getTree().getColumns();
		String[] text = new String[columns.length];
		for (int i = 0; i < columns.length; i++) {
			Column colType = ((Column)columns[i].getData());
			if(Column.REVNO.equals(colType)) {
				String rev = logMessage.getRevision().getValue();
				text[i] = rev==null||"".equals(rev.trim())?logMessage.getRevisionId():rev;
			} else if(Column.TAGS.equals(colType)) {
				
			} else if(Column.COMMITTER.equals(colType)) {
				text[i] = logMessage.getCommiter();
			} else if(Column.TIMESTAMP.equals(colType)) {
				text[i] = logMessage.getTimeStamp();
			} else if(Column.BRANCH_NICK.equals(colType)) {
				text[i] = logMessage.getBranchNick();
			} else if(Column.MESSAGE.equals(colType)) {
				text[i] = logMessage.getMessage().replace("(\r\n|\n)", System.getProperty("line.separator"));
			} else if(Column.FILES.equals(colType)) {
				
			}
		}
		item.setText(text);
		item.setData(logMessage);		
	}

	public void addSelectionListener(SelectionListener listener) {
		tree.addSelectionListener(listener);
	}

	public IBazaarLogMessage getSelection() {
		final TreeItem item = tree.getSelection()[0];
		if(item != null) {
			return (IBazaarLogMessage)item.getData();
		}
		return null;
	}
	
	@Override
    public void setEnabled(boolean enabled) {
        tree.setEnabled(enabled);
    }
}
