/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.markers;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.Job;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * Mark newly created files as not being under version control.
 * 
 * @author johan.walles@gmail.com
 */
public class NewFileMarker implements IResourceStateListener {
	
	private final static int INTERESTING_CHANGES = BzrChangeStateEvent.BRANCH_STATE_MODIFY 
												| BzrChangeStateEvent.WT_STATE_MODIFY
												| BzrChangeStateEvent.RESOURCE_MODIFY;
	
    /**
     * Create a background job that can be run to mark resources not handled by Bazaar.
     * @param resources The resources to check.  Those that are unhandled will be marked.
     * @return The resource checking-and-marking job.
     */
    private WorkspaceJob createMarkJob(final Set<IResource> resources) {
        return new WorkspaceJob("[Bazaar] Creating unhandled files markers") {
            /**
             * Finds out which resources are files not under Bazaar's control.
             * @param allResources The resources to check.
             * @return A set of files not under Bazaar's control.
             * @throws CoreException When status fetching fails
             */
            private Set<IResource> getUnhandledFiles(Set<IResource> allResources) throws CoreException {
                Set<IResource> result = new HashSet<IResource>();
                
                // Collect all unhandled resources
                for (IResource resource : allResources) {
                    if (resource.getType() != IResource.FILE) {
                        // We're only interested in files
                        continue;
                    }
                    if (resource.isDerived()) {
                        // Derived resources aren't supposed to be handled by Bazaar
                        continue;
                    }
                    if (resource.isLinked(IResource.CHECK_ANCESTORS)) {
                        // Linked resources should be handled at their actual location,
                        // not here
                        continue;
                    }
                    IBzrResourceStatus status =
                        EclipseBazaarCore.getDefault().getStatusCacheManager().getStatus(resource);
                    if (status.isUnknown()) {
                        result.add(resource);
                    }
                }
                
                return result;
            }
            
            @Override
            public IStatus runInWorkspace(IProgressMonitor monitor) throws CoreException {
                monitor.beginTask("[Bazaar] Creating unhandled files markers",
                                  resources.size() * 2);
                Set<IResource> unhandledFiles = getUnhandledFiles(resources);
                if (unhandledFiles.size() == 0) {
                    return Status.OK_STATUS;
                }
                monitor.worked(resources.size());
                for (IResource unhandled : unhandledFiles) {
        	        // FIXME: use a multi rule for all the projects involded in the state change event.
                	ISchedulingRule rule = ResourcesPlugin.getWorkspace().getRuleFactory().markerRule(unhandled);
                	getJobManager().beginRule(rule, monitor);
                	IMarker marker = null;
                	IMarker[] markers = unhandled.findMarkers(IMarker.MARKER, true, IResource.DEPTH_ZERO);
                	for (IMarker aMarker : markers) {
						if(aMarker.getAttribute("bazaar_unhanlded", false)) {
							marker = aMarker;
							break;
						}
					}
                	if(marker == null) {
                		marker = unhandled.createMarker(EclipseBazaarUI.getPluginId() + ".marker");
                		marker.setAttribute(IMarker.MESSAGE, "Local addition not under Bazaar control");
                        marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO);
                        marker.setAttribute(IMarker.LOCATION, "");
                        marker.setAttribute("bazaar_unhanlded", true);
                	}
                    monitor.worked(1);
                    getJobManager().endRule(rule);
                }
                return Status.OK_STATUS;
            }

        };
    }
    

	private WorkspaceJob createUnMarkJob(final HashSet<IResource> resources) {
		return new WorkspaceJob("[Bazaar] Cleaning unhandled files markers") {
			@Override
			public IStatus runInWorkspace(IProgressMonitor monitor) throws CoreException {
		    	for (IResource resource : resources) {
		    		if(!resource.isAccessible() || !ResourceUtil.shouldHandle(resource)) {
		    			continue;
		    		}
		    		ISchedulingRule rule = ResourcesPlugin.getWorkspace().getRuleFactory().markerRule(resource);
		    		getJobManager().beginRule(rule, monitor);
		    		IMarker[] markers = resource.findMarkers(IMarker.MARKER, true, IResource.DEPTH_ZERO);
		        	for (IMarker aMarker : markers) {
						if(aMarker.getAttribute("bazaar_unhanlded", false)) {
							aMarker.delete();
						}
					}
		        	monitor.worked(1);
		        	getJobManager().endRule(rule);
		    	}
		    	return Status.OK_STATUS;
			}
		};
		
	}
    
    public void resourceStateChanged(BzrChangeStateEvent event) {
    	boolean enabled = isEnabled();
    	if (!enabled) {
    		Collection<IResource> resources = event.getResources();
    		if(resources == null || resources.size() == 0) {
	            return;
	        }
	    	WorkspaceJob job = createUnMarkJob(new HashSet<IResource>(resources));
	        job.setPriority(Job.DECORATE);
	        job.setSystem(false);
	        job.schedule(600);
    	} else if((event.getType() & INTERESTING_CHANGES) > 0) {
    		Collection<IResource> resources = event.getResources();
	    	if(resources == null || resources.size() == 0) {
	            return;
	        }
	        WorkspaceJob job = createMarkJob(new HashSet<IResource>(resources));
	        job.setPriority(Job.DECORATE);
	        job.setSystem(false);
	        
	        // FIXME: I'm using 600 here just because that's what used in
	        // AbstractResourceChangeListener.  Is there a better value for this?
	        // johan.walles@gmail.com 2008aug03
	        job.schedule(600);
    	}
    }

	private Boolean isEnabled() {
		return EclipseBazaarUI.getDefault().getPreferenceStore().getBoolean(BazaarPreferenceConstants.NOT_VERSIONED_MARKER);
	}

}
