package org.vcs.bazaar.eclipse.ui.preferences;

import java.io.File;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PathEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * Main Preference page, it allow configure: <br>
 * - the bzr executable path<br>
 * - plugins path <br>
 * - xmlrpc service port<br>
 * - nick <br>
 * 
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 */
public class BazaarPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    private FileFieldEditor executable;
    private StringFieldEditor email;
    private PathEditor pluginPath;
    private Label clientValue;
	private IntegerFieldEditor xmlrpcPort;
    private BooleanFieldEditor useBundled;

	public BazaarPreferencePage() {
        super(GRID);
		setPreferenceStore(EclipseBazaarUI.getDefault().getPreferenceStore());
		setDescription(UITexts.BazaarPreferencesPage_description);
	}

	public void init(IWorkbench workbench) {}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	public void createFieldEditors() {
		executable = new FileFieldEditor(BazaarPreference.EXECUTABLE.name(), UITexts.BazaarPreferencesPage_executable, getFieldEditorParent());
		email = new StringFieldEditor(BazaarPreference.BZR_EMAIL.name(), UITexts.BazaarPreferencesPage_userid, getFieldEditorParent());
        useBundled = new BooleanFieldEditor(IBazaarCoreConstants.USE_BUNDLED_PLUGINS,
                            UITexts.BazaarPreferencesPage_enable_bundled_plugins, getFieldEditorParent());
        pluginPath = new PathEditor(BazaarPreference.BZR_PLUGIN_PATH.name(), UITexts.BazaarPreferencesPage_plugins_dir, UITexts.BazaarPreferencesPage_plugins_dir_chooser, getFieldEditorParent());
		addField(executable);
		addField(email);
        addField(useBundled);
		addField(pluginPath);
		createClientTypeControl();
		createXmlRpClientConfigControl();
		executable.getTextControl(getFieldEditorParent()).setEditable(false);
	}
	
	private void createClientTypeControl() {
		final Label label = new Label(getFieldEditorParent(), SWT.NONE);
        label.setText(UITexts.BazaarPreferencesPage_client_type);
		Group clientTypeGroup = new Group(getFieldEditorParent(), SWT.NONE);
		clientTypeGroup.setLayout (new GridLayout (2, false));
		clientTypeGroup.setLayoutData (new GridData (SWT.FILL, SWT.CENTER, false, false));
		clientValue = new Label(clientTypeGroup, SWT.NONE);
		setClientTypeLabel();
		executable.getTextControl(getFieldEditorParent()).addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setClientTypeLabel();
				checkState();
			}
		});
		final Button checkClient = new Button(getFieldEditorParent(), SWT.NONE);
        checkClient.setText(UITexts.BazaarPreferencesPage_check_client);
		checkClient.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing
			}
			public void widgetSelected(SelectionEvent e) {
				checkState();
			}
		});
	}
	
	private void createXmlRpClientConfigControl() {
		xmlrpcPort = new IntegerFieldEditor(BazaarPreference.BZR_XMLRPC_PORT.name(), UITexts.BazaarPreferencesPage_xmlrpc_port, getFieldEditorParent(), 5);
		addField(xmlrpcPort);
	}

	private void setClientTypeLabel() {
		try {
			clientValue.setText(BazaarClientFactory.getPreferredClientType());
		} catch (BazaarClientException e1) {
			clientValue.setText(e1.getMessage());
		}
	}

	@Override
	protected void checkState() {
		super.checkState();
		if(isValid()) {
			setValid(performOk());
		}
	}
	
	@Override
	public boolean performOk() {
		final IPreferenceStore store = EclipseBazaarUI.getDefault().getPreferenceStore();
		executable.store();
		email.store();
		pluginPath.store();
        useBundled.store();
		store.setValue(BazaarPreference.EXECUTABLE.name(), executable.getStringValue());
		store.setValue(BazaarPreference.BZR_EMAIL.name(), email.getStringValue());
		store.setValue(BazaarPreference.BZR_XMLRPC_PORT.name(), Integer.parseInt(xmlrpcPort.getStringValue()));
        store.setValue(IBazaarCoreConstants.USE_BUNDLED_PLUGINS, useBundled.getBooleanValue());
        final StringBuilder pluginPath = new StringBuilder();
        for (String item : this.pluginPath.getListControl(getFieldEditorParent()).getItems()) {
            if (pluginPath.length() > 0) {
                pluginPath.append(File.pathSeparatorChar);
            }
            pluginPath.append(item);
        }
        store.setValue(BazaarPreference.BZR_PLUGIN_PATH.name(), pluginPath.toString());
		boolean validClient = checkValidClient();
		return validClient?super.performOk():validClient;
	}
	
    private boolean checkValidClient() {
		// check the client state
		try {
			return EclipseBazaarCore.getDefault().checkClient();
		} catch (BazaarException e) {
			setErrorMessage(e.getStatus().getMessage());
			return false;
		} finally {
			setClientTypeLabel();
		}
	}
	
}