/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.properties;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPropertyPage;
import org.eclipse.ui.dialogs.PropertyPage;
import org.vcs.bazaar.client.IBazaarInfo;
import org.vcs.bazaar.eclipse.core.commands.InfoCommand;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;

/**
 * @author Guillermo Gonzalez
 * 
 */
public class BazaarPropertyPage extends PropertyPage implements IWorkbenchPropertyPage {

    private IBazaarInfo info = null;
    private Text layout;
    private Text controlDirFormat;
    // Formats
    private Composite formats;
    private Text branchFormat;
    private Text controlFormat;
    private Text repositoryFormat;
    private Text workingTreeFormat;
    // locations
    private Composite locations;
    private Text lightCheckoutRoot;
    private Text repositoryCheckoutRoot;
    private Text checkoutRoot;
    private Text checkoutOfBranch;
    private Text sharedRepository;
    private Text repository;
    private Text repositoryBranch;
    private Text branchRoot;
    private Text boundToBranch;
    // related branches
    private Composite relatedBranches;
    private Text publicBranch;
    private Text pushBranch;
    private Text parentBranch;
    private Text submitBranch;
    // stats
    private Composite workingTreeStats;
    private Text added;
    private Text ignored;
    private Text modified;
    private Text removed;
    private Text renamed;
    private Text unchanged;
    private Text unknown;
    private Text versionedSubdirectories;

    private Composite branchHistory;
    private Text branchRevisions;
    private Text committers;
    private Text daysOld;
    private Text firstRevisionDate;
    private Text latestRevisionDate;

    private Composite repositoryStats;
    private Text repositoryRevisions;
    private Text repositorySize;

    public BazaarPropertyPage() {
        super();
    }

    private void addFirstSection(Composite parent) {
        Composite composite = createDefaultComposite(parent);

        Label label = new Label(composite, SWT.NONE);
        label.setText("Layout: "); //$NON-NLS-1$
        layout = new Text(composite, SWT.SINGLE | SWT.READ_ONLY);

        label = new Label(composite, SWT.NONE);
        label.setText("Control directory format: "); //$NON-NLS-1$
        controlDirFormat = new Text(composite, SWT.SINGLE | SWT.READ_ONLY);

        // populate fields
        if (info == null) {
            return;
        }
        layout.setText(info.getLayout());
        final StringBuilder sb = new StringBuilder();
        String[] formats = info.getFormats();
        for (int idx = 0; idx < formats.length; idx++) {
            sb.append(formats[idx]);
            if (idx + 1 < formats.length) {
                sb.append(" or ");
            }
        }
        controlDirFormat.setText(sb.toString());

        // related branches
        if (info.getRelatedBranches() == null) {
            return;
        }
        label = new Label(composite, SWT.NONE);
        label.setText("Related branches: "); //$NON-NLS-1$
        relatedBranches = createDefaultComposite(composite);
        if (info.getRelatedBranches().getParentBranch() != null) {
            label = new Label(relatedBranches, SWT.NONE);
            label.setText("Parent Branch: "); //$NON-NLS-1$
            parentBranch = new Text(relatedBranches, SWT.SINGLE | SWT.READ_ONLY);
            parentBranch.setText(info.getRelatedBranches().getParentBranch());
        }
        if (info.getRelatedBranches().getPushBranch() != null) {
            label = new Label(relatedBranches, SWT.NONE);
            label.setText("Push branch: "); //$NON-NLS-1$
            pushBranch = new Text(relatedBranches, SWT.SINGLE | SWT.READ_ONLY);
            pushBranch.setText(info.getRelatedBranches().getPushBranch());
        }
        if (info.getRelatedBranches().getPublicBranch() != null) {
            label = new Label(relatedBranches, SWT.NONE);
            label.setText("Public branch: "); //$NON-NLS-1$
            publicBranch = new Text(relatedBranches, SWT.SINGLE | SWT.READ_ONLY);
            publicBranch.setText(info.getRelatedBranches().getSubmitBranch());
        }
        if (info.getRelatedBranches().getSubmitBranch() != null) {
            label = new Label(relatedBranches, SWT.NONE);
            label.setText("Submit branch: "); //$NON-NLS-1$
            submitBranch = new Text(relatedBranches, SWT.SINGLE | SWT.READ_ONLY);
            submitBranch.setText(info.getRelatedBranches().getSubmitBranch());
        }

    }

    private void addSeparator(Composite parent) {
        Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        separator.setLayoutData(gridData);
    }

    private void addSecondSection(Composite parent) {
        Composite composite = createDefaultComposite(parent);
        Label label = new Label(composite, SWT.NONE);
        label.setText("Formats: "); //$NON-NLS-1$
        formats = createDefaultComposite(composite);
        label = new Label(formats, SWT.NONE);
        label.setText("Branch format: "); //$NON-NLS-1$
        branchFormat = new Text(formats, SWT.SINGLE | SWT.READ_ONLY);
        label = new Label(formats, SWT.NONE);
        label.setText("Control format: "); //$NON-NLS-1$
        controlFormat = new Text(formats, SWT.SINGLE | SWT.READ_ONLY);
        label = new Label(formats, SWT.NONE);
        label.setText("Repository format: "); //$NON-NLS-1$
        repositoryFormat = new Text(formats, SWT.SINGLE | SWT.READ_ONLY);
        label = new Label(formats, SWT.NONE);
        label.setText("Working tree format: "); //$NON-NLS-1$
        workingTreeFormat = new Text(formats, SWT.SINGLE | SWT.READ_ONLY);

        //        populate default fields
        if (info == null) {
            return;
        }
        branchFormat.setText(info.getBranchFormat() == null ? "" : info.getBranchFormat());
        controlFormat.setText(info.getControlFormat() == null ? "" : info.getControlFormat());
        repositoryFormat.setText(info.getRepositoryFormat() == null ? "" : info
            .getRepositoryFormat());
        workingTreeFormat.setText(info.getWorkingTreeFormat() == null ? "" : info
            .getWorkingTreeFormat());

        createLocations(composite);

        createWorkingTreeStats(composite);

        createBranchHistory(composite);

        if (info.getRepositoryStats() == null) {
            return;
        }
        label = new Label(composite, SWT.NONE);
        label.setText("Repository"); //$NON-NLS-1$
        repositoryStats = createDefaultComposite(composite);
        if (info.getRepositoryStats().getRevisionCount() != null) {
            repositoryRevisions = new Text(repositoryStats, SWT.WRAP | SWT.READ_ONLY);
            repositoryRevisions.setText(info.getRepositoryStats().getRevisionCount().toString());
            label = new Label(repositoryStats, SWT.NONE);
            label.setText("revisions"); //$NON-NLS-1$
        }
        if (info.getRepositoryStats().getSize() != null) {
            repositorySize = new Text(repositoryStats, SWT.WRAP | SWT.READ_ONLY);
            repositorySize.setText(info.getRepositoryStats().getSize().toString());
            label = new Label(repositoryStats, SWT.NONE);
            label.setText("KiB"); //$NON-NLS-1$
        }

    }

    private void createWorkingTreeStats(Composite parent) {
        if (info.getWorkingTreeStats() == null) {
            return;
        }
        Label label = new Label(parent, SWT.NONE);
        label.setText("In the working tree"); //$NON-NLS-1$
        workingTreeStats = createDefaultComposite(parent);
        if (info.getWorkingTreeStats().getUnchanged() != null) {
            unchanged = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            unchanged.setText(info.getWorkingTreeStats().getUnchanged().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("unchanged"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getModified() != null) {
            modified = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            modified.setText(info.getWorkingTreeStats().getModified().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("modified"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getAdded() != null) {
            added = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            added.setText(info.getWorkingTreeStats().getAdded().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("added"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getRemoved() != null) {
            removed = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            removed.setText(info.getWorkingTreeStats().getRemoved().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("removed"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getRenamed() != null) {
            renamed = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            renamed.setText(info.getWorkingTreeStats().getRenamed().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("renamed"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getUnknown() != null) {
            unknown = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            unknown.setText(info.getWorkingTreeStats().getUnknown().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("unknown"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getIgnored() != null) {
            ignored = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            ignored.setText(info.getWorkingTreeStats().getIgnored().toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("ignored"); //$NON-NLS-1$
        }
        if (info.getWorkingTreeStats().getVersionedSubdirectories() != null) {
            versionedSubdirectories = new Text(workingTreeStats, SWT.WRAP | SWT.READ_ONLY);
            versionedSubdirectories.setText(info.getWorkingTreeStats().getVersionedSubdirectories()
                .toString());
            label = new Label(workingTreeStats, SWT.NONE);
            label.setText("versioned subdirectories"); //$NON-NLS-1$
        }

    }

    private void createLocations(Composite parent) {
        if (info.getLocations() == null) {
            return;
        }
        Label label = new Label(parent, SWT.NONE);
        label.setText("Locations: "); //$NON-NLS-1$
        locations = createDefaultComposite(parent);
        if (info.getLocations().getLightCheckoutRoot() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Light checkout root: "); //$NON-NLS-1$
            lightCheckoutRoot = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            lightCheckoutRoot.setText(info.getLocations().getLightCheckoutRoot());
        }
        if (info.getLocations().getRepositoryCheckoutRoot() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Repository checkout root: "); //$NON-NLS-1$
            repositoryCheckoutRoot = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            repositoryCheckoutRoot.setText(info.getLocations().getRepositoryCheckoutRoot());
        }
        if (info.getLocations().getCheckoutRoot() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Checkout root: "); //$NON-NLS-1$
            checkoutRoot = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            checkoutRoot.setText(info.getLocations().getCheckoutRoot());
        }
        if (info.getLocations().getCheckoutOfBranch() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Checkout of branch: "); //$NON-NLS-1$
            checkoutOfBranch = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            checkoutOfBranch.setText(info.getLocations().getCheckoutOfBranch());
        }
        if (info.getLocations().getSharedRepository() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Shared repository: "); //$NON-NLS-1$
            sharedRepository = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            sharedRepository.setText(info.getLocations().getSharedRepository());
        }
        if (info.getLocations().getRepository() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Repository: "); //$NON-NLS-1$
            repository = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            repository.setText(info.getLocations().getRepository());
        }
        if (info.getLocations().getRepositoryBranch() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Repository branch: "); //$NON-NLS-1$
            repositoryBranch = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            repositoryBranch.setText(info.getLocations().getRepositoryBranch());
        }
        if (info.getLocations().getBranchRoot() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Branch root: "); //$NON-NLS-1$
            branchRoot = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            branchRoot.setText(info.getLocations().getBranchRoot());
        }
        if (info.getLocations().getBoundToBranch() != null) {
            label = new Label(locations, SWT.NONE);
            label.setText("Bound to branch: "); //$NON-NLS-1$
            boundToBranch = new Text(locations, SWT.SINGLE | SWT.READ_ONLY);
            boundToBranch.setText(info.getLocations().getBoundToBranch());
        }
    }

    private void createBranchHistory(Composite parent) {
        if (info.getBranchHistory() == null) {
            return;
        }
        Label label = new Label(parent, SWT.NONE);
        label.setText("Branch history: "); //$NON-NLS-1$
        branchHistory = createDefaultComposite(parent);
        if (info.getBranchHistory().getRevisions() != null) {
            label = new Label(branchHistory, SWT.NONE);
            label.setText("Revisions: "); //$NON-NLS-1$
            branchRevisions = new Text(branchHistory, SWT.WRAP | SWT.READ_ONLY);
            branchRevisions.setText(info.getBranchHistory().getRevisions().toString());
        }
        if (info.getBranchHistory().getCommitters() != null) {
            label = new Label(branchHistory, SWT.NONE);
            label.setText("Committers: "); //$NON-NLS-1$
            committers = new Text(branchHistory, SWT.WRAP | SWT.READ_ONLY);
            committers.setText(info.getBranchHistory().getCommitters().toString());
        }
        if (info.getBranchHistory().getDaysOld() != null) {
            label = new Label(branchHistory, SWT.NONE);
            label.setText("Days old: "); //$NON-NLS-1$
            daysOld = new Text(branchHistory, SWT.WRAP | SWT.READ_ONLY);
            daysOld.setText(info.getBranchHistory().getDaysOld().toString());
        }
        if (info.getBranchHistory().getFirstRevisionDate() != null) {
            label = new Label(branchHistory, SWT.NONE);
            label.setText("First revision date: "); //$NON-NLS-1$
            firstRevisionDate = new Text(branchHistory, SWT.SINGLE | SWT.READ_ONLY);
            firstRevisionDate.setText(info.getBranchHistory().getFirstRevisionDate().toString());
        }
        if (info.getBranchHistory().getLatestRevisionDate() != null) {
            label = new Label(branchHistory, SWT.NONE);
            label.setText("Latest revision date: "); //$NON-NLS-1$
            latestRevisionDate = new Text(branchHistory, SWT.SINGLE | SWT.READ_ONLY);
            latestRevisionDate.setText(info.getBranchHistory().getLatestRevisionDate().toString());
        }
    }

    /*
     * @see PreferencePage#createContents(Composite)
     */
    protected Control createContents(Composite parent) {
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        composite.setLayout(layout);
        GridData data = new GridData(GridData.FILL);

        composite.setLayoutData(data);
        loadInfo();
        addFirstSection(composite);
        addSeparator(composite);
        addSecondSection(composite);

        Dialog.applyDialogFont(parent);

        return composite;
    }

    private Composite createDefaultComposite(Composite parent) {
        Composite composite = new Composite(parent, SWT.NULL);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        composite.setLayout(layout);

        GridData data = new GridData();
        data.verticalAlignment = GridData.FILL;
        composite.setLayoutData(data);

        return composite;
    }

    private void loadInfo() {
        IResource resource = (IResource) getElement();
        try {
            InfoCommand cmd = new InfoCommand(resource.getProject());
            cmd.run(new NullProgressMonitor());
            info = cmd.getInfo();
        } catch (BazaarException e) {
            EclipseBazaarUI.log(e);
        }
    }

    protected void performDefaults() {
    }

    public boolean performOk() {
        return true;
    }

}
