/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.team;

import java.io.BufferedReader;
import java.io.CharArrayWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.subscribers.ISubscriberChangeEvent;
import org.eclipse.team.core.subscribers.ISubscriberChangeListener;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.IStorageDocumentProvider;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.IElementStateListener;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.quickdiff.IQuickDiffReferenceProvider;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFile;
import org.vcs.bazaar.eclipse.core.subscriber.BazaarStatusSuscriber;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * @author Guillermo Gonzalez
 * @basedOn Subversive/Subclipsse IQuickDiffReferenceProvider implementation
 * 
 */
public class BzrQuickDiffReferenceProvider implements IQuickDiffReferenceProvider {

	public static final String ID = "org.vcs.bazaar.eclipse.quickdiff.providers.BazaarReferenceProvider";
	
	private String id;
	private ITextEditor editor = null;
	private IDocument document = null;
	private boolean isDocumentInitialized = false;
	private IDocumentProvider documentProvider = null;
	private SyncInfo syncInfo;
	private Job documentUpdateJob;

	/**
	 * Refresh the document if a change occurs to the related file.
	 */
	private ISubscriberChangeListener suscriberChangeListener = new ISubscriberChangeListener() {
		public void subscriberResourceChanged(ISubscriberChangeEvent[] deltas) {
			if (isDocumentInitialized) {
				for (ISubscriberChangeEvent delta : deltas) {
					IResource resource = delta.getResource();
					if (resource.getType() == IResource.FILE && syncInfo != null && resource.equals(syncInfo.getLocal())) {
						if (delta.getFlags() == ISubscriberChangeEvent.SYNC_CHANGED) {
							updateDocument();
						}
					}
				}
			}
		}
	};

	/**
	 * Updates the document if the document is changed (e.g. replace with)
	 */
	private IElementStateListener documentListener = new IElementStateListener() {
		public void elementDirtyStateChanged(Object element, boolean isDirty) {}

		public void elementContentAboutToBeReplaced(Object element) {}

		public void elementContentReplaced(Object element) {
			if (editor != null && editor.getEditorInput() == element) {
				updateDocument();
			}
		}

		public void elementDeleted(Object element) {}

		public void elementMoved(Object originalElement, Object movedElement) {}
	};

	public IDocument getReference(IProgressMonitor monitor) throws CoreException {
		if (!isDocumentInitialized)
			return null;
		if (document == null) {
			readDocument(monitor);
		}
		return document;
	}

	public void setActiveEditor(ITextEditor targetEditor) {
		if (!(targetEditor.getEditorInput() instanceof IFileEditorInput))
			return;
		editor = targetEditor;
		documentProvider = editor.getDocumentProvider();
		if (documentProvider != null) {
			BazaarStatusSuscriber.getInstance().addListener(suscriberChangeListener);
			((IDocumentProvider) documentProvider).addElementStateListener(documentListener);
		}
		isDocumentInitialized = true;
	}

	public boolean isEnabled() {
		if (!isDocumentInitialized)
			return false;
		try {
			return getManagedFile() != null;
		} catch (CoreException e) {
			return false;
		}
	}

	public void dispose() {
		isDocumentInitialized = false;
		// stop update job
		if (documentUpdateJob != null && documentUpdateJob.getState() != Job.NONE) {
			documentUpdateJob.cancel();
		}
		// remove listeners
		if (documentProvider != null) {
			documentProvider.removeElementStateListener(documentListener);
		}
		BazaarStatusSuscriber.getInstance().removeListener(suscriberChangeListener);
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getId() {
		return id;
	}

	/**
	 * Determine if the file represented by this quickdiff provider has changed with respect to it's remote state. Return true if the remote
	 * contents should be refreshed, and false if not.
	 * @basedOn Subclipse QuickDiffProvider.computeChange
	 */
	private boolean computeChange(IProgressMonitor monitor) throws TeamException {
		boolean needToUpdateReferenceDocument = false;
		if (isDocumentInitialized) {
			IResource editorFile = getFileFromEditor();
			SyncInfo info = editorFile==null?null:BazaarStatusSuscriber.getInstance().getSyncInfo(editorFile);
			if (info == null && syncInfo != null) {
				return true;
			} else if (info == null) {
				return false;
			}

			if (syncInfo == null) {
				needToUpdateReferenceDocument = true;
			} else if (!syncInfo.equals(info)) {
				needToUpdateReferenceDocument = true;
			}
			syncInfo = info;
		}
		return needToUpdateReferenceDocument;
	}

	/**
	 * Creates a document and initializes it with the contents of the last revision of a Bazaar managed resource.
	 * @basedOn Subclipse QuickDiffProvider.readDocument
	 * @param monitor the progress monitor
	 * @throws CoreException
	 */
	private void readDocument(IProgressMonitor monitor) throws CoreException {
		if (!isDocumentInitialized)
			return;
		if (document == null)
			document = new Document();
		if (computeChange(monitor)) {
			IResourceVariant repoFile = (IResourceVariant) syncInfo.getBase();
			if (repoFile != null && documentProvider instanceof IStorageDocumentProvider) {
				IStorageDocumentProvider provider = (IStorageDocumentProvider) documentProvider;
				String encoding = provider.getEncoding(editor.getEditorInput());
				if (encoding == null) {
					encoding = provider.getDefaultEncoding();
				}
				if (!monitor.isCanceled()) {
					InputStream stream = repoFile.getStorage(monitor).getContents();
					if (stream == null || monitor.isCanceled() || !isDocumentInitialized) {
						return;
					}
					setDocumentContent(document, stream, encoding);
				}
			} else {
				// the repoFile is null, so ensure that the document is empty
				if (monitor.isCanceled())
					return;
				document.set("");
			}
		}
	}

	/**
	 * Intitializes the given document with the given stream using the given encoding.
	 * @basedOn Subclipse QuickDiffProvider.setDocumentContent 
	 * 
	 * @param document the document to be initialized
	 * @param contentStream the stream which delivers the document content
	 * @param encoding the character encoding for reading the given stream
	 * @exception CoreException if the given stream can not be read
	 */
	private static void setDocumentContent(IDocument document, InputStream contentStream, String encoding) throws CoreException {
		Reader in = null;
		try {
			final int DEFAULT_FILE_SIZE = 15 * 1024;

			in = new BufferedReader(new InputStreamReader(contentStream, encoding), DEFAULT_FILE_SIZE);
			CharArrayWriter caw = new CharArrayWriter(DEFAULT_FILE_SIZE);
			char[] readBuffer = new char[2048];
			int n = in.read(readBuffer);
			while (n > 0) {
				caw.write(readBuffer, 0, n);
				n = in.read(readBuffer);
			}
			document.set(caw.toString());
		} catch (IOException x) {
			throw new BazaarException(UITexts.error_QuickDiffProvider_readingFile, x); //$NON-NLS-1$
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException x) {
					throw new BazaarException(UITexts.error_QuickDiffProvider_closingFile, x); //$NON-NLS-1$
				}
			}
		}
	}

	/**
	 * Returns the IBzrFile associated with he active editor 
	 * or <code>null</code> if the provider doesn't not have 
	 * access to a Bazaar managed file.
	 * 
	 * @return the Bazaar managed file
	 */
	private IBzrLocalFile getManagedFile() throws CoreException {
		IFile file = getFileFromEditor();
		if (file != null) {
			IBzrLocalFile bzrFile = BzrWorkspaceRoot.getBzrFileFor(file);
			if (bzrFile != null && bzrFile.exists() && bzrFile.isManaged()) {
				IBzrResourceStatus status = bzrFile.getStatus();
				if(status != null && !status.isAdded())
				return bzrFile;
			}
		}
		return null;
	}

	private IFile getFileFromEditor() {
		IEditorInput input = editor != null?editor.getEditorInput():null;
		if (input != null && input instanceof IFileEditorInput) {
			return ((IFileEditorInput) input).getFile();
		}
		return null;
	}

	/**
	 * Runs a job that updates the document. If a previous job is already running, "try" to cancel it before starting the new job.
	 */
	private void updateDocument() {
		if (!isDocumentInitialized)
			return;
		if (documentUpdateJob != null && documentUpdateJob.getState() != Job.NONE) {
			documentUpdateJob.cancel();
		}
		documentUpdateJob = new Job(UITexts.QuickDiffProvider_fetchingContents) { //$NON-NLS-1$
			protected IStatus run(IProgressMonitor monitor) {
				try {
					readDocument(monitor);
				} catch (CoreException e) {
					// return the status associated with the exception, to inform that the update job failed
					return e.getStatus();
				}
				return Status.OK_STATUS;
			}
		};
		documentUpdateJob.schedule();
	}
}
