package org.vcs.bazaar.eclipse.ui.views;

import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.team.internal.ui.ITeamUIImages;
import org.eclipse.team.internal.ui.TeamUIMessages;
import org.eclipse.team.internal.ui.TeamUIPlugin;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.internal.ui.synchronize.SyncInfoModelElement;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.utils.BzrStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.history.HistoryTree;

/**
 * ChangeLog view based on the view example for now
 * 
 * This sample class demonstrates how to plug-in a new workbench view. The view
 * shows data obtained from the model. The sample creates a dummy model on the
 * fly, but a real implementation would connect to the model available either in
 * this or another plug-in (e.g. the workspace). The view is connected to the
 * model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be
 * presented in the view. Each view can present the same model objects using
 * different labels and icons, if needed. Alternatively, a single label provider
 * can be shared between views in order to ensure that objects of the same type
 * are presented in the same way everywhere.
 * <p>
 */

public class LogView extends BazaarView {
	private List<IBazaarLogMessage> changeLog;
	private HistoryTree logTree;
	private IResource resource;
	private RefreshLog refreshLogJob = null;
	
	/**
	 * The constructor.
	 */
	public LogView() {
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		logTree = new HistoryTree(parent);
		makeActions();
		contributeToActionBars();
		getSite().getPage().addPostSelectionListener(selectionListener);
	}

	private void showChangeLog(List<IBazaarLogMessage> changelog) {
		this.changeLog = changelog;
		logTree.populate(changeLog);
	}
	
	@Override
	public void setInput(Object object) {
		if(object instanceof IResource) {
			resource = (IResource)object;
		}
		refresh();
	}
	
	public void refresh() {
		if (this.resource != null && this.resource.getType() != IResource.FILE) {
			if (refreshLogJob == null) {
				refreshLogJob = new RefreshLog();
			}
			if (refreshLogJob.getState() != Job.NONE) {
				refreshLogJob.cancel();
			}
			refreshLogJob.setResource(this.resource);
			refreshLogJob.schedule();
		}
	}
	
	private class RefreshLog extends Job {
		private IResource resource;
		
		public RefreshLog() {
			super("Fetching Bazaar revisions..."); //$NON-NLS-1$
			this.setUser(true);
			this.setPriority(UIJob.LONG);
		}
		
		public void setResource(IResource resource) {
			this.resource = resource;
		}

		public IStatus run(IProgressMonitor monitor) {
			IStatus status = Status.OK_STATUS;
			if (this.resource != null && this.resource.isAccessible()) {
				monitor.beginTask("", 1000);
				try {
					final List<IBazaarLogMessage> logMessages = EclipseBazaarCore.getDefault().getLogCacheManager().getLogs(this.resource);
					monitor.worked(100);
					LogView.this.getSite().getShell().getDisplay().asyncExec(new Runnable() {
						public void run() {
							showChangeLog(logMessages);
						}
					});
					monitor.worked(400);
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
					status = e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(BazaarException.wrapException(e));
					return UIJob.errorStatus(e);
				} finally {
					monitor.done();
				}
			}
			return status;
		}
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(refreshAction);
		manager.add(linkWithSelectionAction);
	}
	
	Action refreshAction;
	Action linkWithSelectionAction;
	
	private void makeActions() {
		refreshAction = new Action(TeamUIMessages.GenericHistoryView_Refresh, TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_REFRESH)) {
			@Override
			public void run() {
				refresh();
			}
		};
		refreshAction.setToolTipText(TeamUIMessages.GenericHistoryView_RefreshTooltip);
		refreshAction.setEnabled(true);

		linkWithSelectionAction = new Action(TeamUIMessages.GenericHistoryView_LinkWithEditor, TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_LINK_WITH)) {
			@Override
			public void run() {
				setLinkingEnabled(isChecked());
			}
			
		};
		linkWithSelectionAction.setChecked(isLinkingEnabled());
		linkWithSelectionAction.setToolTipText(TeamUIMessages.GenericHistoryView_LinkWithTooltip);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		// viewer.getControl().setFocus();
	}
	
	public void resourceStateChanged(final BzrChangeStateEvent event) {
		if(event.getType() == BzrChangeStateEvent.RESOURCE_MODIFY) {
			return;
		}
		final IResource resource = this.resource;
		new UIJob("Checking metadata changes...") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				monitor.beginTask(getName(), event.getResources().size());
				if(resource != null && checkIfPageIsVisible()) {
					for (IResource aResource : event.getResources()) {
						if(aResource.isAccessible() && resource.getProject().equals(aResource.getProject())) {
							setInput(resource);
							break;			
						}
					}
				}
				monitor.done();
				return BzrStatus.OK_STATUS;
			}
		}.schedule();
		
	}
	
	private Object lastSelectedElement;
	boolean linkingEnabled;
	
	private ISelectionListener selectionListener = new ISelectionListener() {

		public void selectionChanged(IWorkbenchPart part, ISelection selection) {

			if (selection instanceof IStructuredSelection) {
				IStructuredSelection structSelection = (IStructuredSelection) selection;
				//Always take the first element - this is not intended to work with multiple selection
				//Also, hang on to this selection for future use in case the view is not visible
				lastSelectedElement = structSelection.getFirstElement();
				
				if (!isLinkingEnabled() || !checkIfPageIsVisible()) {
					return;
				}
				
				if (lastSelectedElement != null){
					Object resource;
					if (lastSelectedElement instanceof SyncInfoModelElement) {
						SyncInfoModelElement syncInfoModelElement = (SyncInfoModelElement) lastSelectedElement;
						resource = syncInfoModelElement.getSyncInfo().getLocal();
					} else {
						resource = Utils.getAdapter(lastSelectedElement, IResource.class);
					}
					if (resource != null)
						setInput((IResource) resource);
					else
						setInput(lastSelectedElement);
					//reset lastSelectedElement
					lastSelectedElement = null;
				}
			}
		}
	};
	
	boolean isLinkingEnabled() {
		return linkingEnabled;
	}

	/**
	 * Enabled linking to the active editor
	 * @param enabled flag indicating whether linking is enabled
	 */
	public void setLinkingEnabled(boolean enabled) {
		this.linkingEnabled = enabled;

		// remember the last setting in the dialog settings		
//		TeamUIPlugin.getPlugin().getPreferenceStore().setValue(IFileHistoryConstants.PREF_GENERIC_HISTORYVIEW_EDITOR_LINKING, enabled);
	}
	

}