/**
 * LICENSE + COPYRIGHT
 */
package org.vcs.bazaar.eclipse.ui.wizards.project;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.NewProjectAction;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.operations.BranchAsProjectOperation;
import org.vcs.bazaar.eclipse.ui.team.BazaarUtilities;
import org.vcs.bazaar.eclipse.ui.wizards.BzrWizardPage;

/**
 * This class implements the new wizard extension.
 * 
 * @author Guillermo Gonzalez <guillo.gonzo at gmail dot com>
 */
public class BranchWizard extends Wizard implements INewWizard {

	private BranchWizardBranchLocationPage createRepoLocationPage;	
	private BranchWizardSelectionPage selectBranchPage;
	private WizardNewProjectCreationPage newProjectPage;
	private BranchWizardProjectPage projectPage;
	
	private IBzrBranch remoteLocation;
	private boolean useNewProjectWizard;

	@Override
	public boolean canFinish() {
		if(useNewProjectWizard) {
			return (getRemoteLocation() != null);
		} else if(getProjectHandle() == null){
			return getRemoteLocation() != null && getNewProjectPage().isPageComplete();
		} else {
			return getRemoteLocation() != null && getProjectHandle() != null;
		}
	}

	protected IProject getProjectHandle() {
		try {
			if(!getProjectPage().useNewProjectWizard() && getNewProjectPage().useDefaults())
				return getNewProjectPage().getProjectHandle();
		} catch(IllegalArgumentException iae) {
			return null;
		}
		return null;
	}

	public boolean performFinish() {
		IProject project = null;
		if(useNewProjectWizard) {
			ProjectListener listener = new ProjectListener();
			ResourcesPlugin.getWorkspace().addResourceChangeListener(listener, IResourceChangeEvent.POST_CHANGE);
			(new NewProjectAction(PlatformUI.getWorkbench().getActiveWorkbenchWindow())).run();
			ResourcesPlugin.getWorkspace().removeResourceChangeListener(listener);
			project = listener.getProject();
		} else {
			project = getProjectHandle();
		}
		return performOperation(project);
	}

	/**
	 * Perform the real stuff (in this case branching)
	 * @param project
	 * @return
	 */
	protected boolean performOperation(IProject project) {
		BranchAsProjectOperation branchOp = null;
		if(project == null) {
			IProjectDescription description = ResourcesPlugin.getWorkspace().newProjectDescription(getNewProjectPage().getProjectName());
			description.setLocation(getNewProjectPage().getLocationPath());
			branchOp = new BranchAsProjectOperation(this.getContainer(), getRemoteLocation(), description, getProjectPage().getRevision());
		} else {
			branchOp = new BranchAsProjectOperation(this.getContainer(), getRemoteLocation(), project, getProjectPage().getRevision());
		}
		try {
			branchOp.run();
		} catch (Exception e) {
			MessageDialog.openError(this.getShell(), "Error while executing branch operation", e.getMessage());
			EclipseBazaarUI.log(e.getMessage(), e);
			return false;
		}
		return true;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		setWindowTitle(UITexts.ImportWizard_wizardTitle);
		setNeedsProgressMonitor(true);
		selectBranchPage = new BranchWizardSelectionPage("selectBranch", "Source branch");
		projectPage = new BranchWizardProjectPage("projectPage", isCheckout());
		createRepoLocationPage = new BranchWizardBranchLocationPage("CreateRepoPage", "Source branch");
		setNewProjectPage(new WizardNewProjectCreationPage("newProjectPage"));
		getNewProjectPage().setTitle("Project");
		getNewProjectPage().setDescription("Create a new project resource");
		getNewProjectPage().setImageDescriptor(BzrWizardPage.getTitleImage());	
		
		if (!EclipseBazaarCore.getDefault().isClientAvailable()) {
			BazaarUtilities.configureExecutable();
		}
	}

	protected boolean isCheckout() {
		return false;
	}

	@Override
	public void addPages() {
		super.addPages();
		addPage(selectBranchPage);
		addPage(createRepoLocationPage);
		addPage(getProjectPage());
		addPage(getNewProjectPage());
	}
	
	@Override
	public IWizardPage getNextPage(IWizardPage page) {
		if(page != null) {
			if(page.getName().equals(selectBranchPage.getName())) {
				if(selectBranchPage.isCreateNewBranchLocation()) {
					return createRepoLocationPage;
				} else {
					setRemoteLocation(selectBranchPage.getLocation());
					getProjectPage().setLocation(getRemoteLocation());
					return getProjectPage();
				}
			} else if(page.getName().equals(getProjectPage().getName())) {
				if(getProjectPage().useNewProjectWizard()) {
					useNewProjectWizard = true;
					return null;
				} else {
					useNewProjectWizard = false;
					return getNewProjectPage();
				}
			} else if(page.getName().equals(createRepoLocationPage.getName())) {
				setRemoteLocation(createRepoLocationPage.getLocation());
				getProjectPage().setLocation(getRemoteLocation());
				return super.getNextPage(page);
			}
		}
		return super.getNextPage(page);
	}
	
	@Override
	public IWizardPage getPreviousPage(IWizardPage page) {
		if(page != null) {
			if(page.getName().equals(createRepoLocationPage.getName())) {
				return selectBranchPage;
			} else if(page.getName().equals(getProjectPage().getName())) {
				if(selectBranchPage.isCreateNewBranchLocation()) {
					return createRepoLocationPage;
				} 
			} else if(page.getName().equals(getNewProjectPage().getName())) {
				return getProjectPage();
			}
		}
		return super.getPreviousPage(page);
	}

	
	protected boolean isUseNewProjectWizard() {
		return useNewProjectWizard;
	}
	
	protected void setUseNewProjectWizard(boolean useNewProjectWizard) {
		this.useNewProjectWizard = useNewProjectWizard;
	}
	
	/**
	 * @param remoteLocation the remoteLocation to set
	 */
	protected void setRemoteLocation(IBzrBranch remoteLocation) {
		this.remoteLocation = remoteLocation;
	}

	/**
	 * @return the remoteLocation
	 */
	protected IBzrBranch getRemoteLocation() {
		return remoteLocation;
	}

	/**
	 * @param newProjectPage the newProjectPage to set
	 */
	protected void setNewProjectPage(WizardNewProjectCreationPage newProjectPage) {
		this.newProjectPage = newProjectPage;
	}

	/**
	 * @return the newProjectPage
	 */
	protected WizardNewProjectCreationPage getNewProjectPage() {
		return newProjectPage;
	}

	/**
	 * @param projectPage the projectPage to set
	 */
	protected void setProjectPage(BranchWizardProjectPage projectPage) {
		this.projectPage = projectPage;
	}

	/**
	 * @return the projectPage
	 */
	protected BranchWizardProjectPage getProjectPage() {
		return projectPage;
	}

	private static final class ProjectListener implements IResourceChangeListener {
		private IProject project = null;

		public void resourceChanged(IResourceChangeEvent event) {
			IResourceDelta[] deltas = event.getDelta().getAffectedChildren();
			for (int i = 0; i < deltas.length; i++) {
				IResource resource = deltas[i].getResource();
				if (resource instanceof IProject) {
					if (deltas[i].getKind() == IResourceDelta.ADDED) {
						project = (IProject) resource;
					} else if (deltas[i].getKind() == IResourceDelta.REMOVED && project == resource) {
						project = null;
					}
				}
			}
		}

		public IProject getProject() {
			return project;
		}
	}
}
