/*******************************************************************************
 * Copyright (c) 2006 Software Balm Consulting Inc.
 * 
 * This software is licensed under the zlib/libpng license.
 * 
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it freely,
 * subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not 
 *            claim that you wrote the original software. If you use this 
 *            software in a product, an acknowledgment in the product 
 *            documentation would be appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *            misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source distribution.
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.project;

import java.util.Set;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchLocationManager;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.wizards.BzrWizardPage;

/**
 * @basedOn Peter Hunnisett <peter_hge at softwarebalm dot com>
 * @author Guillermo Gonzalez <guillo.gonzo at gmail dot com>
 * This file implements a wizard page which will have the user choose a
 * repository location or opt to create one. The creation of a repository
 * location, if desired, is left to another wizard page.
 * 
 */
public class BranchWizardSelectionPage extends BzrWizardPage implements IWizardPage, ISelectionChangedListener {
	private Button useExistingBranchLocation;
	private Button createNewBranchLocation;
	private ListViewer branchList;

	public BranchWizardSelectionPage(String pageName, String title) {
		super(pageName);
		setTitle(title);
		setDescription(UITexts.Branch_Wizard_page_description_select_source_branch);
		setPageComplete(false);
	}

	public boolean canFlipToNextPage() {
		if(useExistingBranchLocation.getSelection()) { 
			return branchList.getSelection() != null && !branchList.getSelection().isEmpty();
		}
		return (createNewBranchLocation != null) && (createNewBranchLocation.getSelection());
	}

	public void createControl(Composite parent) {
		Composite outerContainer = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		outerContainer.setLayout(layout);
		outerContainer.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		createNewBranchLocation = new Button(outerContainer, SWT.RADIO);
		createNewBranchLocation.setText(UITexts.Branch_Wizard_page_init_new_branch);

		useExistingBranchLocation = new Button(outerContainer, SWT.RADIO);
		useExistingBranchLocation.setText(UITexts.Branch_Wizard_page_use_existing_branch);

		createNewBranchLocation.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (createNewBranchLocation.getSelection()) {
					setPageComplete(true);
					branchList.getControl().setEnabled(false);
				}				
			}
		});
		
		useExistingBranchLocation.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (useExistingBranchLocation.getSelection()) {
					boolean isPageComplete = false;
					if(branchList.getSelection() != null && !branchList.getSelection().isEmpty()) {
						isPageComplete = true;
					}
					setPageComplete(isPageComplete);
					branchList.getControl().setEnabled(true);
				}
			}
		});
		
		branchList = createBranchList(outerContainer);
		
		setControl(outerContainer);

		// This will trigger a setPageComplete call to set the next button
		// appropriately.
		createNewBranchLocation.setSelection(true);
		createNewBranchLocation.setFocus();
		useExistingBranchLocation.setSelection(false);
		branchList.getControl().setEnabled(false);
	}
	
	private ListViewer createBranchList(Composite parent) {
		ListViewer viewer = new ListViewer(parent);
		viewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				IBzrBranch branch = (IBzrBranch)element;
				return branch.getBranchLocation().toString();
			}
		});
		viewer.addSelectionChangedListener(this);
		viewer.setComparator(new ViewerComparator() {
			public int compare(Viewer viewer, Object e1, Object e2) {
				IBzrBranch b1 = (IBzrBranch)e1;
				IBzrBranch b2 = (IBzrBranch)e2;
				return b1.compareTo(b2);
			}
		});
		List list = viewer.getList();
		list.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Set<IBzrBranch> branchList = BranchLocationManager.getInstance().getAllBranches();
		for (IBzrBranch bzrBranch : branchList) {
			viewer.add(bzrBranch);
		}
		return viewer;
	}

	/*
	 * For the moment only handle the change selection in the branch list
	 * 
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		if(event.getSelection() != null) {
			setPageComplete(true);
		}
	}

	public boolean isCreateNewBranchLocation() {
		if(createNewBranchLocation != null) {
			return createNewBranchLocation.getSelection();
		}
		return false;
	}

	public IBzrBranch getLocation() {
		IStructuredSelection selection = (IStructuredSelection)branchList.getSelection();
		return (IBzrBranch)selection.getFirstElement();
	}

	@Override
	public void dispose() {
		if(useExistingBranchLocation != null) {
			useExistingBranchLocation.dispose();
		}
		if(createNewBranchLocation != null) {
			createNewBranchLocation.dispose();
		}
		if(branchList != null) {
			branchList.getList().dispose();
		}
		super.dispose();
	}
}
